/* global lqMobileModule, LQMobile, console */
lqMobileModule.factory('activeStaysEndpointService',
  [
    '$http',
    'serviceEndPoints',
    'lodash',
    'satelliteTrackingModelService',
    function (
      $http,
      serviceEndPoints,
      lodash,
      satelliteTrackingModelService
    ) {
      'use strict';

      var endpointService = {};

      endpointService.findStays = function (returnsNumber, activeOnly, token) {
        var serviceEndpoint = serviceEndPoints.getActiveStaysEndPoint(returnsNumber);
        if (typeof activeOnly === 'undefined' || activeOnly !== false) {
          activeOnly = true;
        }

        // overload response
        // serviceEndpoint = './test/findActiveStays_1ActiveCheckIn.json';

        return $http({
          method: 'get',
          url: serviceEndpoint,
          headers: {
            'Cache-Control': 'no-cache',
            'Content-Type': 'application/json',
            // active: activeOnly,
            LQ_AUTH_KEY: lodash.trim(token),
            apikey: serviceEndPoints.getApiKey()
          }
        }).then(
          // Success
          function (response) {
            return response.data;
          },
          // Error
          function (error) {
            console.error(JSON.stringify(error));
            throw error;
          }
        );
      };

      endpointService.checkInAction = function (token, innNumber, confirmationNumber, roomNumber) {
        var serviceURL = serviceEndPoints.getMobileCheckinAction(
          innNumber,
          confirmationNumber,
          roomNumber
        );

        console.log('attempting to checkin via: ' + serviceURL);
        return $http({
          method: 'post',
          url: serviceURL,
          data: {},
          headers: {
            'Cache-Control': 'no-cache',
            'Content-Type': 'application/json',
            LQ_AUTH_KEY: lodash.trim(token),
            apikey: serviceEndPoints.getApiKey()
          }
        }).then(
          // Success
          function (response) {
            satelliteTrackingModelService.mobileCheckinCompleted();
            console.log('checkin success');
            return response.data;
            // return response;
          },
          // Error
          function (error) {
            satelliteTrackingModelService.mobileCheckinUnsuccessful();
            console.error(JSON.stringify(error));
            throw error;
          }
        );
      };


      endpointService.getRooms = function (innNumber, startDate, endDate, roomCode, token) {
        // var token = window.sessionStorage.getItem('token');
        var serviceURL = serviceEndPoints.getMobileCheckinGetRooms(innNumber);

        // temp override endpoint
        // serviceURL = './test/roomResponse.json';

        return $http({
          method: 'get',
          url: serviceURL,
          params: {
            startDate: LQMobile.Util.dateToString(startDate, '/'),
            endDate: LQMobile.Util.dateToString(endDate, '/'),
            roomCode: roomCode
          },
          headers: {
            'Cache-Control': 'no-cache',
            'Content-Type': 'application/json',
            LQ_AUTH_KEY: lodash.trim(token),
            apikey: serviceEndPoints.getApiKey()
          }
        }).then(
          // Success
          function (response) {
            return response.data;
            // return response;
          },
          // Error
          function (error) {
            console.error(JSON.stringify(error));
            throw error;
          }
        );
      };

      endpointService.getHotelDetails = function (token, innNumber) {
        var serviceURL = serviceEndPoints.getMobileCheckinHotelDetails();

        return $http({
          method: 'get',
          url: serviceURL,
          params: {
            id: innNumber,
            apikey: serviceEndPoints.getApiKey()
          },
          headers: {
            'Cache-Control': 'no-cache',
            'Content-Type': 'application/json'
          }
        }).then(
          // Success
          function (response) {
            console.log('hotel details: ' + response.data);
            return response.data[0];
          },
          // Error
          function (error) {
            console.error(JSON.stringify(error));
            throw error;
          }
        );
      };

      return endpointService;
    }
  ]
);
