/* global lqMobileModule, CQ*/
/* jshint maxparams: 11 */
lqMobileModule.factory(
  'activeStaysModel',
  [
    '$q',
    '$timeout',
    'activeStaysEndpointService',
    'mobileCheckoutEndpointService',
    'mobileCheckoutModel',
    function (
      $q,
      $timeout,
      activeStaysEndpointService,
      mobileCheckoutEndpointService,
      mobileCheckoutModel
    ) {
      'use strict';

      var dataModel = {
        activeStay: undefined,
        availableRooms: []
      };
      var monthNames = ['Jan', 'Feb', 'Mar', 'Apr', 'May', 'Jun', 'Jul', 'Aug', 'Sep', 'Oct', 'Nov', 'Dec'];
      var STAY_ACTIVE = 'ACTIVE';
      var STAY_CHECK_IN = 'CHECK_IN';
      var STAY_CHECK_IN_SOON = 'CHECK_IN_SOON';
      var STAY_CHECK_OUT = 'CHECK_OUT';


      function getRooms() {
        var defer = $q.defer();
        activeStaysEndpointService.getRooms(
          dataModel.activeStay.hotel.number,
          dataModel.activeStay.startDate,
          dataModel.activeStay.endDate,
          dataModel.activeStay.room.code,
          window.sessionStorage.getItem('token')
        ).then(
          function (roomData) {
            if (Array.isArray(roomData) && roomData.length > 1) {
              roomData = [roomData.shift()];
            }
            if (Array.isArray(roomData) && roomData.length) {
              dataModel.availableRooms =
                roomData.reduce(function (roomsAvail, dateObject) {
                  dateObject.rooms.filter(
                    function (room) {
                      // for now every room is eligible until Tres
                      // figures out how these endpoints are suppose
                      // to work
                      return true;
                      // var isAvailable = room.available === true;
                      // var isOccupied = room.occupied === true;
                      // return isAvailable
                      //   && !isOccupied
                      //   && room.roomType.roomCode.trim() === dataModel.activeStay.room.code;
                    }
                  ).forEach(function (room) {
                    if (roomsAvail.find(function (rm) {
                      return rm.roomNumber === room.roomNumber;
                    }) === undefined) {
                      roomsAvail.push({
                        roomNumber: room.roomNumber,
                        floor: room.floor
                      });
                    }
                  });

                  return roomsAvail;
                }, []);
            } else {
              dataModel.availableRooms = [];
            }
            defer.resolve();
          },
          function () {
            dataModel.availableRooms = [];
            defer.resolve();
          }
        );
        return defer.promise;
      }

      function validateCheckInStay(activeStay) {
        var stayIsEligible = activeStay.stayEligibleActivity.usedEligibility === false;
        var isCheckIn = activeStay.stayEligibleActivity.eligibleActivity === STAY_CHECK_IN;
        // set times based on response
        var startDate = new Date(activeStay.stayEligibleActivity.startTime);
        var endDate = new Date(activeStay.stayEligibleActivity.endTime);
        var deviceTime = Date.now();
        var isPreArrival = activeStay.stayStatus === 'PRE_ARRIVAL';

        var validCheckInTime = startDate.getTime() <= deviceTime
          && deviceTime <= endDate.getTime();

        return isCheckIn && stayIsEligible && validCheckInTime && isPreArrival;
      }

      function validateCheckOutStay(activeStay) {
        var deviceHour = (new Date()).getHours();
        var isPreDepart = activeStay.stayStatus === 'PRE_DEPARTURE';
        // deviceHour = 11;
        return deviceHour < 12 && isPreDepart;
      }

      function validateActiveStay(activeStay) {
        var timezone = activeStay.hotelInfo.hotelTimeZone;

        var arrivalDate = moment.tz(activeStay.arrivalDate, timezone).startOf('day').toDate();
        var departureDate = moment.tz(activeStay.departureDate, timezone).endOf('day').toDate();

        var now = new Date();
        var validStayTime;
        var isValidStatus = activeStay.stayStatus === 'ON_PROPERTY' || activeStay.stayStatus === 'PRE_DEPARTURE';

        validStayTime = arrivalDate.getTime() <= now.getTime()
          && now.getTime() <= departureDate.getTime();

        return validStayTime && isValidStatus
          && !validateCheckInStay(activeStay);
      }

      function validateCheckInSoon(activeStay) {
        var startDate = new Date(activeStay.stayEligibleActivity.startTime);
        var deviceTime = Date.now();

        var stayIsEligible = activeStay.stayEligibleActivity.usedEligibility === false;
        var validStayTime = deviceTime < startDate.getTime();
        var isPreArrival = activeStay.stayStatus === 'PRE_ARRIVAL';

        return stayIsEligible && validStayTime && isPreArrival;
      }

      function buildFromStayGeneric(stay, stayType) {
        var hotelTitle = stay.hotelInfo.hotelName.toLowerCase().indexOf('la quinta inn & suites') === -1 ? 'La Quinta Inn' : 'La Quinta Inn & Suites';
        var hotelFullName = stay.hotelInfo.hotelName.substr(hotelTitle.length + 1);
        var startDate = (moment(stay.arrivalDate)).toDate();
        var endDate = (moment(stay.departureDate)).toDate();

        // get address parts
        var street1 = stay.hotelInfo.address ? stay.hotelInfo.address.street : '';
        var street2 = stay.hotelInfo.address ? stay.hotelInfo.address.street2 : '';
        var city = stay.hotelInfo.address ? stay.hotelInfo.address.city : '';
        var state = stay.hotelInfo.address ? stay.hotelInfo.address.stateProvince : '';
        var zip = stay.hotelInfo.address ? stay.hotelInfo.address.postalCode : '';

        dataModel.activeStay = {
          hotel: {
            name: hotelFullName,
            title: hotelTitle,
            number: stay.hotelInfo.innNumber.id,
            phone: stay.hotelInfo.phone.areaCode + '-' + stay.hotelInfo.phone.number,
            address: stay.hotelInfo.address ? street1 + ' ' + street2 + ' ' + city + ', ' + state + ' ' + zip : ''
          },
          confirmationNumber: stay.confirmationNumber.number,
          status: stay.stayStatus,
          type: stayType,
          room: {
            number: stay.roomNumber,
            code: typeof stay.roomType.roomCode === 'string' ? stay.roomType.roomCode.trim() : '',
            description: stay.roomType.roomDescription
          },
          startDate: startDate,
          startDateDisplay: monthNames[startDate.getMonth()] + ' ' + startDate.getDate(),
          endDate: endDate,
          endDateDisplay: monthNames[endDate.getMonth()] + ' ' + endDate.getDate(),
          availableRooms: []
        };
      }

      function buildFromCheckout(activeStayData) {
        // build stay data
        buildFromStayGeneric(activeStayData, STAY_CHECK_OUT);
      }

      function buildFromCheckIn(checkIn) {
        buildFromStayGeneric(checkIn, STAY_CHECK_IN);
      }

      function buildFromActiveStay(activeStay) {
        buildFromStayGeneric(activeStay, STAY_ACTIVE);
      }

      function buildFromCheckInSoon(checkInSoon) {
        buildFromStayGeneric(checkInSoon, STAY_CHECK_IN_SOON);
      }

      /**
       * Promise resolves to the checkout data, or rejects
       *
       * @returns {Promise}
       */
      function fetchCheckoutResponse() {
        var defer = $q.defer();

        if (!mobileCheckoutModel.suppressEligibilityCheck()) {
          mobileCheckoutModel.checkEligibility().then(
            //
            // Success
            function (eligibilityResponse) {
              // check if eligible
              if (mobileCheckoutModel.isEligible()) {
                defer.resolve(eligibilityResponse);
              } else {
                defer.reject();
              }
            },
            function () {
              defer.reject();
            }
          );
        } else {
          defer.reject();
        }

        return defer.promise;
      }

      function isCheckoutStay(allStays) {
        var defer = $q.defer();
        var i;
        var iMax;
        var possibleCheckOutStays = [];

        // check for eligibility on user
        fetchCheckoutResponse().then(
          function (data) {
            // filter for check out stays
            for (i = 0, iMax = allStays.length; i < iMax; i += 1) {
              if (validateCheckOutStay(allStays[i])) possibleCheckOutStays.push(allStays[i]);
            }

            defer.resolve({
              possibleCheckOutStays: possibleCheckOutStays,
              eligibility: data
            });
          },
          function () {
            defer.reject();
          }
        );

        return defer.promise;
      }

      dataModel.hasActiveStay = function () {
        return typeof dataModel.activeStay !== 'undefined';
      };

      dataModel.isActiveStay = function () {
        if (dataModel.hasActiveStay() && (!dataModel.inTimeout() || (dataModel.inTimeout() && dataModel.isShortTimeout))) {
          return dataModel.activeStay.type === STAY_ACTIVE;
        }
        return false;
      };

      dataModel.isActiveCheckOut = function () {
        if (dataModel.hasActiveStay() && (!dataModel.inTimeout() || (dataModel.inTimeout() && dataModel.isShortTimeout))) {
          return dataModel.activeStay.type === STAY_CHECK_OUT;
        }
        return false;
      };

      dataModel.isActiveCheckIn = function () {
        if (dataModel.hasActiveStay() && (!dataModel.inTimeout() || (dataModel.inTimeout() && dataModel.isShortTimeout))) {
          return dataModel.activeStay.type === STAY_CHECK_IN;
        }
        return false;
      };

      dataModel.isCheckInSoon = function () {
        if (dataModel.hasActiveStay() && (!dataModel.inTimeout() || (dataModel.inTimeout() && dataModel.isShortTimeout))) {
          return dataModel.activeStay.type === STAY_CHECK_IN_SOON;
        }
        return false;
      };

      dataModel.isCheckedIn = function () {
        if (dataModel.hasActiveStay() && (!dataModel.inTimeout() || (dataModel.inTimeout() && dataModel.isShortTimeout))) {
          return dataModel.activeStay.status === 'PRE_DEPARTURE' || dataModel.activeStay.status === 'ON_PROPERTY';
        }
        return false;
      };

      dataModel.getDateText = function () {
        if (dataModel.activeStay) return dataModel.isActiveCheckOut() ? 'Today, ' + dataModel.activeStay.endDateDisplay : dataModel.activeStay.startDateDisplay + ' – ' + dataModel.activeStay.endDateDisplay;
        return '';
      };

      dataModel.getDateSuperText = function () {
        if (dataModel.activeStay) return dataModel.isActiveCheckOut() ? 'Checkout' : 'Staying';
        return '';
      };

      dataModel.getRoomDescription = function () {
        if (dataModel.activeStay) return dataModel.activeStay.room.description;
        return '';
      };

      dataModel.getRoomNumber = function () {
        if (dataModel.activeStay) return dataModel.activeStay.room.number;
        return null;
      };

      dataModel.setRoomNumber = function (roomNumber) {
        if (dataModel.activeStay) dataModel.activeStay.room.number = roomNumber;
      };

      dataModel.getPhoneNumber = function () {
        if (dataModel.activeStay) return dataModel.activeStay.hotel.phone;
        return null;
      };

      dataModel.getTimeout = function () {
        if (dataModel.timeout) return dataModel.timeout;
        return 0;
      };

      /**
       *
       * @param shortTimeout {boolean} - set timeout to short value
       */
      dataModel.setTimeout = function (shortTimeout) {
        var timeoutMilliseconds = shortTimeout ? lqmobileConfigs.activeStaysTimeoutShort : lqmobileConfigs.activeStaysTimeout;
        dataModel.isShortTimeout = shortTimeout;
        dataModel.timeout = Date.now() + timeoutMilliseconds;
      };

      dataModel.clearTimeout = function () {
        dataModel.isShortTimeout = null;
        dataModel.timeout = null;
      };

      dataModel.inTimeout = function () {
        return Date.now() <= dataModel.getTimeout();
      };

      /**
       * Requests and partially parses active stay data
       * @param {String} returnsNumber - user's returns number
       * @returns {Function}
       */
      dataModel.getActiveStay = function (returnsNumber) {
        var defer = $q.defer();
        var authToken = window.sessionStorage.getItem('token');

        if (!dataModel.inTimeout()) {
          activeStaysEndpointService.findStays(returnsNumber, true, authToken).then(
            function (response) {
              var isValidResponse = Array.isArray(response);
              // normalize eligibleStayActivity to an object:

              if (!isValidResponse) {
                defer.reject();
              } else {
                // this hacks the stayEligibleActivity back to an
                // object, since it wasn't contracted to be an array
                // On 2017-08-22: Tres said the only eligibility we should
                // be looking at is CHECK_IN ; everything else can be ignore
                // so i'm reverting the array to an object of only check_in
                // or some face eligibility that can effectively be ignored
                for (var i = 0, iMax = response.length; i < iMax; i += 1) {
                  if (
                    response[i].hasOwnProperty('stayEligibleActivity')
                    && Array.isArray(response[i].stayEligibleActivity)
                  ) {
                    // rip it down to just a check in one
                    response[i].stayEligibleActivity = response[i].stayEligibleActivity.filter(
                      function (eligibility) {
                        return eligibility.eligibleActivity.trim() === 'CHECK_IN';
                      }
                    );

                    if (response[i].stayEligibleActivity.length === 1) {
                      response[i].stayEligibleActivity = response[i].stayEligibleActivity[0];
                    } else {
                      response[i].stayEligibleActivity = {
                        startTime: '2008-09-15T15:53:00+00:00',
                        endTime: '2008-09-15T15:53:00+00:00',
                        timeEligibilityUsed: '2008-09-15T15:53:00+00:00',
                        usedEligibility: true,
                        eligibleActivity: 'FAKE'
                      };
                    }
                  } else if (
                    response[i].hasOwnProperty('stayEligibleActivity')
                    && Object.prototype.toString.call(response[i].stayEligibleActivity) !== '[object Object]'
                  ) {
                    // if it's not an object... then someone sent us bad data,
                    // let's just add or fake one
                    response[i].stayEligibleActivity = {
                      startTime: '2008-09-15T15:53:00+00:00',
                      endTime: '2008-09-15T15:53:00+00:00',
                      timeEligibilityUsed: '2008-09-15T15:53:00+00:00',
                      usedEligibility: true,
                      eligibleActivity: 'FAKE'
                    };
                  } else if (!response[i].hasOwnProperty('stayEligibleActivity')) {
                    // if it doesn't have one at all, that'll cause errors to, add a fake bad one
                    response[i].stayEligibleActivity = {
                      startTime: '2008-09-15T15:53:00+00:00',
                      endTime: '2008-09-15T15:53:00+00:00',
                      timeEligibilityUsed: '2008-09-15T15:53:00+00:00',
                      usedEligibility: true,
                      eligibleActivity: 'FAKE'
                    };
                  }
                }
                dataModel.buildActiveStay(response).then(
                  function () {
                    if (dataModel.hasActiveStay()) {
                      // set the short timeout after a successful call
                      dataModel.setTimeout(true);

                      defer.resolve(dataModel);
                    } else {
                      defer.reject();
                    }
                  },
                  function () {
                    defer.reject();
                  }
                );
              }
            },
            function (error) {
              console.log(error);
              defer.reject();
            }
          );
        } else if (dataModel.isShortTimeout) {
          $timeout(function () {
            defer.resolve();
          }, 500);
        } else {
          defer.reject();
        }

        return defer.promise;
      };

      dataModel.buildActiveStay = function (activeStayResponse) {
        var defer = $q.defer();
        var i;
        var iMax;
        var checkOutStay;
        var checkInStays = [];
        var activeStays = [];
        var checkInSoonStays = [];

        activeStayResponse = activeStayResponse.sort(function (a, b) {
          return a.hotelInfo.hotelName.toLowerCase().localeCompare(
            b.hotelInfo.hotelName.toLowerCase()
          );
        });

        // is check out stay
        isCheckoutStay(activeStayResponse).then(
          function (data) {
            // get correct stay
            for (i = 0, iMax = data.possibleCheckOutStays.length; i < iMax; i += 1) {
              if (data.eligibility.confirmationNumber.number == data.possibleCheckOutStays[i].confirmationNumber.number) checkOutStay = data.possibleCheckOutStays[i];
            }

            // temp check for confirmation number comparison
            if (checkOutStay) {
              // populate active stay model
              // with information from checkout-endpoint
              // this is temporary until the stays endpoint
              // returns correct information about checkouts
              buildFromCheckout(checkOutStay);
              defer.resolve();
            } else {
              // eligible for checkout but doesn't match any stay or there are none
              // get hotel info
              activeStaysEndpointService.getHotelDetails(
                window.sessionStorage.getItem('token'),
                data.eligibility.innNumber.id
              ).then(
                function (hotelInfo) {
                  // build out fake stay data...
                  hotelInfo.hotelName = hotelInfo.name;
                  hotelInfo.phone = hotelInfo.primaryPhoneNumber;
                  buildFromCheckout({
                    hotelInfo: hotelInfo,
                    arrivalDate: '',
                    departureDate: new Date(),
                    confirmationNumber: data.eligibility.confirmationNumber,
                    stayStatus: 'PRE_DEPARTURE',
                    roomNumber: '',
                    roomType: {}
                  });

                  defer.resolve();
                },
                function () {
                  // that hotel doesn't exist?
                  defer.reject();
                }
              );
            }
          },
          function () {
            // filter for rest of stays
            for (i = 0, iMax = activeStayResponse.length; i < iMax; i += 1) {
              if (validateCheckInStay(activeStayResponse[i])) {
                checkInStays.push(activeStayResponse[i]);
              } else if (validateCheckInSoon(activeStayResponse[i])) {
                checkInSoonStays.push(activeStayResponse[i]);
              } else if (validateActiveStay(activeStayResponse[i])) {
                activeStays.push(activeStayResponse[i]);
              }
            }

            // if it didn't resolve, move on to the
            // next possible stay types
            if (activeStays.length >= 1) {
              buildFromActiveStay(activeStays[0]);
              defer.resolve();
            } else if (checkInStays.length === 1) {
              buildFromCheckIn(checkInStays[0]);
              getRooms().then(defer.resolve);
            } else if (checkInSoonStays.length === 1) {
              buildFromCheckInSoon(checkInSoonStays[0]);
              defer.resolve();
            } else {
              defer.reject();
            }
          });

        return defer.promise;
      };

      dataModel.processCheckin = function (roomNumber) {
        //
        // Get Auth token
        var token = window.sessionStorage.getItem('token');

        console.log('I Want To Checkin From Confirmation #: ' + dataModel.activeStay.confirmationNumber);
        /** @var {activeStaysEndpointService} activeStaysEndpointService */

        //
        // Send Request
        return activeStaysEndpointService.checkInAction(
          token,
          dataModel.activeStay.hotel.number,
          dataModel.activeStay.confirmationNumber,
          roomNumber
        ).then(
          // Success
          function (response) {
            console.log('Active Stays: Checkin Complete' + JSON.stringify(response));

            dataModel.clearTimeout();

            return response;
          },
          // Fail
          function (error) {
            console.log('There is an error calling the checkin service');

            // set check again timeout
            dataModel.setTimeout(false);

            throw 'Checkin Error: ' + error.data;
          }
        );
      };

      /**
      * processCheckout
      *
      * Process this Checkout by forming and sending a request to LQ service.
      */
      dataModel.processCheckout = function () {
        //
        // Get Auth token
        var token = window.sessionStorage.getItem('token');

        console.log('I Want To Checkout From Confirmation #: ' + dataModel.activeStay.confirmationNumber);
        /** @var {mobileCheckoutEndpointService} mobileCheckoutEndpointService */

        //
        // Send Request
        return mobileCheckoutEndpointService.checkoutAction(
          token, dataModel.activeStay.confirmationNumber
        ).then(
          // Success
          function (response) {
            console.log('Active Stays: checkout complete' + JSON.stringify(response));

            dataModel.clearTimeout();

            return response;
          },
          // Fail
          function (error) {
            console.log('There is an error calling the checkout service');

            // set check again timeout
            dataModel.setTimeout(false);

            throw 'Checkout Error: ' + error.data;
          }
        );
      };

      dataModel.reset = function () {
        dataModel.activeStay = undefined;
        dataModel.clearTimeout();
      };

      return dataModel;
    }
  ]);
