/* global lqMobileModule, LQMobile, console*/
/* jshint maxparams: 10 */
lqMobileModule.factory('bookingEndpointService',
  [
    '$window',
    '$q',
    '$http',
    '$filter',
    'serviceEndPoints',
    'ReservationRequest',
    'Reservation',
    'ConvertHold',
    'paymentTypes',
    function (
      $window,
      $q,
      $http,
      $filter,
      serviceEndPoints,
      ReservationRequest,
      Reservation,
      ConvertHold,
      paymentTypes
    ) {
      'use strict';

      var endpointService = {};


      function getMainContactInfo(bookingData) {
        var mainContact;

        if (typeof bookingData.emailAddress === 'string') {
          bookingData.emailAddress = bookingData.emailAddress.trim();
        }
        if (typeof bookingData.firstName === 'string') {
          bookingData.firstName = bookingData.firstName.trim();
        }
        if (typeof bookingData.lastName === 'string') {
          bookingData.lastName = bookingData.lastName.trim();
        }

        mainContact = {
          emailAddress: [{
            email: bookingData.emailAddress,
            emailDesignation: 'PERSONAL',
            primary: true,
            notifications: []
          }],
          firstName: bookingData.firstName,
          lastName: bookingData.lastName,
          phoneNumbers: [{
            phoneDesignation: 'HOME',
            phoneNumber: {
              country: {
                code: 'US',
                description: 'United States'
              },
              number: bookingData.mobilePhone
            },
            primary: false
          }, {
            phoneDesignation: 'MOBILE',
            phoneNumber: {
              country: {
                code: 'US',
                description: 'United States'
              },
              number: bookingData.mobilePhone
            },
            primary: true
          }],
          postalAddress: {
            city: bookingData.city,
            countryCode: bookingData.country,
            countryDisplay: '',
            county: '',
            postalCode: bookingData.zipCode,
            stateProvince: bookingData.state,
            street: bookingData.street,
            street2: bookingData.street2
          }
        };
        return mainContact;
      }

      function getPaymentTypeInfo(bookingData, searchData, returnsData, tkn) {
        var paymentInfoArray = [];
        var returnsPaymentType;
        var ccType;
        var ccPaymentType;
        var dbPaymentType;

        if (bookingData.paymentType === paymentTypes.CREDITCARD) {
          ccType = bookingData.creditCard.ccType;
          if (typeof bookingData.firstName === 'string') {
            bookingData.firstName = bookingData.firstName.trim();
          }
          if (typeof bookingData.lastName === 'string') {
            bookingData.lastName = bookingData.lastName.trim();
          }

          ccPaymentType = {
            type: 'CreditCard',
            creditCard: {
              creditCardType: bookingData.creditCard.ccNumber.indexOf('*') > -1 ? ccType : LQMobile.Util.findCCType(bookingData.creditCard.ccNumber),
              nameOnCard: bookingData.firstName + ' ' + bookingData.lastName,
              expirationDate: bookingData.creditCard.ccYear + '-'
                + (parseInt(bookingData.creditCard.ccMonth, 10) + 1),
              creditCardLastFour: bookingData.creditCard.ccNumber
                .substr(bookingData.creditCard.ccNumber.length - 4),
              cardNumber: tkn === undefined ? bookingData.creditCard.ccNumber : tkn
            }
          };
          paymentInfoArray.push(ccPaymentType);
        } else if (bookingData.paymentType === paymentTypes.DIRECTBILL) {
          dbPaymentType = {
            type: 'DirectBill',
            directBillNumber: bookingData.directBill.Number
          };
          paymentInfoArray.push(dbPaymentType);
        }

        if (searchData.isBookWithPointsEnabled) {
          returnsPaymentType = {
            type: 'FreeNightCertificate',
            certificateNumber: {
              creationDate: $filter('date')(new Date(), 'yyyy-MM-dd'),
              number: '1234567890',
              points: searchData.selectedRoom.computedRates.totalPoints,
              returnsNumber: {
                id: returnsData.userProfile.returnsNumber
              }
            }
          };
          paymentInfoArray.push(returnsPaymentType);
        }
        return paymentInfoArray;
      }

      function getSpecialRequests() {
        var specialRequests = [];

        return specialRequests;
      }

      endpointService.completeBooking =
        function (
          searchData, hotelData, bookingData, isUserLoggedIn,
          returnsData, reservation, tkn
        ) {
          var deferred = $q.defer();
          var mainContact = getMainContactInfo(bookingData);
          var paymentInfoArray = getPaymentTypeInfo(bookingData, searchData, returnsData, tkn);
          var specialRequests = getSpecialRequests(bookingData);
          var reservationRequest;
          var serviceURL;
          var method;

          if (isUserLoggedIn) {
            mainContact.returnsNumber = {
              id: returnsData.userProfile.returnsNumber
            };
          }

          method = '';
          if (reservation) {
            if (reservation.status === 'ONHOLD') {
              reservationRequest = ConvertHold.buildConvertHold(
                searchData, mainContact, paymentInfoArray, hotelData, reservation
              );
            } else {
              reservationRequest = Reservation.buildReservationRequest(
                searchData, mainContact, paymentInfoArray, hotelData,
                specialRequests, reservation.confirmationNumber
              );
            }
            method = 'put';
          } else {
            reservationRequest = ReservationRequest.buildReservationRequest(
              searchData, mainContact, paymentInfoArray, hotelData, specialRequests
            );
            method = 'post';
          }

          serviceURL = serviceEndPoints.getBookingConfirmationEndPoint();

          $http({
            method: method,
            url: serviceURL,
            data: reservationRequest
          }).success(function (response) {
            deferred.resolve(response);
          }).error(function (data) {
            deferred.reject(data);
          });
          return deferred.promise;
        };

      endpointService.postReadyForYou = function (phoneNumber, bookingConfirmation, arrivalTime) {
        var deferred = $q.defer();
        var serviceURL = serviceEndPoints.getReadyForYouEndPoint();

        $http({
          method: 'post',
          url: serviceURL,
          headers: {
            'Content-Type': 'text/plain;charset=UTF-8'
          },
          data: {},
          params: {
            phones: phoneNumber,
            confirmationNumber: bookingConfirmation,
            arrivalTimeCodeName: arrivalTime
          }
        }).success(function (response) {
          deferred.resolve(response);
        }).error(function (data) {
          deferred.reject(data);
        });

        return deferred.promise;
      };

      function loadAddToCalendarAPI() {
        var loadAddToCalendarScriptURL = lqmobileConfigs.httpBase + '/etc/designs/lq-mobile-app-wrapper2/footerlibs/js/thirdparty/atc.min.js';
        $.getScript(loadAddToCalendarScriptURL, function () {
          endpointService.deferredloadAddToCalendarAPI.resolve();
        });
      }

      endpointService.isAddToCalendarAPILoaded = function () {
        if (endpointService.deferredloadAddToCalendarAPI === undefined) {
          return false;
        }
        return true;
      };

      endpointService.loadAddToCalendarAPI = function () {
        if (endpointService.deferredloadAddToCalendarAPI === undefined) {
          endpointService.deferredloadAddToCalendarAPI = $q.defer();
          loadAddToCalendarAPI();
        }
        return endpointService.deferredloadAddToCalendarAPI.promise;
      };


      return endpointService;
    }]);
