/* global lqMobileModule, lqMobile, $*/
/* jshint maxstatements: 50 */
/* jshint maxparams: 13 */
lqMobileModule.controller('bookingGuaranteeController', ['$scope',
  'hotelInformationModel',
  'searchParamModelService',
  'bookingModelService',
  'routingService',
  'paymentTypes',
  '$locale',
  '$rootScope',
  'events',
  '$location',
  '$timeout',
  'satelliteTrackingModelService',
  'dateFormatterService',
  '$element',
  function ($scope,
    hotelInformationModel,
    searchParamModelService,
    bookingModelService,
    routingService,
    paymentTypes,
    $locale,
    $rootScope,
    events,
    $location,
    $timeout,
    satelliteTrackingModelService,
    dateFormatterService,
    $element
  ) {
    'use strict';

    var i;

    $scope.bookingForm = bookingModelService.data;
    $scope.searchInfo = searchParamModelService.data;
    $scope.hotelInfo = hotelInformationModel.data.hotelDetails;
    $scope.cancellationPolicy = CANCELLATION_POLICY;

    // used to prevent back button of the browser.
    if ($scope.bookingForm.confirmationNumber !== '') {
      routingService.goToHomePage();
    } else if ($scope.bookingForm.paymentType !== '') {
      $scope.paymentSelected = true;
      $location.search('paymentSelected', true);
    } else {
      $scope.paymentSelected = false;
    }


    $scope.showDirectBill = false;
    $scope.init = function (disabledHotelsForDirectBill) {
    // always make sure element is shown
      $($element).show();

      if (isEditMode) {
        $scope.showDirectBill = true;
      } else {
        $scope.showDirectBill = disabledHotelsForDirectBill
            .indexOf($scope.hotelInfo.hotelId) === -1;
      }
      if (!$scope.showDirectBill) {
        $scope.selectPaymentOption(paymentTypes.CREDITCARD);
      }
    };

    $scope.clearPaymentType = function () {
      $timeout(function () {
        bookingModelService.resetPaymentData();
        $location.search('paymentSelected', null);
        $scope.paymentSelected = false;
      }, 5);
    };

    $scope.creditCardExpirationError = false;

    $scope.minCCMonth = lqMobile.Defaults.getDefaultCCMonth();
    $scope.minCCYear = lqMobile.Defaults.getDefaultCCYear();

    $scope.servicesError = '';

    $scope.months = $locale.DATETIME_FORMATS.MONTH;
    $scope.years = [];

  // fix for making the month and year dropdown work in all phones
    $scope.ccMonth = {
      isopen: false
    };
    $scope.ccYear = {
      isopen: false
    };

    for (i = 0; i < 12; i += 1) {
      $scope.years.push($scope.minCCYear + i);
    }

    function handleError(error) {
      if (error && error.length > 0) {
        $scope.servicesError = error.trim();
      }
      $scope.ccForm.submitDisabled = false;
      $scope.loadingModal.close();
    }

    function forwardToConfirmation() {
      var oneDay = 1000 * 60 * 60 * 24;
      var now = new Date();
      var checkInTime = searchParamModelService.data.checkinDate.getTime();
      var checkoutTime = searchParamModelService.data.checkoutDate.getTime();
      var checkInCheckOutDiffMs = checkoutTime - checkInTime;
      var nowCheckInDiffMs = checkInTime - now.getTime();
      var dateDiffInDays = Math.round(checkInCheckOutDiffMs / oneDay);
      var daysUntilCheckIn = Math.round(nowCheckInDiffMs / oneDay);
      var daysToArrival = daysUntilCheckIn;
      var numberOfNights = dateDiffInDays;
      var checkinDate = searchParamModelService.data.checkinDate;
      var checkoutDate = searchParamModelService.data.checkoutDate;
      var numberOfAdults = searchParamModelService.data.numberOfAdults;
      var promoCode = searchParamModelService.data.promoCode;
      var totalRate = searchParamModelService.data.selectedRoom.hotelRoomRate.totalRate.amount;
      var reservationRevenue =
        searchParamModelService.data.selectedRoom.hotelRoomRate.totalRateWithTaxes.amount;
      var totalTaxes = reservationRevenue - totalRate;
      var roomTypeName =
        searchParamModelService.data.selectedRoom.hotelRoomRate.roomType.roomDescription;
      var guestPostalCode = bookingModelService.data.zipCode;
      var roomBookingType = 'mobile';
      var confirmationNum = bookingModelService.data.confirmationNumber;
      var isFreeNight = false;
      var currencyCode =
        typeof searchParamModelService.data.selectedRoom.hotelRoomRate.totalRate.currencyCode !== 'undefined' ? searchParamModelService.data.selectedRoom.hotelRoomRate.totalRate.currencyCode : 'USD';

      // always make sure element is hidden
      $($element).hide();
      $location.search('paymentSelected', null);
      routingService.goToBookingConfirmation();
      $scope.loadingModal.close();


      if (daysUntilCheckIn < 1) {
        daysToArrival = 0;
      }

      satelliteTrackingModelService.standardConfirmationLoaded(
         numberOfNights,
        totalRate,
        totalTaxes,
        currencyCode,
        dateFormatterService.getFormattedDate(checkinDate),
        dateFormatterService.getFormattedDate(checkoutDate),
        numberOfAdults,
        daysToArrival,
        roomTypeName,
        guestPostalCode,
        promoCode,
        roomBookingType,
        confirmationNum,
        isFreeNight,
        reservationRevenue
    );
    // $timeout(function() {
      // $rootScope.$broadcast(events.standardConfirmationLoaded);

    // }, 500);
    }

    $scope.selectYear = function (year) {
      bookingModelService.data.creditCard.ccYear = year;
      $scope.ccYear.isopen = !$scope.ccYear.isopen;
    };

    $scope.selectMonth = function (month) {
      bookingModelService.data.creditCard.ccMonth = month;
      $scope.ccMonth.isopen = !$scope.ccMonth.isopen;
    };

    $scope.selectPaymentOption = function (selectedPaymentOption) {
      $scope.setPaymentOption(selectedPaymentOption);
      $location.search('paymentSelected', true);
      $scope.paymentSelected = true;
    };

    $scope.setPaymentOption = function (selectedPaymentOption) {
      if (selectedPaymentOption === 'creditcard') {
        $scope.bookingForm.paymentType = paymentTypes.CREDITCARD;
        satelliteTrackingModelService.creditCardOptionSelected($scope.hotelInfo.hotelId);
      } else if (selectedPaymentOption === 'directbill') {
        $scope.bookingForm.paymentType = paymentTypes.DIRECTBILL;
        satelliteTrackingModelService.directBillOptionSelected($scope.hotelInfo.hotelId);
      }
    };

    $scope.dbForm = {};
    $scope.dbForm.formSubmitted = false;
    $scope.dbForm.submitForm = function (isdbNumberValid) {
      $scope.dbForm.formSubmitted = true;
      $scope.dbForm.submitDisabled = true;
      if (isdbNumberValid) {
        $scope.servicesError = '';
        bookingModelService.persistData();
        $scope.dbForm.submitDisabled = false;
        $scope.loadingModal = $scope.showLoadingModal();
        bookingModelService.completeBooking().then(forwardToConfirmation, handleError);
      } else {
        $scope.dbForm.submitDisabled = false;
      }
    };

    $scope.ccForm = {};
    $scope.ccForm.formSubmitted = false;
    $scope.ccForm.submitForm = function (
      isCCNumberValid, isCCLabelValid, isCCMonthValid, isCCYearValid
    ) {
      $scope.ccForm.formSubmitted = true;
      $scope.ccForm.submitDisabled = true;

      if ((isCCNumberValid && (isCCMonthValid || isCCYearValid))) {
        $scope.servicesError = '';
        bookingModelService.persistData();
        $scope.ccForm.submitDisabled = false;
        $scope.loadingModal = $scope.showLoadingModal();
        bookingModelService.completeBooking().then(forwardToConfirmation, handleError);
      } else {
        $scope.ccForm.submitDisabled = false;
      }
    };

    $scope.$on('$routeUpdate', function () {
      var paymentSelected = $location.search().paymentSelected;
      if (paymentSelected) {
        $scope.paymentSelected = true;
      } else {
        $scope.paymentSelected = false;
      }
    });

    satelliteTrackingModelService.bookingGuaranteeInfoLoaded($scope.hotelInfo.hotelId);
  }]);
