/* global lqMobileModule, lqMobile*/
/* jshint maxparams: 10 */
lqMobileModule.factory('bookingModelService',
  [
    '$q',
    'storageModelService',
    'searchParamModelService', 'hotelInformationModel', 'bookingEndpointService', 'returnsUserProfileModel', 'reservationsModel', 'paymentTypes', 'ccTokenizationService',
    function (
      $q,
      storageModelService,
      searchParamModelService,
      hotelInformationModel,
      bookingEndpointService,
      returnsUserProfileModel,
      reservationsModel,
      paymentTypes,
      ccTokenizationService
    ) {
      'use strict';

      var dataModel = {};
      var modErrorMsg = 'We\'re sorry, your reservation cannot be modified at this time. Please call 1-800-642-4271 for assistance.';
      var tokErrorMsg = 'We\'re sorry, an unknown error has ocurred within our system. Please call 1-888-250-6469 to continue with your reservation or request further assistance';

      dataModel.data = {
        firstName: '',
        lastName: '',
        emailAddress: '',
        mobilePhone: '',
        zipCode: '',
        joinReturns: '',
        paymentType: '',
        previousPaymentType: '',
        city: 'UNKNOWN',
        country: 'XX',
        state: 'XX',
        street: 'UNKNOWN',
        street2: '',
        creditCard: {
          ccMonth: lqMobile.Defaults.getDefaultCCMonth(),
          ccYear: lqMobile.Defaults.getDefaultCCYear(),
          ccNumber: null,
          ccType: null
        },
        directBill: {
          Number: ''
        },
        confirmationNumber: '',
        creatingUserProfilePromess: null,
        creatingProfileError: false
      };

      function resetAddressBookingData() {
        dataModel.data.firstName = '';
        dataModel.data.lastName = '';
        dataModel.data.emailAddress = '';
        dataModel.data.mobilePhone = '';
        dataModel.data.zipCode = '';
        dataModel.data.city = 'UNKNOWN';
        dataModel.data.country = 'XX';
        dataModel.data.state = 'XX';
        dataModel.data.street = 'UNKNOWN';
        dataModel.data.street2 = '';
      }

      function resetBasicBookingInfo() {
        dataModel.data.joinReturns = '';
        dataModel.data.previousPaymentType = dataModel.data.paymentType;
        dataModel.data.paymentType = '';
        dataModel.data.directBill.Number = '';
        dataModel.data.creatingUserProfilePromess = null;
        dataModel.data.creatingProfileError = false;
      }

      function resetCCInfo() {
        dataModel.data.creditCard.ccMonth = lqMobile.Defaults.getDefaultCCMonth();
        dataModel.data.creditCard.ccYear = lqMobile.Defaults.getDefaultCCYear();
        dataModel.data.creditCard.ccNumber = null;
        dataModel.data.creditCard.ccType = null;
      }

      function setAddressData(data) {
        dataModel.data.city = data.city;
        dataModel.data.country = data.country;
        dataModel.data.state = data.state;
        dataModel.data.street = data.streetAddress1;
        dataModel.data.street2 = data.streetAddress2;
      }

      function isReservationBeingModifiedInstantHold() {
        var reservation = reservationsModel.getSelectedFullReservation();
        if (reservation && reservation.lastName === 'INSTANTHOLD') {
          return true;
        }
        return false;
      }

      function hasCreditCardInformation(reservation) {
        if (reservation.creditCard) {
          return true;
        }
        return false;
      }

      function isMasked(ccNumber) {
        return ccNumber != undefined && ccNumber.indexOf('*') > -1;
      }

      function setProfileData(data) {
        dataModel.data.firstName = data.firstName;
        dataModel.data.lastName = data.lastName;
        dataModel.data.emailAddress = data.emailAddress;
        dataModel.data.zipCode = data.zipCode;
        dataModel.data.mobilePhone = data.mobilePhone;
      }

      function setCreditCardData(creditCard) {
        dataModel.data.creditCard.ccNumber = creditCard.ccNumber;
        dataModel.data.creditCard.ccMonth = creditCard.ccMonth - 1;
        dataModel.data.creditCard.ccYear = creditCard.ccYear;
        dataModel.data.creditCard.ccType = creditCard.ccType;
      }

      function completeEnrollmentProcess() {
        var deferred = $q.defer();

        if (dataModel.data.creatingUserProfilePromess !== null) {
          dataModel.data.creatingUserProfilePromess.then(function () {
            deferred.resolve();
          }, function () {
            dataModel.data.creatingProfileError = true;
            deferred.resolve();
          });
        } else {
      // if the user didn't select to create a profile, there's nothing to be done
          deferred.resolve();
        }

        return deferred.promise;
      }

      dataModel.resetGuaranteeData = function () {
        resetCCInfo();
        dataModel.data.directBill.Number = '';
      };

      dataModel.resetPaymentData = function () {
        dataModel.data.paymentType = '';
      };

      // this function is used when the user starts a new booking to
      // remove *some* data from the previous one
      dataModel.resetBookingData = function () {
        var userProfile;
        // if it was a booking modification, clear the reservation
        if (reservationsModel.isReservationBeingModified()) {
          reservationsModel.clearModel();
        }

        if (returnsUserProfileModel.isLoggedIn()) {
          userProfile = returnsUserProfileModel.getUserProfileData();
          setAddressData(userProfile);
        } else if (!returnsUserProfileModel.isLoggedIn()) {
          resetCCInfo();
        }

        resetBasicBookingInfo();
      };

      dataModel.resetFullBookingData = function () {
        resetCCInfo();
        resetBasicBookingInfo();
        resetAddressBookingData();
        dataModel.resetConfirmationNumber();
      };

      dataModel.resetConfirmationNumber = function () {
        dataModel.data.confirmationNumber = '';
      };

      dataModel.updateBookingDataFromReservation = function () {
        var reservation = reservationsModel.getSelectedFullReservation();
        setProfileData(reservation);
        setAddressData(reservation);
        if (hasCreditCardInformation(reservation)) {
          setCreditCardData(reservation.creditCard);
        }
      };

      dataModel.updateBookingDataFromProfile = function () {
        var userProfile = returnsUserProfileModel.getUserProfileData();
    // if there's not credit card data, use the one from the user profile
        if (!dataModel.data.creditCard.ccNumber) {
          setCreditCardData(userProfile.creditCard);
        }
        if (dataModel.data.mobilePhone === '') {
          dataModel.setMobilePhone(userProfile.mobilePhone);
        }
        setAddressData(userProfile);
        if (
          !reservationsModel.isReservationBeingModified() || isReservationBeingModifiedInstantHold()
        ) {
          setProfileData(userProfile);
        }
      };

      dataModel.setMobilePhone = function (mobilePhone) {
        dataModel.data.mobilePhone = mobilePhone;
      };

      function completeBookingProcess(tkn) {
        var deferred = $q.defer();
        var isUserLoggedIn = returnsUserProfileModel.isLoggedIn();

        bookingEndpointService.completeBooking(
          searchParamModelService.data, hotelInformationModel.data, dataModel.data,
          isUserLoggedIn, returnsUserProfileModel.data,
          reservationsModel.data.selectedFullReservation, tkn
        ).then(
          function (data) {
            var confirmationData = data;
            if (typeof confirmationData !== 'undefined') {
              if (confirmationData.length > 0) {
                confirmationData = data[0];
              }
              dataModel.data.confirmationNumber =
                confirmationData.reservation.confirmationNumber.number;
              // if the user is not logged in, save the confirmation into local storage
              if (!isUserLoggedIn) {
                reservationsModel.addLocalReservation(
                  dataModel.data.confirmationNumber,
                  dataModel.data.firstName,
                  dataModel.data.lastName
                );
              }

              if (isUserLoggedIn && searchParamModelService.data.isBookWithPointsEnabled) {
                returnsUserProfileModel.redeemPoints(
                  searchParamModelService.data.selectedRoom.computedRates.totalPoints
                );
              }
            } else if (reservationsModel.isReservationBeingModified()) {
              dataModel.data.confirmationNumber =
                reservationsModel.getSelectedReservation().confirmationNumber;
            }
            dataModel.resetBookingData();
            dataModel.persistData();
            deferred.resolve();
          },
          function (data) {
            deferred.reject(data);
          }
        );
        return deferred.promise;
      }

      dataModel.completeBooking = function () {
        var deferred = $q.defer();

        completeEnrollmentProcess().then(function () {
          if (isMasked(dataModel.data.creditCard.ccNumber)) {
            completeBookingProcess().then(
          function (completeBookingResponse) {
            deferred.resolve(completeBookingResponse);
          }, function (error) {
              deferred.reject(error);
            });
          } else if (dataModel.data.paymentType === paymentTypes.CREDITCARD) {
            ccTokenizationService.tokenize(dataModel.data.creditCard).then(
              function (ccTokenizeResponse) {
                completeBookingProcess(ccTokenizeResponse.token).then(
                  function (completeBookingResponse) {
                    deferred.resolve(completeBookingResponse);
                  }, function (error) {
                  deferred.reject(error);
                });
              },
              function () {
                // ML isn't available
                if (reservationsModel.isReservationBeingModified()) {
                  deferred.reject(modErrorMsg);
                } else {
                  deferred.reject(tokErrorMsg);
                }
              }
            );
          } else {
            completeBookingProcess().then(
            function (completeBookingResponse) {
              deferred.resolve(completeBookingResponse);
            }, function (error) {
              deferred.reject(error);
            });
          }
        });

        return deferred.promise;
      };

      dataModel.persistData = function () {
        storageModelService.setBookingData(dataModel.data);
      };

      function getBookingData() {
        var bookingDataFromSessionStorage = storageModelService.getBookingData();
        if (bookingDataFromSessionStorage && bookingDataFromSessionStorage.hasOwnProperty('firstName')) {
          return bookingDataFromSessionStorage;
        }
        return '';
      }

      function updateBookingDataModel() {
        var bookingData = getBookingData();

        if (bookingData) {
          dataModel.data = bookingData;
        }
        storageModelService.setBookingData(dataModel.data.hotelDetails);
      }

      updateBookingDataModel();

      return dataModel;
    }]);
