/* global lqMobileModule, console*/
lqMobileModule.controller('readyForYouController', ['$scope', '$modal', '$log', 'events', '$rootScope', 'satelliteTrackingModelService',
  function ($scope, $modal, $log, events, $rootScope, satelliteTrackingModelService) {
    'use strict';

    $scope.selectRoomReady = function () {
      $modal.open({
        templateUrl: 'readyForYouModal.html',
        controller: 'readyForYouModalController',
        windowClass: 'app-modal-window border-radius',
        resolve: {
          showLoadingModal: function () {
            return $scope.showLoadingModal;
          }
        }
      });
      satelliteTrackingModelService.readyForYouOptionSelected();
    };
  }]);

lqMobileModule.controller('readyForYouModalController', ['$scope', '$modalInstance', 'routingService', 'bookingModelService', 'bookingEndpointService', '$timeout', 'showLoadingModal',
  function (
    $scope,
    $modalInstance,
    routingService,
    bookingModelService,
    bookingEndpointService,
    $timeout,
    showLoadingModal
  ) {
    'use strict';

    $scope.defaultArrivalText = CQ.I18n.getMessage('Select your arrival time');
    $scope.bookingConf = bookingModelService.data;
    $scope.readyForm = {};
    $scope.readyForm.phone = bookingModelService.data.mobilePhone;
    $scope.formSubmitted = false;
    $scope.formSubmittedOk = false;
    $scope.validFields = true;
    $scope.validArrivalTime = true;
    $scope.validPhoneNumber = true;
    $scope.arrivalTime = {
      pointRight: true,
      value: $scope.defaultArrivalText
    };

    $scope.status = {
      isopen: false
    };

    $scope.setArrivalTime = function (event, pointer, val) {
      $scope.arrivalTime.pointRight = pointer;
      $scope.arrivalTime.value = val;
      $scope.status.isopen = !$scope.status.isopen;
    };

    $scope.servicesError = '';
    $scope.submit = function (validMobilePhone) {
      $scope.formSubmitted = true;
      $scope.validFields = true;
      $scope.servicesError = '';
      if (validMobilePhone && $scope.arrivalTime.value !== $scope.defaultArrivalText) {
        $scope.loadingModal = showLoadingModal();
        bookingEndpointService.postReadyForYou(
          $scope.readyForm.phone, $scope.bookingConf.confirmationNumber, $scope.arrivalTime.value
        ).then(function () {
          $scope.formSubmittedOk = true;
          $scope.loadingModal.close();
        }, function (error) {
          $scope.loadingModal.close();
          $scope.servicesError = CQ.I18n.getMessage(error);
        });
      } else {
        $scope.validFields = false;
        if (!validMobilePhone) {
          $scope.validPhoneNumber = false;
        } else {
          $scope.validArrivalTime = false;
        }
      }
    };

    $scope.ok = function () {
      $modalInstance.dismiss('ok');
    };

    $scope.close = function () {
      $modalInstance.dismiss('close');
    };

    $scope.clearPhone = function () {
      $scope.readyForm.phone = '';
      $timeout(function () {
        $('#mobilePhone').focus();
      }, 200);
    };

    $scope.getFormFieldCssClass = function (formSubmitted, isFieldValid) {
      if (formSubmitted && !isFieldValid) {
        return 'form-control has-error';
      }
      return 'form-control';
    };
  }]);
