/* global lqMobileModule, Modernizr, navigator, LQMobile, generatedSessionId*/
/* jshint maxparams: 22 */
var privacyPolicyModalInstanceCtrl = function ($scope, $modalInstance) {
  'use strict';

  $scope.close = function () {
    $modalInstance.dismiss('cancel');
  };
};


lqMobileModule.controller('baseController', ['$scope',
  '$q',
  '$rootScope',
  '$window',
  '$timeout',
  '$modal',
  '$location',
  'satelliteTrackingModelService',
  'persistentStorageModelService',
  'searchParamModelService',
  'favoriteHotelsModel',
  'googleMapsEndpointService',
  'events',
  'routingService',
  'hotelSearchModelService',
  'bookingModelService',
  'reservationsModel',
  'sessionIdEndpointService',
  'editSearchAlertFactory',
  'Idle',
  'returnsUserProfileModel',
  'hotelReviewsModel',
  'urlService',
  function ($scope,
    $q,
    $rootScope,
    $window,
    $timeout,
    $modal,
    $location,
    satelliteTrackingModelService,
    persistentStorageModelService,
    searchParamModelService,
    favoriteHotelsModel,
    googleMapsEndpointService,
    events,
    routingService,
    hotelSearchModelService,
    bookingModelService,
    reservationsModel,
    sessionIdEndpointService,
    editSearchAlertFactory,
    Idle,
    returnsUserProfileModel,
    hotelReviewsModel,
    urlService
  ) {
    'use strict';

    var servicesSessionId = window.sessionStorage.getItem('sessionIdFromServices');
    var assignedSessionId = window.sessionStorage.getItem('global.sessionId');

    // set device type
    $rootScope.deviceType = window.device.platform.toLowerCase();

    if (lqmobileConfigs.timeout && lqmobileConfigs.timeout > 0) {
      Idle.watch();
      $scope.$on('IdleTimeout', function () {
        Idle.watch();
        // if the user's been idle for a long time, log them out,
        // and back in to refresh their api key
        if (returnsUserProfileModel.isLoggedIn()) {
          // returnsUserProfileModel.logoutUser();
          // $location.search('iddleTimeOut', true);
          returnsUserProfileModel.authenticateUserFromStorage();
        }
      });
    }


    $scope.viewFullSiteLink = function () {
      if (typeof lqmobileConfigs.fullSiteURL !== 'undefined') {
        urlService.openExternalUrl(lqmobileConfigs.fullSiteURL);
      } else {
        // default to prod link:
        urlService.openExternalUrl('https://www.lq.com/en.html?stop_mobi=yes');
      }
    };

    $scope.goToCustomerSupport = function () {
      urlService.openExternalUrl(lqmobileConfigs.customerSupportLink);
    };

    function getMapsLink(lat, lng) {
      var mapsLink = 'geo:' + lat + ',' + lng;
      if (navigator.userAgent.match(/Android/i)) {
        mapsLink = 'geo:' + lat + ',' + lng + '?q=' + lat + ',' + lng;
      } else if (navigator.userAgent.match(/iPad|iPhone|iPod/g)) {
        mapsLink = 'maps://maps.apple.com/?q=' + lat + ',' + lng + '&z=11';
      } else {
        mapsLink = 'geo:' + lat + ',' + lng;
      }
      return mapsLink;
    }

    function openModal() {
      satelliteTrackingModelService.tripAdvisorRatingSelected();
      $modal.open({
        templateUrl: 'tripadvisorReviewsModal.html',
        controller: 'tripadvisorReviewsModalController',
        windowClass: 'app-modal-window trip-advisor-modal'
      });
    }


    if (!servicesSessionId) {
      sessionIdEndpointService.generateSessionIdCookie(assignedSessionId).then(function () {
        window.sessionStorage.setItem('sessionIdFromServices', assignedSessionId);
      });
    } else {
      window.sessionStorage.setItem('global.sessionId', assignedSessionId);
    }

    $scope.generatedSessionId = assignedSessionId;
    $scope.servicesEndPoint = lqmobileConfigs.servicesEndpoint;

    $rootScope.$on('$routeChangeSuccess', function (event, currentRoute) {
      if (currentRoute.originalPath === '/booking-confirmation') {
        $scope.addThisClass = 'show-addthis';
      } else {
        $scope.addThisClass = 'hide-addthis';
      }
    });

    $scope.Modernizr = Modernizr;
    $scope.isSetValueOnBlurActive = true;
    $scope.showReturs = true;
    $scope.generalErrorMsg = '';

    $scope.searchData = searchParamModelService.data;
    $scope.favHotels = favoriteHotelsModel.data;

    $scope.doPlaceSearch = function () {
    // for now do nothing
    };

    $scope.findHotels = function () {
    // $rootScope.$broadcast(events.findHotelsButtonClicked);
      satelliteTrackingModelService.findHotelsButtonClicked();
      routingService.goToHotelSearch();
    };

    $scope.showLoadingModal = function () {
      return $modal.open({
        templateUrl: 'loadingModalContent.html',
        windowClass: 'loading-modal-window',
        backdrop: 'static',
        keyboard: false
      });
    };

    $scope.openMapModalByAddress = function (address) {
      $scope.mapModalLoading = $scope.showLoadingModal();

      $scope.mapModalLoading.opened.then(
        function () {
          googleMapsEndpointService.loadGoogleMapsAPI().then(
            function () {
              hotelSearchModelService.geolocateAddress(address).then(
                function (geolocatedAddressObject) {
                  var latitude = geolocatedAddressObject.geometry.location.lat();
                  var longitude = geolocatedAddressObject.geometry.location.lng();
                  $scope.openMapModal(latitude, longitude);
                },
                function () {
                  $scope.mapModalLoading.close();
                }
              );
            },
            function () {
              $scope.mapModalLoading.close();
            }
          );
        }
      );
    };

    $scope.openMapModal = function (latitude, longitude) {
      // satelliteTrackingModelService.hotelAddressSelected();
      $scope.mapModalInstance = $modal.open({
        backdrop: 'static',
        templateUrl: 'mapModalContent.html',
        controller: function ($scope, $modalInstance, lat, lng, href) {
          'use strict';

          $scope.lat = lat;
          $scope.lng = lng;
          $scope.href = href;
          // $scope.render = true;

          $scope.ok = function () {
            $modalInstance.close();
          };

          $scope.cancel = function () {
            $modalInstance.dismiss('cancel');
          };
          $scope.mycallback = function (map, iLat, iLng) {
            google.maps.event.trigger(map, 'resize');
            $scope.mymap = map;
            $scope.mymap.setCenter({ lat: iLat, lng: iLng });
            $scope.$apply();
          };
        },
        resolve: {
          lat: function () {
            return latitude;
          },
          lng: function () {
            return longitude;
          },
          href: function () {
            return getMapsLink(latitude, longitude);
          }
        }
      });
      $scope.mapModalInstance.opened.then(
        function () {
          if ($scope.mapModalLoading) $scope.mapModalLoading.close();
        },
        function () {
          if ($scope.mapModalLoading) $scope.mapModalLoading.close();
        }
      );
    };

    $scope.getFormFieldCssClass = function (formSubmitted, isFieldValid) {
      if (formSubmitted && !isFieldValid) {
        return 'form-control has-error';
      }
      return 'form-control';
    };

    $scope.openMainMenu = function () {
    // $rootScope.$broadcast(events.globalMenuOpened);
      satelliteTrackingModelService.globalMenuOpened();
    };

    $scope.closeMainMenu = function () {
      satelliteTrackingModelService.globalMenuClosed();
    // $rootScope.$broadcast(events.globalMenuClosed);
    };

    $scope.globalMenuLinkSelected = function (linkLabel, link, target) {
      LQMobile.Header.closeMainMenu();
      satelliteTrackingModelService.globalMenuLinkSelected(linkLabel);
      $window.open(link, target);
    };

    $scope.openTopMenuLink = function (linkTarget, link, linkLabel) {
      var hasParams = link.indexOf('?') !== -1;
      var canBeRoutedWithAngular = link.indexOf('#') === 0;

      if ($window.location.hash !== link) {
        reservationsModel.clearModel();
      }
      editSearchAlertFactory.clearAlerts();
      LQMobile.Header.closeMainMenu();
      satelliteTrackingModelService.globalMenuLinkSelected(linkLabel);
      if (canBeRoutedWithAngular && !hasParams) {
        $timeout(function () {
          routingService.turnOffBackButton();
          $location.path(link.slice(1));
        }, 5);
      } else {
        $window.location.href = link;
      }
    };

  // intercept tel: links and force a confirmation dialog
    $scope.confirmTelephoneCall = function (phoneNumber) {
      var formatted = phoneNumber.replace('tel:+', '');

    //
    // Use device native confirm dialog
      navigator.notification.confirm(
      '', // message
      function (buttonIndex) {
        //
        // If "Call" button is tapped
        if (buttonIndex == 2) {
          //
          // click on hidden tel link
          window.open(phoneNumber, '_system');
        }
      },
      formatted,           // title
      ['Cancel', 'Call']    // buttonLabels
    );
    };

    $scope.hotelPhoneClick = function (phoneNumber) {
    // $rootScope.$broadcast(events.hotelPhoneLinkSelected, phoneNumber);
      satelliteTrackingModelService.hotelPhoneLinkSelected(phoneNumber);
    };

    $scope.toggleFavorite = function (hotelId, hotelName, hotelAddress, hotelPhone, hotelLocation) {
      var promise;
      if (event.type === 'touchend' || event.type === 'touch') {
        promise = favoriteHotelsModel.toogleFavoriteHotel(
          hotelId, hotelName, hotelAddress, hotelPhone, hotelLocation
        );
        if (promise !== null) {
          $scope.favLoadingModal = $scope.showLoadingModal();
          promise.then(function () {
            console.log('favLoading close');
            $scope.favLoadingModal.close();
          }, function (error) {
            console.log('favLoading error close');
            $scope.favLoadingModal.close();
            $scope.generalErrorMsg = error;
          });
        }
      }
    };

    $rootScope.$on(events.signInActionTaken, function () {
      favoriteHotelsModel.updateHotelList();
      bookingModelService.updateBookingDataFromProfile();
      satelliteTrackingModelService.signInActionTaken();
    });

    $rootScope.$on(events.logoutLinkSelected, function () {
      favoriteHotelsModel.updateHotelList();
      bookingModelService.resetFullBookingData();
      reservationsModel.clearModel();
      satelliteTrackingModelService.logoutLinkSelected();
    });

    $scope.loadHotelDetails = function (hotelId, location) {
      console.log('HOTEL CLICK');
      searchParamModelService.setCurrentPlace(location);
      routingService.goToHotelDetails(hotelId);
    };

    $scope.goToHotelSearch = function () {
      routingService.goToHotelSearch();
    };

    $scope.goToAccountActivity = function () {
      routingService.goToAccountActivity();
    };

    $scope.goToAccountSummary = function () {
      routingService.goToAccountSummary();
    };

    $scope.goToMyReservations = function () {
      routingService.goToMyReservations();
    };

    $scope.hideSuggestions = function () {
      $rootScope.$broadcast('hideSuggestions');
    };

    $scope.toggleMainMenu = function () {
      LQMobile.Header.toggleMainMenu();
      if ($('.mainMenuContainer').hasClass('active')) {
        satelliteTrackingModelService.globalMenuOpened();
      } else {
        satelliteTrackingModelService.globalMenuClosed();
      }
    };

    $scope.goBack = function () {
      routingService.turnOffBackButton();
      $window.history.back();
      // history.go(-1);
      // history.back();
    };

    $scope.closeMainMenu = function () {
      LQMobile.Header.closeMainMenu();
      satelliteTrackingModelService.globalMenuClosed();
    };

    $scope.openTripAdvisorModal = function (id) {
      if (hotelReviewsModel.data.hotelReviews[id]) {
        hotelReviewsModel.setSelectedHotelReviews(hotelReviewsModel.data.hotelReviews[id]);
        openModal();
      } else {
      // if we haven't already got reviews for this hotel
        $scope.loadingModal = $scope.showLoadingModal();
        hotelReviewsModel.loadHotelReviews(id).then(
          function () {
            console.log('openTripAdvsior Close');
            $scope.loadingModal.close();
            openModal();
          }
        );
      }
    };

    $scope.openPrivacyPolicyModal = function () {
      $modal.open({
        templateUrl: lqmobileConfigs.httpBase + '/content/lq/lq-mobile-app-wrapper2/' + lqmobileConfigs.currentLocale + '/findandbook/privacy-policy.content.html',
        controller: privacyPolicyModalInstanceCtrl,
        windowClass: 'app-modal-window privacy-policy-modal'
      });
    };

    /** GLOBAL ACTIONS FOR CHECK IN AND CHECK OUT */

    /**
    * executeUberRequest
    *
    * Implement Uber deep linking API with possible future params
    */
    $scope.executeUberRequest = function (address) {
      var scheme;
      var addressParams = address ? '?action=setPickup&pickup=my_location&dropoff[formatted_address]=' + address : '';

      // Determine whether or not the Uber app is installed
      try {
        //
        // set Uber app scheme based on platform.
        if (device.platform === 'iOS') {
          scheme = 'uber://';
        } else if (device.platform === 'Android') {
          scheme = 'com.ubercab';
        } else {
          throw 'Could not detect device.';
        }

          //
        // Perform App availablity check using plugin
        appAvailability.check(

          scheme,

          // success, app IS installed.
          function () {
            console.log(scheme + ' is available...');

            urlService.openExternalUrl('uber://' + addressParams);
          },

          // failure, NO app.
          function () {
            console.log(scheme + ' is not available!');

            urlService.openExternalUrl('https://m.uber.com/ul/' + addressParams);
          }
        );
      } catch (e) {
        //
        // Error: Fall back to HTTP link, it is safer.
        urlService.openExternalUrl('https://tinyurl.com/z974ck5');
      }
    };

    /**
    * executeGetDirections
    *
    * Open Google Maps in external browser.
     *
     * @param {string} address - address to navigate to from current location
    */
    $scope.executeGetDirections = function (address) {
      var endPosition;

        // Open Google Maps in browser.
      if (device.platform === 'iOS') {
        endPosition = address ? '&daddr=' + address : '';
        urlService.openExternalUrl('https://maps.apple.com/?saddr=Current+Location' + endPosition);
      } else {
        endPosition = address ? '&destination=' + address : '';
        urlService.openExternalUrl('https://www.google.com/maps/dir/?api=1&origin=Current+Location' + endPosition);
      }
    };

    /**
     * executePhoneCall
     *
     * @param {String} phoneNumber - number to hotel
     */
    $scope.executePhoneCall = function (phoneNumber) {
      document.location.href = 'tel:' + phoneNumber;
    };
  }]);
