/* global lqMobileModule, LQMobile*/
/* jshint maxparams: 10 */
lqMobileModule.factory('hotelInformationEndpointService', ['$http', 'serviceEndPoints',
  function ($http, serviceEndPoints) {
    'use strict';

    var endpointService = {};

    endpointService.getHotelDetails = function (hotelId) {
      var serviceURL = serviceEndPoints.getHotelDetailsEndPoint(hotelId);

      return $http.get(serviceURL).then(
      function (data) {
        return data;
      },
      function (error) {
        console.error(JSON.stringify(error));
        throw error;
      }
    );
    };

    endpointService.getHotelSummaries = function () {
      var serviceURL = serviceEndPoints.getHotelSummariesEndPoint();

      return $http.get(serviceURL).then(
      function (response) {
                // Ensure URLs are // instead of http://
        response.data.forEach(function (result) {
          result.hotelRating.imageURL = result.hotelRating.imageURL.replace(/^https?:/, '');
        });
        return response.data;
      },
      function (error) {
        throw error;
      }
    );
    };

    endpointService.getHotelReviews = function (hotelId) {
      var serviceURL = serviceEndPoints.getHotelReviewsEndPoint(hotelId);

      return $http.get(serviceURL).then(
                function (data) {
                  return data;
                },
                function (error) {
                  console.error(JSON.stringify(error));
                  throw error;
                }
        );
    };

    endpointService.getHotelNearbyPlaces = function (lat, log, category) {
      var auth = {
        consumerKey: yelpConfig.consumerKey,
        consumerSecret: yelpConfig.consumerSecret,
        accessToken: yelpConfig.accessToken,
        accessTokenSecret: yelpConfig.secretToken,
        serviceProvider: {
          signatureMethod: yelpConfig.signatureMethod
        }
      };
      var accessor = {
        consumerSecret: auth.consumerSecret,
        tokenSecret: auth.accessTokenSecret
      };
      var parameters = [];
      var callbackId = angular.callbacks.counter.toString(36);
      var callbackName = 'angular.callbacks._' + callbackId;
      var message;
      var parameterMap;

      parameters.push(['ll', lat + ',' + log]);
      if (category != 'everything') {
        parameters.push(['category_filter', category]);
      }
      parameters.push(['radius_filter', 40000]);
      parameters.push(['sort', 1]);
      parameters.push(['callback', callbackName]);
      parameters.push(['oauth_consumer_key', auth.consumerKey]);
      parameters.push(['oauth_consumer_secret', auth.consumerSecret]);
      parameters.push(['oauth_token', auth.accessToken]);
      parameters.push(['oauth_signature_method', 'HMAC-SHA1']);

      message = {
        action: 'https://api.yelp.com/v2/search',
        method: 'GET',
        parameters: parameters
      };

      OAuth.setTimestampAndNonce(message);
      OAuth.SignatureMethod.sign(message, accessor);

      parameterMap = OAuth.getParameterMap(message.parameters);

      return $http.jsonp(message.action, { params: parameterMap }).then(
                function (data) {
                  return data;
                },
                function (error) {
                  console.error(JSON.stringify(error));
                  throw error;
                }
        );
    };

    return endpointService;
  }]);
