/* global lqMobileModule*/
lqMobileModule.factory('hotelInformationModel', ['$q', 'storageModelService', 'hotelInformationEndpointService',
  function ($q, storageModelService, hotelInformationEndpointService) {
    'use strict';

    var dataModel = {
      data: {
        hotelDetails: {},
        hotelSummaries: null,
        hotelSummariesPromise: null
      }
    };

    dataModel.setHotelDetails = function (hotelDetails) {
      dataModel.data.hotelDetails = hotelDetails;
      storageModelService.setHotelDetails(dataModel.data.hotelDetails);
    };

    dataModel.loadHotelDetails = function (hotelId) {
      return hotelInformationEndpointService.getHotelDetails(hotelId).then(
        function (response) {
          if (response.data === '') {
            throw 'Could not retrieve hotel details';
          }
          dataModel.setHotelDetails(response.data);
        },
        function () {
          throw 'Could not retrieve hotel details';
        });
    };

    function getHotelDetails() {
      var searchParamsFromSessionStorage = storageModelService.getHotelDetails();
      if (searchParamsFromSessionStorage && searchParamsFromSessionStorage.hasOwnProperty('hotelId')) {
        return searchParamsFromSessionStorage;
      }
      return '';
    }

    function updateHotelDetails() {
      var hotelDetails = getHotelDetails();

      if (hotelDetails) {
        dataModel.data.hotelDetails = hotelDetails;
      }
      storageModelService.setHotelDetails(dataModel.data.hotelDetails);
    }

    function loadHotelSummaries() {
      dataModel.data.hotelSummariesPromise = hotelInformationEndpointService.getHotelSummaries();
      dataModel.data.hotelSummariesPromise.then(function (data) {
        dataModel.data.hotelSummaries = data;
      });
    }

    updateHotelDetails();
    loadHotelSummaries();

    return dataModel;
  }]);
