/* global lqMobileModule*/
lqMobileModule.factory('routingService', [
  '$window', '$location', '$timeout', '$route',
  function ($window, $location, $timeout, $route) {
    'use strict';

    var service = {};

    function setPathAsync(path, force) {
      $timeout(function () {
        $location.path(path);
        if (force) {
          $timeout(function () {
            $route.reload();
          }, 5);
        }
      }, 5);
    }

    function setPathNoTimeout(path, force) {
      $location.path(path);
      if (force) {
        $route.reload();
      }
    }

    service.turnOnBackButton = function () {
      $('.navbar-header').addClass('back');
    };

    service.turnOffBackButton = function () {
      $('.navbar-header').removeClass('back');
    };

    /*
     * goTo uses $location.path, which does not require a browser refresh.
     * load uses $window.location.href, which will cause a browser refresh.
     */

    service.goToHomePage = function () {
      setPathAsync('/home');
      service.turnOffBackButton();
    };

    service.goToSignInPage = function (force) {
      if (typeof force === 'undefined') {
        force = false;
      }
      setPathAsync('/sign-in', force);
      service.turnOffBackButton();
    };

    service.goToHotelSearch = function () {
      setPathAsync('/hotel-search');
      service.turnOffBackButton();
    };

    service.goToHotelDetails = function (hotelId) {
      setPathAsync('/hotel-details/' + hotelId);
      service.turnOnBackButton();
    };

    service.goToBookingRecap = function () {
      setPathAsync('/booking-recap');
      service.turnOnBackButton();
    };

    service.goToBookingGuest = function () {
      setPathAsync('/booking-guest-info');
      service.turnOnBackButton();
    };

    service.goToBookingGuarantee = function () {
      setPathAsync('/booking-guarantee-info');
      service.turnOnBackButton();
    };

    service.goToInstantHold = function () {
      setPathAsync('/instant-hold');
      service.turnOnBackButton();
    };

    service.goToBookingConfirmation = function () {
      setPathAsync('/booking-confirmation');
      service.turnOnBackButton();
    };

    service.goToSingUp = function () {
      setPathAsync('/join-lq-returns');
      service.turnOnBackButton();
    };

    service.goToAccountSummary = function () {
      setPathAsync('/account-summary');
      service.turnOffBackButton();
    };

    service.goToAccountActivity = function () {
      setPathAsync('/account-activity');
      service.turnOffBackButton();
    };

    service.goToMyReservations = function () {
      setPathAsync('/my-reservations');
      service.turnOffBackButton();
    };

    service.goToReservationDetails = function () {
      setPathAsync('/reservation-details');
      service.turnOffBackButton();
    };

    service.goToEditProfile = function () {
      setPathAsync('/edit-profile');
      service.turnOnBackButton();
    };

    service.loadHotelSearch = function () {
      $window.location.href = '/content/lq/lq-mobile-app-wrapper2/en.html#/hotel-search';
    };

    service.goToFavoriteHotels = function () {
      setPathAsync('/favorite-hotels');
      service.turnOffBackButton();
    };

    service.goToChangePassword = function () {
      setPathAsync('/change-password');
      service.turnOnBackButton();
    };

    service.goToMobileCheckout = function () {
      setPathAsync('/mobile-checkout');
      service.turnOffBackButton();
    };

    service.goToCheckoutConfirmation = function (force) {
      if (typeof force === 'undefined') {
        force = false;
      }
      setPathAsync('/checkout-confirmation', force);
      service.turnOffBackButton();
    };

    service.goToCheckoutUnavailable = function (force) {
      if (typeof force === 'undefined') {
        force = false;
      }
      setPathAsync('/checkout-unavailable', force);
      service.turnOffBackButton();
    };

    service.goToMobileCheckin = function () {
      setPathAsync('/mobile-check-in');
      service.turnOffBackButton();
    };

    service.goToMobileCheckinResult = function (force, noTimeout) {
      if (typeof force === 'undefined') {
        force = false;
      }
      if (noTimeout) {
        setPathNoTimeout('/mobile-check-in-result', force);
      } else {
        setPathAsync('/mobile-check-in-result', force);
      }
      service.turnOffBackButton();
    };

    service.goToFreeNightRoomSelection = function (hotelId) {
      setPathAsync('/free-night/room-search/' + hotelId);
      service.turnOnBackButton();
    };

    service.goToFreeNightConfirmation = function () {
      setPathAsync('/free-night/confirmation');
      service.turnOnBackButton();
    };

    service.goToFreeNightInfo = function () {
      setPathAsync('/free-night/info');
      service.turnOnBackButton();
    };

    service.goToCustomPage = function () {
      setPathAsync('/custom-page');
      service.turnOnBackButton();
    };

    return service;
  }]);
