/* global lqMobileModule, lqMobile, LQMobile*/
/* jshint maxstatements: 50 */
lqMobileModule.factory('searchParamModelService', ['storageModelService', '$location', 'lodash', 'reservationsModel',
  function (storageModelService, $location, lodash, reservationsModel) {
    'use strict';

    var service = {};
    var parseLocation = function (location) {
      var pairs = location.substring(1).split('&');
      var obj = {};
      var pair;
      var index;
      var pairLength = pairs.length;

      for (index = 0; index < pairLength; index += 1) {
        pair = pairs[index];
        if (pair && pair !== '') {
          pair = pair.split('=');
          obj[decodeURIComponent(pair[0])] = decodeURIComponent(pair[1]);
        }
      }
      return obj;
    };

    service.data = {
      checkinDate: lqMobile.Defaults.getDefaultCheckin(),
      checkoutDate: lqMobile.Defaults.getDefaultCheckout(),
      numberOfAdults: 1,
      numberOfChildren: 0,
      numberOfRooms: 1,
      numGuests: '',
      rateType: LQMobile.Util.getDefaultRate(),
      range: lqMobile.Defaults.getDefaultSearchRange(),
      currentPlace: null,
      latitude: null,
      longitude: null,
      resultsViewType: 'list',
      promoCode: '',
      cid: '',
      icid: '',
      iata: '',
      searchType: '',
      selectedRoom: null,
      isBookWithPointsEnabled: false,
      rate: {}
    };

    service.setSelectedRoom = function (room, isBookWithPointsEnabled) {
      service.data.selectedRoom = room;
      service.data.isBookWithPointsEnabled = isBookWithPointsEnabled;
      storageModelService.setSearchParam(service.data);
    };

    service.setCurrentPlace = function (address) {
      service.clearHotelSearchParams();
      service.data.currentPlace = address;
      service.data.searchType = 'address';
      storageModelService.setSearchParam(service.data);
    };

    service.setCurrentLocation = function (latitude, longitude) {
      service.clearHotelSearchParams();
      service.data.latitude = latitude;
      service.data.longitude = longitude;
      service.data.searchType = 'location';
      storageModelService.setSearchParam(service.data);
    };

    service.setNoPermission = function () {
      service.clearHotelSearchParams();
      service.data.searchType = 'nopermission';
    };

    service.clearHotelSearchParams = function () {
      service.data.currentPlace = null;
    };

    service.updateSearchData = function (searchData) {
      service.data = searchData;
      storageModelService.setSearchParam(service.data);
    };

    service.setResultsViewType = function (resultsViewType) {
      service.data.resultsViewType = resultsViewType;
      storageModelService.setSearchParam(service.data);
    };

    service.updateDataFromSelectedReservation = function () {
      var selectedReservation = reservationsModel.getSelectedFullReservation();
      service.data.checkinDate = new Date(selectedReservation.arrivalDate);
      service.data.checkoutDate = new Date(selectedReservation.departureDate);
      service.data.numberOfAdults = selectedReservation.adults;
      service.data.currentPlace = selectedReservation.hotelCity + ', ' + selectedReservation.hotelStateProvinceDisplay;
      if (selectedReservation.rateType !== 'LQHOLD') {
        service.data.rate.rateCode = selectedReservation.rateQualifier.rateCode.code;
        service.data.rate.rateType = selectedReservation.rateQualifier.rateCode.type;
      } else {
        service.data.rateType = LQMobile.Util.getDefaultRate();
        service.data.selectedRoom = selectedReservation.selectedRoom;
      }
    };

    function getSearchParamValue(propertyName) {
      var searchParamsFromSessionStorage = storageModelService.getSearchParam();
      if (
        searchParamsFromSessionStorage
        && searchParamsFromSessionStorage.hasOwnProperty(propertyName)
      ) {
        return searchParamsFromSessionStorage[propertyName];
      }
      return '';
    }

    function setCheckinDate(checkinDate) {
      var dateObject = LQMobile.Util.sanitizeDate(checkinDate);
      var currentDate = new Date();
      if (dateObject === null) {
        dateObject = lqMobile.Defaults.getDefaultCheckin();
      }
      currentDate.setHours(0, 0, 0, 0);

      if (dateObject.getTime() > currentDate.getTime()) {
        service.data.checkinDate = dateObject;
      }
    }

    function setCheckoutDate(checkoutDate) {
      var dateObject = LQMobile.Util.sanitizeDate(checkoutDate);
      var dayAfterCheckin;

      if (dateObject === null) {
        dateObject = lqMobile.Defaults.getDefaultCheckout();
      }
      if (dateObject.getTime() > service.data.checkinDate.getTime()) {
        service.data.checkoutDate = dateObject;
      } else {
        dayAfterCheckin = new Date(dateObject);
        dayAfterCheckin.setDate(dayAfterCheckin.getDate() + 1);
        service.data.checkoutDate = dayAfterCheckin;
      }
    }

    function getURLParameters() {
      var result = $location.search();
      result = lodash.merge(result, parseLocation(window.location.search));
      return result;
    }

  /*
   * URL Params have priority over session storage
   */
    function updateSearchParams() {
      var urlParams = getURLParameters();
      var checkinDate = urlParams.indate ? urlParams.indate : getSearchParamValue('checkinDate');
      var checkoutDate = urlParams.outdate ? urlParams.outdate : getSearchParamValue('checkoutDate');
      var currentPlace = urlParams.searchCity ? urlParams.searchCity : getSearchParamValue('currentPlace');
      var resultsViewType = urlParams.resultsViewType ? urlParams.resultsViewType : getSearchParamValue('resultsViewType');
      var rateType = urlParams.specialRates ? urlParams.specialRates : getSearchParamValue('rateType');
      var promoCode = urlParams.promoCode ? urlParams.promoCode : getSearchParamValue('promoCode');
      var cid = urlParams.cid ? urlParams.cid : getSearchParamValue('cid');
      var icid = urlParams.icid ? urlParams.icid : getSearchParamValue('icid');
      var iata = urlParams.iata ? urlParams.iata : getSearchParamValue('iata');
      var numberOfAdults = urlParams.adults ? parseInt(urlParams.adults, 10) : getSearchParamValue('numberOfAdults');
      var range = getSearchParamValue('range');
      var searchType = getSearchParamValue('searchType');
      var latitude = getSearchParamValue('latitude');
      var longitude = getSearchParamValue('longitude');
      var selectedRoom = getSearchParamValue('selectedRoom');
      var isBookWithPointsEnabled = getSearchParamValue('isBookWithPointsEnabled');

      if (checkinDate) {
        setCheckinDate(checkinDate);
      }
      if (checkoutDate) {
        setCheckoutDate(checkoutDate);
      }
      if (currentPlace) {
        service.data.currentPlace = currentPlace;
      }
      if (rateType) {
        service.data.rateType = rateType;
      }
      if (promoCode) {
        service.data.rateType = 'CORPORATE';
        service.data.promoCode = promoCode;
      }
      if (resultsViewType) {
        service.data.resultsViewType = resultsViewType;
      }
      if (cid) {
        service.data.cid = cid;
      }
      if (icid) {
        service.data.icid = icid;
      }
      if (iata) {
        service.data.iata = iata;
      }
      if (
        numberOfAdults && numberOfAdults > 0
        && numberOfAdults <= LQMobile.Util.getMaxAllowedAdults()
      ) {
        service.data.numberOfAdults = numberOfAdults;
      }
      if (range) {
        service.data.range = range;
      }
      if (searchType) {
        service.data.searchType = searchType;
      }
      if (longitude) {
        service.data.longitude = longitude;
      }
      if (latitude) {
        service.data.latitude = latitude;
      }
      if (selectedRoom) {
        service.data.selectedRoom = selectedRoom;
      }
      if (isBookWithPointsEnabled) {
        service.data.isBookWithPointsEnabled = isBookWithPointsEnabled;
      }

      storageModelService.setSearchParam(service.data);
    }

    updateSearchParams();

    return service;
  }]);
