/* global lqMobileModule*/
lqMobileModule.factory('favoriteHotelsModel', ['$rootScope', '$q', 'persistentStorageModelService', 'returnsUserProfileModel', 'events', 'satelliteTrackingModelService',
  function (
    $rootScope,
    $q,
    persistentStorageModelService,
    returnsUserProfileModel,
    events,
    satelliteTrackingModelService
  ) {
    'use strict';

    var dataModel = {
      data: {
        favoriteHotels: {},
        favoriteIds: []
      }
    };

    // this function doesn't seem used in scope, but could be useful later, disabled lint
    function getFavoriteHotelsFromProfile(favoriteHotels) { // eslint-disable-line
      var favHotels = {};
      var i;
      for (i = 0; i < favoriteHotels.length; i += 1) {
        dataModel.data.favoriteIds.push(favoriteHotels[i].innNumber.id);
        favHotels[favoriteHotels[i].innNumber.id] = favoriteHotels[i];
      }
      return favHotels;
    }

    function updateFavoriteHotelsInStorage() {
      var defer = $q.defer();
      persistentStorageModelService.setFavoriteHotels(dataModel.data.favoriteIds).then(
        function () {
          defer.resolve();
        },
        function () {
          defer.reject();
        }
      );
      return defer.promise;
    }

    function updateLocalModel() {
      var favArray = dataModel.data.favoriteIds;
      var favArrayLength = favArray.length;
      var index;
      var hotelObj;

      dataModel.data.favoriteHotels = {};

      for (index = 0; index < favArrayLength; index += 1) {
        hotelObj = favArray[index];
        dataModel.data.favoriteHotels[hotelObj.id] = hotelObj;
      }
    }

    function getFavoriteHotelsFromStorage() {
      var defer = $q.defer();
      persistentStorageModelService.getFavoriteHotels().then(
        function (hotelsFromStorage) {
          if (hotelsFromStorage && hotelsFromStorage.length > 0) {
            defer.resolve(hotelsFromStorage);
          } else {
            defer.resolve([]);
          }
        },
        function () {
          // error, return empty array
          defer.resolve([]);
        }
      );
      return defer.promise;
    }

    function loadFavHotelsData() {
      if (returnsUserProfileModel.isLoggedIn()) {
        dataModel.data.favoriteIds = returnsUserProfileModel.data.userProfile.favoriteHotels;
        updateLocalModel();
      } else {
        getFavoriteHotelsFromStorage().then(
          function (favIDs) {
            dataModel.data.favoriteIds = favIDs;
            updateLocalModel();
          },
          function () {
            dataModel.data.favoriteIds = [];
            updateLocalModel();
          }
        );
      }
    }

    function updateFavoriteHotelsInProfile() {
      returnsUserProfileModel.data.userProfile.favoriteHotels = dataModel.data.favoriteIds;
      return returnsUserProfileModel.editUserProfile(returnsUserProfileModel.data.userProfile);
    }

    function addFavoriteHotel(hotelId, hotelName, hotelAddress, phone, hotelLocation) {
      var localFavArray = dataModel.data.favoriteIds;
      var hotelObj = {
        name: hotelName,
        id: hotelId,
        address: hotelAddress,
        phone: phone,
        location: hotelLocation
      };
      localFavArray.unshift(hotelObj);
      // if list is bigger than 10, remove last hotel
      if (localFavArray.length > 10) {
        localFavArray = localFavArray.slice(0, 10);
      }
      return localFavArray;
    }

    function removeFavoriteHotel(hotelId) {
      var result = dataModel.data.favoriteIds;
      var resultLength = result.length;
      var index;
      var hotel;

      for (index = 0; index < resultLength; index += 1) {
        hotel = result[index];
        if (hotel.id === hotelId) {
          result.splice(index, 1);
          break;
        }
      }

      return result;
    }

    dataModel.toogleFavoriteHotel = function (
      hotelId, hotelName, hotelAddress, hotelPhone, hotelLocation
    ) {
      var modifiedFavHotels = null;
      var promise;
    // add or remove
      if (dataModel.data.favoriteHotels[hotelId]) {
        satelliteTrackingModelService.hotelRemovedFromFavorites(hotelId);
        modifiedFavHotels = removeFavoriteHotel(hotelId);
      } else {
        satelliteTrackingModelService.hotelSavedInFavorites(hotelId);
        modifiedFavHotels = addFavoriteHotel(
          hotelId, hotelName, hotelAddress, hotelPhone, hotelLocation
        );
      }
    // update model or storage
      if (modifiedFavHotels !== null) {
        if (returnsUserProfileModel.isLoggedIn()) {
          promise = updateFavoriteHotelsInProfile().then(function () {
            dataModel.data.favoriteIds = modifiedFavHotels;
            return updateLocalModel();
          }, function (error) {
            throw error;
          });
        } else {
          promise = updateFavoriteHotelsInStorage().then(
            function () {
              dataModel.data.favoriteIds = modifiedFavHotels;
              updateLocalModel();
            },
            function () {
              // sometimes things happen, and there's nothing we can do about it.
              dataModel.data.favoriteIds = modifiedFavHotels;
              updateLocalModel();
            }
          );
        }
      }
      return promise;
    };

    dataModel.updateHotelList = function () {
      loadFavHotelsData();
    };

    // try to auth user
    // returnsUserProfileModel.authenticateUserFromStorage().then(loadFavHotelsData);
    loadFavHotelsData();

    return dataModel;
  }]);
