/* global lqMobileModule*/
/*
 * Filter to express currency with the desired format at LQ.
 * Default will use the decimal separator ',' and it's undesired.
 */
lqMobileModule.filter('lqcurrency', function ($filter) {
  'use strict';

    // TODO: do we really want to re-assign parameters
  return function (input, curSymbol, decPlaces, thouSep, decSep) {
    var out;
    var minus;

    curSymbol = curSymbol || '$';
    decPlaces = decPlaces || 2;
    thouSep = thouSep || ',';
    decSep = decSep || '.';

    if (typeof input !== 'undefined' && input !== 0) {
            // Check for invalid inputs
      out = isNaN(input) || input === '' || input === null ? 0.0 : input;

            // Deal with the minus (negative numbers)
      minus = input < 0;
      out = Math.abs(out);
      out = $filter('number')(out, decPlaces);

      // Replace the thousand and decimal separators.
      // This is a two step process to avoid overlaps between the two
      if (thouSep !== ',') {
        out = out.replace(/,/g, 'T');
      }
      if (decSep !== '.') {
        out = out.replace(/\./g, 'D');
      }
      out = out.replace(/T/g, thouSep);
      out = out.replace(/D/g, decSep);

            // Add the minus and the symbol
      if (minus) {
        return '-' + curSymbol + out;
      }
      return curSymbol + out;
    }
    return curSymbol + 0;
  };
});
