/* global lqMobileModule */
lqMobileModule.controller(
  'freeNightConfirmationController',
  [
    '$scope',
    '$modal',
    '$window',
    '$rootScope',
    'routingService',
    'searchParamModelService',
    'reservationsModel',
    'hotelInformationModel',
    'bookingModelService',
    'editSearchPanelCollapseFactory',
    'hotelRoomsModel',
    'freeNightModel',
    'returnsUserProfileModel',
    'ReservationSummary',
    '$timeout',
    'events',
    'lodash',
    'satelliteTrackingModelService',
    'dateFormatterService',
    function ($scope,
      $modal,
      $window,
      $rootScope,
      routingService,
      searchParamModelService,
      reservationsModel,
      hotelInformationModel,
      bookingModelService,
      editSearchPanelCollapseFactory,
      hotelRoomsModel,
      freeNightModel,
      returnsUserProfileModel,
      ReservationSummary,
      $timeout,
      events,
      lodash,
      satelliteTrackingModelService,
      dateFormatterService) {
      'use strict';

      $scope.today = new Date();
      $scope.tomorrow = new Date(new Date().getTime() + (24 * 60 * 60 * 1000));
      $scope.hotelInfo = hotelInformationModel.data.hotelDetails;
      $scope.selectedRoom = freeNightModel.selectedRoom;
      $scope.userLoggedIn = returnsUserProfileModel.isLoggedIn();
      $scope.phoneError = false;
      $scope.fncError = false;

      $scope.showLoadingModal = function () {
        var loadingModalInstance = $modal.open({
          templateUrl: 'loadingModalContent.html',
          windowClass: 'loading-modal-window',
          backdrop: 'static',
          keyboard: false
        });
        return loadingModalInstance;
      };

      $scope.open = function () {
        satelliteTrackingModelService.bookingInfoModalLoaded($scope.hotelInfo.hotelId);
        $modal.open({
          templateUrl: 'loginModal.html',
          controller: 'loginModalInstanceController',
          windowClass: 'app-modal-window',
          resolve: {
            showLoadingModal: function () {
              return $scope.showLoadingModal;
            }
          }
        });
      };

      function hasValidPhone() {
        var phoneNumberToValidate = '';

        if ($scope.user.mobilePhone !== null && $scope.user.mobilePhone.length > 0) {
          phoneNumberToValidate = $scope.user.mobilePhone;
        } else if ($scope.user.secondPhone !== null && $scope.user.secondPhone.length > 0) {
          phoneNumberToValidate = $scope.user.secondPhone;
        } else if (lodash.trim($scope.guestMobilePhone) !== '') {
          phoneNumberToValidate = lodash.trim($scope.guestMobilePhone);
        } else {
          return false;
        }
        phoneNumberToValidate = phoneNumberToValidate.replace(/\D/g, '');

        return phoneNumberToValidate.length >= 10;
      }


      function forwardToConfirmation() {
        var daysToArrival = 0;
        var numberOfNights = 1;
        var checkinDate = searchParamModelService.data.checkinDate;
        var checkoutDate = searchParamModelService.data.checkoutDate;
        var numberOfAdults = searchParamModelService.data.numberOfAdults;
        var promoCode = searchParamModelService.data.promoCode;
        var totalRate = searchParamModelService.data.selectedRoom.hotelRoomRate.totalRate.amount;
        var reservationRevenue =
          searchParamModelService.data.selectedRoom.hotelRoomRate.totalRateWithTaxes.amount;
        var totalTaxes = reservationRevenue - totalRate;
        var roomTypeName =
          searchParamModelService.data.selectedRoom.hotelRoomRate.roomType.roomDescription;
        var guestPostalCode = bookingModelService.data.zipCode;
        var roomBookingType = 'mobile';
        var confirmationNum = bookingModelService.data.confirmationNumber;
        var isFreeNight = true;
        var currencyCode =
          searchParamModelService.data.selectedRoom.hotelRoomRate.averageNightlyRate.currencyCode;

        routingService.goToBookingConfirmation();
        $scope.confirmLoadingModal.close();

        satelliteTrackingModelService.standardConfirmationLoaded(
                   numberOfNights,
                  totalRate,
                  totalTaxes,
                  currencyCode,
                  dateFormatterService.getFormattedDate(checkinDate),
                  dateFormatterService.getFormattedDate(checkoutDate),
                  numberOfAdults,
                  daysToArrival,
                  roomTypeName,
                  guestPostalCode,
                  promoCode,
                  roomBookingType,
                  confirmationNum,
                  isFreeNight,
                  reservationRevenue
              );
      }

      // no value to return, disabling lint
      $scope.confirmFreeNight = function () { // eslint-disable-line
        var overridePhone;

        $scope.fncError = false; // in case they tried to confirm again

        if (!hasValidPhone()) {
          $('#userPhone').addClass('has-error');
          $('#free-night-phone-error').removeClass('ng-hide');
          $scope.phoneError = true;
          return true;
        }
        overridePhone = $scope.displayPhone;
        $scope.phoneError = false;
        $('#userPhone').removeClass('has-error');
        $('#free-night-phone-error').addClass('ng-hide');

        if (lodash.trim(overridePhone) === '') {
          overridePhone = $scope.guestMobilePhone;
        }

        $scope.confirmLoadingModal = $scope.showLoadingModal();

        freeNightModel.completeBooking(
          freeNightModel.selectedRoom.hotelRoomRate, overridePhone
        ).then(
          function (response) {
            var confirmationData = response.data;
            if (typeof confirmationData !== 'undefined') {
              if (confirmationData.length > 0) {
                confirmationData = confirmationData[0];
              }
              bookingModelService.data.confirmationNumber =
                confirmationData.reservation.confirmationNumber.number;
              returnsUserProfileModel.redeemPoints(
                confirmationData.reservation.roomReservations[0].nightlyRoomRates[0].points
              );
            }

            bookingModelService.resetBookingData();
            bookingModelService.persistData();

            searchParamModelService.data.checkinDate = $scope.today;
            searchParamModelService.data.checkoutDate = $scope.tomorrow;
            searchParamModelService.data.numberOfAdults = 1;
            searchParamModelService.data.numberOfRooms = 1;

            freeNightModel.selectedRoom.computedRates = {};
            freeNightModel.selectedRoom.computedRates.totalPoints =
              freeNightModel.selectedRoom.hotelRoomRate.nightlyRates[0].points;
            searchParamModelService.data.selectedRoom = freeNightModel.selectedRoom;
            searchParamModelService.data.selectedRoom.hotelRoomRate.totalRate.amount = 0;
            // searchParamModelService.data.selectedRoom.computedRates.totalPoints = ;
            searchParamModelService.data.selectedRoom.hotelRoomRate.totalRate.amount = 0;
            bookingModelService.data.confirmationNumber =
              confirmationData.reservation.confirmationNumber.number;
            bookingModelService.data.firstName = confirmationData.reservation.mainContact.firstName;
            bookingModelService.data.lastName = confirmationData.reservation.mainContact.lastName;
            bookingModelService.data.paymentType = 'freeNight';
            freeNightModel.reset();
            forwardToConfirmation();
          },
          function () {
            // error....
            $scope.confirmLoadingModal.close();
            // @todo: need to add error messaging for when a FNC reservation fails
            $scope.fncError = true;
          }
        );
      };

      $scope.user = returnsUserProfileModel.data.userProfile;

      $scope.userHasPhone = function () {
        return ($scope.user.mobilePhone !== null && $scope.user.mobilePhone.length > 0)
          ||
        ($scope.user.secondPhone !== null && $scope.user.secondPhone.length > 0);
      };

      $scope.displayPhone = (function () {
        if ($scope.user.mobilePhone !== null && $scope.user.mobilePhone.length > 0) {
          return $scope.user.mobilePhone;
        } else if ($scope.user.secondPhone !== null && $scope.user.secondPhone.length > 0) {
          return $scope.user.secondPhone;
        }
        return '';
      }());

      $scope.goToGuestInfo = function () {
        routingService.goToBookingGuest();
      };

      $scope.cancelBooking = function () {
        routingService.goToHomePage();
      };

      satelliteTrackingModelService.bookingRecapLoaded(
        $scope.hotelInfo.hotelId, searchParamModelService.data.promoCode
      );
    }
  ]
);
