/* global lqMobileModule */
/**
 * Created by jeffrey on 7/28/16.
 */

/* global lqMobileModule, console, angular*/
lqMobileModule.factory(
  'freeNightEndpointService',
  [
    '$window',
    '$q',
    '$http',
    'serviceEndPoints',
    'lodash',
    'geolocationService',
    function (
      $window,
      $q,
      $http,
      serviceEndPoints,
      lodash,
      geolocationService
    ) {
      'use strict';

      var endpointService = {};

      /**
       * checks if a user is eligible for mobile checkout by LQ_AUTH_KEY
       * this value should be provided when the user is authenticated as "token"
       *
       * @param token string
       * @param arrivalDate string yyyy-mm-dd
       * @param departureDate string yyyy-mm-dd
       * @param latParam Number (float)
       * @param lngParam Number (float)
       *
       * @returns {Promise}
       */
      endpointService.getEligibility = function (
        token, latParam, lngParam
      ) {
        // pull built service URL from ../core/serviceEndpoints.js

        var deferred = $q.defer();
        var serviceURL = serviceEndPoints.getFreeNightEligibilityUrl();
        // var serviceURL = './test/fnc.json';

        $http({
          method: 'post',
          url: serviceURL,
          data: {
            Location: {
              latitude: latParam,
              longitude: lngParam
            }
          },
          headers: {
            'Cache-Control': 'no-cache',
            LQ_AUTH_KEY: token,
            apikey: serviceEndPoints.getApiKey()
          }

        }).then(
          // Success
          function (response) {
            deferred.resolve(response);
          },
          // Error
          function (error) {
            console.error(JSON.stringify(error));
            deferred.reject(error);
          }
        );

        return deferred.promise;
      };


      endpointService.completeBooking = function (selectedRoom, userProfile, overridePhone) {
        // Fixed time for FNC call DE2605
        var now = new Date(); // GMT based
        var todayUTC = new Date(Date.UTC(now.getFullYear(), now.getMonth(), now.getDate())); // UTC
        var tomorrowUTC = new Date(
          Date.UTC(now.getFullYear(), now.getMonth(), now.getDate() + 1)
        ); // UTC

        var todayFormatted = todayUTC.toISOString().slice(0, 10).replace(/-/g, '-'); // toISOString is UTC based
        var tomorrowFormatted = tomorrowUTC.toISOString().slice(0, 10).replace(/-/g, '-'); // toISOString is UTC based

        var reservationRequest = {
          ReservationRequest: {
            channel: 'WEB',
            arrivalDate: todayFormatted,
            departureDate: tomorrowFormatted,
            iata: null,
            innNumber: {
              id: selectedRoom.innNumber.id
            },
            mainContact: {
              emailAddress: [{
                email: userProfile.emailAddress.trim(),
                emailDesignation: 'PERSONAL',
                primary: true,
                notifications: []
              }],
              firstName: userProfile.firstName.trim(),
              lastName: userProfile.lastName.trim(),
              phoneNumbers: [{
                phoneDesignation: 'MOBILE',
                phoneNumber: {
                  country: {
                    code: 'US',
                    description: 'United States'
                  },
                  number: (typeof overridePhone !== 'undefined' && overridePhone.length > 0) ? overridePhone : userProfile.mobilePhone
                },
                primary: true
              }],
              postalAddress: {
                city: userProfile.city,
                countryCode: userProfile.country,
                countryDisplay: '',
                county: '',
                postalCode: userProfile.zipCode,
                stateProvince: userProfile.state,
                street: userProfile.streetAddress1,
                street2: userProfile.streetAddress2
              },
              returnsNumber: {
                id: userProfile.returnsNumber
              }
            },
            paymentInfo: [{
              type: 'FreeNightCertificate',
              certificateNumber: {
                creationDate: todayFormatted,
                number: '1234567890',
                points: selectedRoom.nightlyRates[0].points,
                returnsNumber: {
                  id: userProfile.returnsNumber
                }
              }
            }],
            quotedTotalRate: selectedRoom.totalRate,
            quotedTotalRateWithTaxes: selectedRoom.totalRateWithTaxes,
            rateQualifier: selectedRoom.rateQualifier,
            roomReservations: [{
              numberOfAdults: 1,
              numberOfChildren: 0,
              numberOfRooms: 1,
              roomType: selectedRoom.roomType,
              nightlyRoomRates: selectedRoom.nightlyRates
            }],
            specialRequests: [''],
            language: {
              code: 'en'
            }
          }
        };
        var serviceURL = serviceEndPoints.getBookingConfirmationEndPoint();

        return $http({
          method: 'post',
          url: serviceURL,
          data: reservationRequest
        });
      };


      return endpointService;
    }
  ]
);
