/* global lqMobileModule, CQ*/
/* jshint maxparams: 11 */
lqMobileModule.factory(
  'freeNightModel',
  [
    '$q',
    '$filter',
    '$rootScope',
    'lodash',
    'storageModelService',
    'freeNightEndpointService',
    'geolocationService',
    'satelliteTrackingModelService',
    'persistentStorageModelService',
    function (
      $q,
      $filter,
      $rootScope,
      lodash,
      storageModelService,
      freeNightEndpointService,
      geolocationService,
      satelliteTrackingModelService,
      persistentStorageModelService
    ) {
      'use strict';

      var lastPermissionRequest = null;
      //
      // Initialize the data model and define public properties.
      var dataModel = {
        innNumber: null,
        hotelRoomRates: null,
        selectedRoom: null,
        built: false,
        activeCall: false
      };

      function updateLastPermissionRequest() {
        lastPermissionRequest = (new Date()).getTime();
        persistentStorageModelService.setValue('lastPermissionPrompt', lastPermissionRequest);
      }

      function canRequestPermissions() {
        var defer = $q.defer();
        var now = (new Date()).getTime();
        if (lastPermissionRequest === null) {
          persistentStorageModelService.getValue('lastPermissionPrompt').then(
            function (value) {
              lastPermissionRequest = value;
              defer.resolve(
                (now - lqmobileConfigs.declineLocationTimeout) > lastPermissionRequest
              );
            },
            function () {
              defer.resolve(true);
            }
          );
        } else {
          defer.resolve(
            (now - lqmobileConfigs.declineLocationTimeout) > lastPermissionRequest
          );
        }
        return defer.promise;
      }

      dataModel.checkEligibility = function () {
        var authToken = window.sessionStorage.getItem('token');
        canRequestPermissions().then(
          function (promptForLocation) {
            geolocationService.getCurrentPositionPromise(promptForLocation).then(
              function (result) {
                freeNightEndpointService.getEligibility(
                  authToken, result.coords.latitude, result.coords.longitude
                ).then(
                  function (response) {
                    dataModel.activeCall = false;
                    if (typeof response !== 'undefined' && response.status == 200) {
                      dataModel.build(response.data);
                      satelliteTrackingModelService.freeNightBannerShown(
                        response.data[0].hotelRoomRate.innNumber.id
                      );
                      return true;
                    }
                    return false;
                  },
                  function (error) {
                    if (error === false && promptForLocation) {
                      // if we attempted to ask for permissions, and we were declined
                      // update the last time we asked for permissions
                      updateLastPermissionRequest();
                    }
                    // mark as build, and move on
                    dataModel.built = true;
                  }
                );
              },
              function (error) {
                if (error === false && promptForLocation) {
                  // if we attempted to ask for permissions, and we were declined
                  // update the last time we asked for permissions
                  updateLastPermissionRequest();
                }
                // failed to get current position,
                dataModel.built = true;
              }
            );
          }
        // this promise does not reject
        );
      };

      dataModel.reset = function () {
        dataModel.innNumber = null;
        dataModel.hotelRoomRates = null;
        dataModel.selectedRoom = null;
        dataModel.built = false;
      };

      dataModel.build = function (response) {
        var i;
        var iMax;

        dataModel.innNumber = response[0].hotelRoomRate.innNumber.id;
        dataModel.hotelRoomRates = [];
        for (i = 0, iMax = response.length; i < iMax; i += 1) {
          dataModel.hotelRoomRates.push(response[i]);
        }
        dataModel.built = true;
      };

      dataModel.getAvailableRooms = function () {
        if (dataModel.isEligible()) {
          return dataModel.hotelRoomRates;
        }
        return null;
      };

      dataModel.isEligible = function () {
        return dataModel.built && dataModel.hotelRoomRates !== null
          && dataModel.hotelRoomRates.length > 0;
      };

    /**
     *
     * @param selectedRoom
     * @param overridePhoneNumber - the number to use for the booking
     * @returns promise -> bookingConfirmation
     */
      dataModel.completeBooking = function (selectedRoom, overridePhoneNumber) {
        var userProfile = storageModelService.getUserProfile();
        return freeNightEndpointService.completeBooking(
          selectedRoom, userProfile, overridePhoneNumber
        );
      };


      return dataModel;
    }
  ]);
