/* global lqMobileModule, google, document*/
lqMobileModule.factory('googleMapsEndpointService', ['$window', '$q', 'mapIcons',
  function ($window, $q, mapIcons) {
    'use strict';

    var endpointService = {},
      METERS_PER_MILE = 1609.34;

    function loadMapsAPI() {
      $window.initMapsAPI = function () {
        endpointService.deferredMapAPI.resolve();
      };
      var script = document.createElement('script');
      // updated to include client id
      script.src = 'https://maps.googleapis.com/maps/api/js?v=3&libraries=geometry,places&callback=initMapsAPI&client=' + GOOGLE_API_CLIENT_ID;
      script.type = 'text/javascript';
      document.body.appendChild(script);
    }

    endpointService.addAutocomplete = function () {
      return new google.maps.places.AutocompleteService();
    };

    endpointService.addGeocoder = function () {
      return new google.maps.Geocoder();
    };

    endpointService.geoLocateAddress = function (geocoder, address) {
      var deferred = $q.defer();

      geocoder.geocode({
        address: address
      }, function (results, status) {
        if (status === google.maps.GeocoderStatus.OK) {
          deferred.resolve(results[0]);
        } else {
          deferred.reject('Geocode was not successful for the following reason: ' + status);
        }
      });

      return deferred.promise;
    };

    endpointService.loadGoogleMapsAPI = function () {
      if (endpointService.deferredMapAPI === undefined) {
        endpointService.deferredMapAPI = $q.defer();
        loadMapsAPI();
      }
      return endpointService.deferredMapAPI.promise;
    };

    endpointService.getMap = function (mapDivId) {
      var lqMapOptions = {
        zoom: 3,
        center: {
          lat: 38.13,
          lng: -98.52
        },
        scaleControl: true,
        streetViewControl: false,
        mapTypeControl: false
      };

      var deferredMap = $q.defer();
      endpointService.loadGoogleMapsAPI().then(function () {
        deferredMap.resolve(new google.maps.Map(document.getElementById(mapDivId), lqMapOptions));
      });
      return deferredMap.promise;
    };

    endpointService.centerMap = function (map, lat, lng, zoom) {
      var center = new google.maps.LatLng(lat, lng);
      map.setCenter(center);

      if (zoom) {
        map.setZoom(zoom);
      }
    };

    endpointService.createGooglePlacePinMarker = function (
      map, latitude, longitude, address, marker
    ) {
      var googlePlacePinImage;

      if (marker === null) {
        googlePlacePinImage = {
          url: mapIcons.googlePlacePin,
          scaledSize: new google.maps.Size(18, 25),
          origin: new google.maps.Point(0, 0),
          anchor: new google.maps.Point(20, 20)
        };
        return new google.maps.Marker({
          position: { lat: latitude, lng: longitude },
          map: map,
          title: address,
          visible: true,
          draggable: false,
          zIndex: 1000000,
          optimized: false,
          icon: googlePlacePinImage
        });
      }
      marker.setOptions({ position: { lat: latitude, lng: longitude } });
      return marker;
    };

    // this function doesn't seem to do anything, but i'm nervious to remove it
    // so just disabling lint for the whole thing
    endpointService.addMarkerClustererToMap = function (googleMap, markerArray) { // eslint-disable-line
      var markerClusterer = new MarkerClusterer(  // eslint-disable-line
          googleMap,                              // eslint-disable-line
          markerArray,                            // eslint-disable-line
          { gridSize: 20, maxZoom: 8 }            // eslint-disable-line
      );                                          // eslint-disable-line
    };                                            // eslint-disable-line

    endpointService.createMarkerOnMap = function (
      googleMap, markerName, latitude, longitude, iconUrl
    ) {
      return new google.maps.Marker({
        position: { lat: latitude, lng: longitude },
        map: googleMap,
        title: markerName,
        visible: true,
        optimized: true,
        draggable: false,
        icon: iconUrl
      });
    };

    endpointService.getDistanceFrom2Coords = function (
      latitude1, longitude1, latitude2, longitude2
    ) {
      var deferred = $q.defer();

      endpointService.loadGoogleMapsAPI().then(function () {
        var position1 = new google.maps.LatLng(latitude1, longitude1);
        var position2 = new google.maps.LatLng(latitude2, longitude2);
        var distBetweenPoints =
          google.maps.geometry.spherical.computeDistanceBetween(position1, position2);
        var distance = Math.round((distBetweenPoints / METERS_PER_MILE) * 10) / 10;
        deferred.resolve(distance);
      });

      return deferred.promise;
    };

    return endpointService;
  }]);
