/* global lqMobileModule, google, lqMobile*/
lqMobileModule.factory('googleMapsModelService',
  ['$q', 'googleMapsEndpointService', 'searchParamModelService', 'mapIcons', 'hotelInformationModel',
    function (
      $q,
      googleMapsEndpointService,
      searchParamModelService,
      mapIcons,
      hotelInformationModel
    ) {
      var modelService = {};
      var mapZoom = lqMobile.Defaults.getDefaultMapZoom();
      var mapAttachDeferred = $q.defer();

      modelService.data = {
        map: null,
        mapCenter: {},
        mapZoom: null,
        markers: [],
        geolocatedUserMarker: null,
        currentlySelectedMarker: null,
        googlePlacePinMarker: null,
        pointOfInterestLocation: null,
        pointOfInterestCoordinates: {},
        compileContent: null,
        bounds: null
      };

      function waitForLoad() {
        var loadDeferred = $q.defer();

        google.maps.event.addListenerOnce(modelService.data.map, 'tilesloaded', function () {
          loadDeferred.resolve();
        });

        return loadDeferred.promise;
      }

      function addMarkerListener(marker, hotel, markerListener) {
        google.maps.event.addListener(marker, 'click', function () {
          markerListener(hotel);
        });
      }

      function generateMarkersFromSummaryData(markerListener) {
        var summariesArray = hotelInformationModel.data.hotelSummaries;
        var summariesLenght = summariesArray.length;
        var icon;
        var index;
        var summaryObj;
        var marker;

        for (index = 0; index < summariesLenght; index += 1) {
          summaryObj = summariesArray[index];
          icon = mapIcons.hotelNotSelected;
          marker = googleMapsEndpointService.createMarkerOnMap(
            modelService.data.map,
            summaryObj.hotelId,
            parseFloat(summaryObj.geoLocation.latitude),
            parseFloat(summaryObj.geoLocation.longitude),
            icon
          );
          marker.availability = true;
          marker.hotel_id = summaryObj.hotelId;
          addMarkerListener(marker, summaryObj, markerListener);
          modelService.data.markers.push(marker);
        }
      }

      function isBoundsEmpty() {
        return !modelService.data.bounds || modelService.data.bounds.isEmpty();
      }

      function addInfoDismissClickListenerToMap(dismissListener) {
        google.maps.event.addListener(modelService.data.map, 'click', dismissListener);
      }

      function addInfoDismissDragEndListenerToMap(dismissListener) {
        google.maps.event.addListener(modelService.data.map, 'dragend', dismissListener);
      }

      function findMarkerFromHotelId(hotelId) {
        var i;
        // changing for in loop might have other effects
        for (i in modelService.data.markers) { // eslint-disable-line
          if (modelService.data.markers[i].hotel_id === hotelId) {
            return modelService.data.markers[i];
          }
        }
        return null;
      }

      function createGooglePlacePinMarker(latitude, longitude, address) {
        modelService.data.googlePlacePinMarker =
          googleMapsEndpointService.createGooglePlacePinMarker(
            modelService.data.map, latitude, longitude, address,
            modelService.data.googlePlacePinMarker
          );
      }

      function removeGooglePlacePinMarker() {
        if (modelService.data.googlePlacePinMarker) {
          modelService.data.googlePlacePinMarker.setMap(null);
          modelService.data.googlePlacePinMarker = null;
        }
      }

      modelService.attachMap = function (mapDivId, markerListener, dismissListener) {
        var mapPromise = googleMapsEndpointService.getMap(mapDivId);

        $q.all([mapPromise, hotelInformationModel.data.hotelSummariesPromise]).then(
          function (results) {
            modelService.data.map = results[0];

            if (modelService.data.markers.length > 1) {
              modelService.data.markers = LQMobile.Util.clearArray(modelService.data.markers);
            }

            generateMarkersFromSummaryData(markerListener);
            googleMapsEndpointService.addMarkerClustererToMap(
              modelService.data.map, modelService.data.markers
            );

            waitForLoad().then(function () {
              if (!isBoundsEmpty()) {
                modelService.data.map.fitBounds(modelService.data.bounds);
              }

              addInfoDismissClickListenerToMap(dismissListener);
              addInfoDismissDragEndListenerToMap(dismissListener);

              mapAttachDeferred.resolve();
            });
          }
        );

        return mapAttachDeferred.promise;
      };

      modelService.selectMarker = function (hotelSummary) {
        var marker = findMarkerFromHotelId(hotelSummary.innNumber.id);
        modelService.dismissSelectedMarker();
        modelService.data.currentlySelectedMarker = marker;
        modelService.centerMap(hotelSummary.location.latitude, hotelSummary.location.longitude);
        marker.setIcon(mapIcons.hotelSelected);
      };

      modelService.refreshMap = function () {
        if (modelService.data.map) {
          google.maps.event.trigger(modelService.data.map, 'resize');
          modelService.zoom();
        }
      };

      modelService.zoom = function () {
        if (modelService.data.map) {
          fireFocusEvent = false;
          firePanEvent = false;
          modelService.dismissSelectedMarker();
          removeGooglePlacePinMarker();


          modelService.data.pointOfInterestCoordinates = {
            lat: searchParamModelService.data.latitude,
            lng: searchParamModelService.data.longitude
          };

          modelService.centerMap(
            searchParamModelService.data.latitude, searchParamModelService.data.longitude, mapZoom
          );

          createGooglePlacePinMarker(
            searchParamModelService.data.latitude, searchParamModelService.data.longitude,
            searchParamModelService.data.currentPlace
          );
        }
      };

      modelService.dismissSelectedMarker = function () {
        if (modelService.data.currentlySelectedMarker) {
          modelService.data.currentlySelectedMarker.setIcon(mapIcons.hotelNotSelected);
          modelService.data.currentlySelectedMarker = null;
        }
      };

      modelService.centerMap = function (lat, lng, zoom) {
        modelService.data.mapCenter = {
          lat: lat,
          lng: lng,
          zoom: zoom
        };

        if (modelService.data.map) {
          googleMapsEndpointService.centerMap(modelService.data.map, lat, lng, zoom);
          google.maps.event.trigger(modelService.data.map, 'resize');
        }
      };

      return modelService;
    }]);
