/* global lqMobileModule*/
/* jshint maxparams: 10 */
lqMobileModule.controller('hotelRoomsController',
  ['$scope',
    '$modal',
    '$rootScope',
    'hotelRoomsModel',
    'searchParamModelService',
    'routingService',
    'returnsUserProfileModel',
    'hotelInformationModel',
    'events',
    'satelliteTrackingModelService',
    'currencyService',
    function (
      $scope,
      $modal,
      $rootScope,
      hotelRoomsModel,
      searchParamModelService,
      routingService,
      returnsUserProfileModel,
      hotelInformationModel,
      events,
      satelliteTrackingModelService,
      currencyService
    ) {
      'use strict';

      function handleRouting(isInstantHoldActive, isBookWithPointsSelected) {
        if (isBookWithPointsSelected || !isInstantHoldActive) {
          routingService.goToBookingRecap();
        } else if (isInstantHoldActive) {
          routingService.goToInstantHold();
        }
      }

      $scope.hotelRooms = hotelRoomsModel.data;
      $scope.searchParams = searchParamModelService.data;
      $scope.userLogged = returnsUserProfileModel.isLoggedIn();
      $scope.hotelInfo = hotelInformationModel.data.hotelDetails;

      // if the active currency is null, get the currency from hotelInfo
      if (currencyService.getActiveCurrency() !== null) {
        $scope.currentCurrency = currencyService.getActiveCurrency();
      } else {
        $scope.currentCurrency = hotelInformationModel.data.hotelDetails.currencyCode;
      }
      
      $scope.allCurrencies = currencyService.getAllCurrencies();
      $scope.status = { isopen: false };

      $scope.count = 0;

      $scope.open = function (room, isInstantHoldActive) {
        var modalInstance = $modal.open({
          templateUrl: 'bookRoomModalContent.html',
          controller: 'bookRoomModalInstanceCtrl',
          windowClass: 'app-modal-window'
        });

        modalInstance.result.then(function (isBookWithPointsSelected) {
          if (isBookWithPointsSelected) {
            searchParamModelService.setSelectedRoom(
              $scope.hotelRooms.fncRooms[room.hotelRoomRate.roomType.roomCode], true
            );
          } else {
            searchParamModelService.setSelectedRoom(room, false);
          }
          handleRouting(isInstantHoldActive, isBookWithPointsSelected);
        });
      };

      $scope.openHotelDetailsModal = function (room, isInstantHoldActive) {
        var modalInstance = $modal.open({
          templateUrl: 'roomDetailsModalContent.html',
          controller: 'roomDetailsModalInstanceCtrl',
          windowClass: 'app-modal-window',
          resolve: {
            room: function () {
              return room;
            }
          }
        });

        satelliteTrackingModelService.roomDetailsLinkSelected();

        modalInstance.result.then(function (iRoom) {
          $scope.selectRoom(iRoom, isInstantHoldActive);
        });
      };

      $scope.selectRoom = function (room, isInstantHoldActive) {
        if ($scope.hotelRooms.fncRooms[room.hotelRoomRate.roomType.roomCode]) {
          $scope.open(room, isInstantHoldActive);
        } else {
          searchParamModelService.setSelectedRoom(room, false);
          handleRouting(isInstantHoldActive, false);
        }
      };

      $scope.changeCurrency = function (currencyCode) {
        // update currency
        currencyService.setActiveCurrency(currencyCode);
        $scope.currentCurrency = currencyService.getActiveCurrency();

        // close dropdown
        $scope.status.isopen = !$scope.status.isopen;

        // update rooms available
        hotelRoomsModel.getAvailableRooms();
      };

      // get initial available rooms
      hotelRoomsModel.getAvailableRooms();
    }]);

lqMobileModule.controller('bookRoomModalInstanceCtrl', ['$scope', '$modalInstance', function ($scope, $modalInstance) {
  'use strict';

  $scope.usePoints = function () {
    $modalInstance.close(true);
  };

  $scope.useNormalRate = function () {
    $modalInstance.close(false);
  };
}]);

lqMobileModule.controller('roomDetailsModalInstanceCtrl', ['$scope', 'hotelRoomsModel', '$modalInstance', 'searchParamModelService', 'routingService', 'hotelInformationModel', 'room',
  function (
    $scope,
    hotelRoomsModel,
    $modalInstance,
    searchParamModelService,
    routingService,
    hotelInformationModel,
    room
  ) {
    'use strict';

    function getFeaturesByRoomType(roomCode, rooms) {
      var features;
      var size = rooms.length;
      var i;
      for (i = 0; i < size; i += 1) {
        if (rooms[i].roomCode === roomCode) {
          features = rooms[i].roomFeatures;
        }
      }
      return features;
    }

    $scope.hotelInfo = hotelInformationModel.data.hotelDetails;
    $scope.selectedRoom = room;
    $scope.features = getFeaturesByRoomType(
      room.hotelRoomRate.roomType.roomCode, hotelInformationModel.data.hotelDetails.roomsList
    );
    $scope.hotelRooms = hotelRoomsModel.data;

    $scope.cancel = function () {
      $modalInstance.dismiss('cancel');
    };

    $scope.selectRoom = function () {
      $modalInstance.close($scope.selectedRoom);
    };
  }]);
