/* global lqMobileModule, LQMobile*/
lqMobileModule.factory('hotelRoomsEndpointService', ['$window', '$q', '$http', 'serviceEndPoints',
  function ($window, $q, $http, serviceEndPoints) {
    'use strict';

    var endpointService = {};

    endpointService.getAvailableRooms = function (
      searchParams, hotelId, currencyCode, isUserLoggedIn, returnsData, isInstantHoldOptional
    ) {
      var deferred = $q.defer();
      var defaultRate = LQMobile.Util.getDefaultRate();
      var rateCodes;
      var jsonObj;
      var serviceURL;

      rateCodes = [{
        code: defaultRate,
        type: LQMobile.Util.getRateCode(defaultRate)
      }];
      if (isInstantHoldOptional) {
        rateCodes.push({
          code: LQMobile.Util.getInstantHoldRateCode(),
          type: 'INSTANT_HOLD'
        });
      }

      if (searchParams.rateType === 'CORPORATE') {
        rateCodes.push({
          code: searchParams.promoCode,
          type: 'CORPORATE'
        });
      } else if (searchParams.rateType !== defaultRate) {
        rateCodes.push({
          code: searchParams.rateType,
          type: LQMobile.Util.getRateCode(searchParams.rateType)
        });
      }

      jsonObj = {
        HotelAvailabilityRequest: {
          arrivalDate: LQMobile.Util.dateToString(searchParams.checkinDate),
          departureDate: LQMobile.Util.dateToString(searchParams.checkoutDate),
          numberOfAdults: searchParams.numberOfAdults,
          numberOfChildren: searchParams.numberOfChildren,
          numberOfRooms: searchParams.numberOfRooms,
          currencyCode: currencyCode,
          rateCodes: rateCodes,
          propertyIds: [{
            id: hotelId
          }]
        }
      };

      serviceURL = serviceEndPoints.getHotelRoomAvailabilityEndPoint();
      if (isUserLoggedIn) {
        serviceURL = serviceURL + '&returnsNumber=' + returnsData.userProfile.returnsNumber;
      }

      $http({
        method: 'post',
        url: serviceURL,
        data: jsonObj
      }).success(function (response) {
        deferred.resolve(response);
      }).error(function (data) {
        deferred.reject(data);
      });
      return deferred.promise;
    };

    return endpointService;
  }]);
