/* global lqMobileModule, LQMobile, CQ*/
lqMobileModule.factory('hotelRoomsModel',
  [
    '$q',
    '$rootScope',
    'hotelRoomsEndpointService',
    'searchParamModelService',
    'hotelInformationModel',
    'returnsUserProfileModel',
    'events',
    'reservationsModel',
    'satelliteTrackingModelService',
    'currencyService',
    function (
      $q,
      $rootScope,
      hotelRoomsEndpointService,
      searchParamModelService,
      hotelInformationModel,
      returnsUserProfileModel,
      events,
      reservationsModel,
      satelliteTrackingModelService,
      currencyService
    ) {
      'use strict';

      var dataModel = {
        data: {
          rooms: [],
          filteredRooms: [],
          fncRooms: [],
          instantHoldRooms: [],
          loadingRooms: false,
          error: ''
        }
      };

      function handleError(error) {
        satelliteTrackingModelService.hotelRoomCompleted(
          true, hotelInformationModel.data.hotelDetails.hotelTitle,
          hotelInformationModel.data.hotelDetails.hotelId
        );
        dataModel.data.error = CQ.I18n.getMessage(error.trim());
        dataModel.data.rooms = [];
        dataModel.data.filteredRooms = [];
        dataModel.data.fncRooms = [];
        dataModel.data.instantHoldRooms = [];
        dataModel.data.loadingRooms = false;
      }

      function getMinRate(roomObj) {
        var rates = roomObj.hotelRoomRate.nightlyRates;
        var ratesArrayLength = roomObj.hotelRoomRate.nightlyRates.length;
        var minRate = 999999;
        var minRateObj = {};
        var totalRate = 0;
        var totalPoints = 0;
        var pos;
        var rateObj;
        var result;

        for (pos = 0; pos < ratesArrayLength; pos += 1) {
          rateObj = rates[pos];
          if (rateObj.hasOwnProperty('nightlyRate')) {
            if ((rateObj.nightlyRate.amount < minRate) ||
              (rateObj.nightlyRate.amount === minRate && rateObj.points < minRateObj.points)) {
              minRateObj = rateObj;
              minRate = rateObj.nightlyRate.amount;
            }
            totalRate += rateObj.nightlyRate.amount;
            totalPoints += rateObj.points;
          }
        }
        result = {
          lowestRate: minRateObj,
          totalRate: totalRate,
          totalPoints: totalPoints
        };
        return result;
      }

      function isLQHoldAvailable(code, currentPos, array, arrayLength) {
        var pos;
        var roomObj;
        var roomCode;
        var rateCode;
        for (pos = parseInt(currentPos, 10) + 1; pos < arrayLength; pos += 1) {
          roomObj = array[pos];
          roomCode = roomObj.hotelRoomRate.roomType.roomCode;
          rateCode = roomObj.hotelRoomRate.rateQualifier.rateCode.code;
          if (code === roomCode && LQMobile.Util.getInstantHoldRateCode() === rateCode) {
            return true;
          }
        }
        return false;
      }


      dataModel.getAvailableRooms = function () {
        var instantHoldRateCode = LQMobile.Util.getInstantHoldRateCode();
        var isUserLoggedIn = returnsUserProfileModel.isLoggedIn();
        var isInstantHoldOptional = !reservationsModel.data.selectedFullReservation;
        var activeCurrency = currencyService.getActiveCurrency();

        dataModel.data.loadingRooms = true;
        dataModel.data.rooms = [];
        dataModel.data.filteredRooms = [];
        dataModel.data.fncRooms = [];
        dataModel.data.error = '';

        hotelRoomsEndpointService.getAvailableRooms(
          searchParamModelService.data, hotelInformationModel.data.hotelDetails.hotelId,
          activeCurrency,
          isUserLoggedIn, returnsUserProfileModel.data, isInstantHoldOptional
        ).then(function (rooms) {
          var availableRooms = [];
          var roomsLength = rooms.length;
          var instantHoldRooms = [];
          var filteredRooms = [];
          var fncRooms = [];
          var index;
          var roomObj;
          var roomCode;
          var rateCode;

          // changing for in could have side effects, disabled lint
          for (index in rooms) { // eslint-disable-line
            try {
              roomObj = rooms[index];
              if (roomObj.hotelRoomRate.rateStatus === 'OPEN') {
                roomCode = roomObj.hotelRoomRate.roomType.roomCode;
                rateCode = roomObj.hotelRoomRate.rateQualifier.rateCode.code;
                if (instantHoldRateCode === rateCode) {
                  instantHoldRooms[roomCode] = true;
                  roomObj.computedRates = getMinRate(roomObj);
                  availableRooms.push(roomObj);
                } else if (LQMobile.Util.getDefaultRate() === rateCode) {
                  if (
                    !instantHoldRooms[roomCode]
                      && !isLQHoldAvailable(roomCode, index, rooms, roomsLength)
                  ) {
                    roomObj.computedRates = getMinRate(roomObj);
                    availableRooms.push(roomObj);
                  }
                } else if (rateCode.indexOf('FNC') !== -1) {
                  roomObj.computedRates = getMinRate(roomObj);
                  fncRooms[roomCode] = roomObj;
                } else {
                  roomObj.computedRates = getMinRate(roomObj);
                  filteredRooms.push(roomObj);
                }
              }
            } catch (err) {}
          }
          if (availableRooms.length === 0 && filteredRooms.length === 0 && fncRooms.length === 0) {
            dataModel.data.error = 'No rooms are available at this hotel for the requested check-in and check-out dates.';
          } else if (searchParamModelService.data.rateType === 'CORPORATE' && filteredRooms.length === 0) {
            dataModel.data.error = 'The promo/corporate rate code you requested is not available, choose from our Best Available Rates below.';
          } else if (searchParamModelService.data.rateType !== 'RAC' && filteredRooms.length === 0) {
            dataModel.data.error = 'The rate type you requested is not available, choose from our Best Available Rates below.';
          }
          dataModel.data.rooms = availableRooms;
          dataModel.data.filteredRooms = filteredRooms;
          dataModel.data.instantHoldRooms = instantHoldRooms;
          dataModel.data.fncRooms = fncRooms;
          dataModel.data.loadingRooms = false;

          satelliteTrackingModelService.hotelRoomCompleted(
            false, hotelInformationModel.data.hotelDetails.hotelTitle,
            hotelInformationModel.data.hotelDetails.hotelId
          );
        }, handleError);
      };

      return dataModel;
    }]);
