/* global lqMobileModule, $, LQMobile, CQ*/
/* jshint maxparams: 10 */
lqMobileModule.controller('hotelSearchConfigController', ['$scope', '$q', '$rootScope', 'searchParamModelService', 'hotelSearchModelService', 'editSearchCollapseFactory', 'editSearchPanelCollapseFactory', 'editSearchAlertFactory', '$location', 'events', 'reservationsModel', 'satelliteTrackingModelService',
  function (
    $scope,
    $q,
    $rootScope,
    searchParamModelService,
    hotelSearchModelService,
    editSearchCollapseFactory,
    editSearchPanelCollapseFactory,
    editSearchAlertFactory,
    $location,
    events,
    reservationsModel,
    satelliteTrackingModelService
  ) {
    'use strict';

    var originalSearchEdit = {};
    var maxNumberOfGuests = LQMobile.Util.getMaxAllowedAdults();

    $scope.specialRate = {
      rateType: '',
      pointRight: true,
      showCodeField: false
    };

    $scope.status = {
      isopen: false
    };

    $scope.searchData = searchParamModelService.data;

    $scope.isReservationSelected =
      reservationsModel.data.selectedFullReservation != undefined
        && reservationsModel.data.selectedFullReservation != null;

    $scope.$on('$routeUpdate', function () {
      if ($location.search().editSearch) {
        if (editSearchPanelCollapseFactory.isCollapsed) {
          editSearchPanelCollapseFactory.toggleSearchEditPanel('.hotelSearchResults');
        }
      } else if (!editSearchPanelCollapseFactory.isCollapsed) {
        editSearchPanelCollapseFactory.toggleSearchEditPanel('.hotelSearchResults');
      }
    });

    $scope.toggleEditDatesPanel = function () {
      $rootScope.$broadcast('showCalendar');
      editSearchCollapseFactory.toggleCollapsible();
      $location.search('editDates', true);
    };

    $scope.today = new Date();
    $scope.tomorrow = new Date(new Date().getTime() + (24 * 60 * 60 * 1000));

    function undoChanges() {
            // Undo checkin/out dates, adults, miles and special rates
      $scope.searchData.checkinDate = originalSearchEdit.originalCheckinDate;
      $scope.searchData.checkoutDate = originalSearchEdit.originalCheckoutDate;
      $scope.searchData.numberOfAdults = originalSearchEdit.originalNumberOfAdults;
      $scope.searchData.range = originalSearchEdit.originalRange;
      $scope.searchData.rateType = originalSearchEdit.originalRateType;
      $scope.specialRate.rateType = originalSearchEdit.rateType;
      $scope.specialRate.pointRight = originalSearchEdit.pointRight;
      $scope.specialRate.showCodeField = originalSearchEdit.showCodeField;
      $scope.searchData.promoCode = originalSearchEdit.originalPromoCode;
      $scope.status.isopen = originalSearchEdit.originalOpenStatus;
      $scope.searchData.currentPlace = originalSearchEdit.originalCurrentPlace;
    }

    $scope.toggleSearchEditPanel = function () {
      if (!editSearchPanelCollapseFactory.isCollapsed) {
        undoChanges();
      }
      editSearchPanelCollapseFactory.toggleSearchEditPanel();
      editSearchAlertFactory.clearAlerts();
      editSearchAlertFactory.clearPersistentAlerts();
      if (
        !editSearchPanelCollapseFactory.isCollapsed
        && reservationsModel.data.selectedFullReservation
      ) {
        LQMobile.Util.changeCssAttr('.hotelSearchConfig .omnibox-inner-container', 'display', 'none');
        editSearchAlertFactory.addPersistentAlert('danger', CQ.I18n.getMessage('If you intend to switch hotels, please cancel your existing reservation and access the “Home” option from the menu above to start a new booking to avoid duplicate reservations.'));
      }
    };

    $scope.setSpecialRate = function (e, p, key) {
      satelliteTrackingModelService.rateTypeChange(key);
      $scope.specialRate.rateType = e.currentTarget.innerText;
      $scope.specialRate.pointRight = p;
      $scope.specialRate.showCodeField = !p;
      $scope.searchData.rateType = key;
      editSearchAlertFactory.clearAlerts();
      e.preventDefault();
      e.stopPropagation();
      $scope.status.isopen = !$scope.status.isopen;
      if (p) {
        $scope.searchData.promoCode = '';
      }
    };


        // Keep a copy of search values in case the search panel is closed
    function setOriginalValues() {
      originalSearchEdit.originalCurrentPlace = $scope.searchData.currentPlace;
      originalSearchEdit.originalCheckinDate = $scope.searchData.checkinDate;
      originalSearchEdit.originalCheckoutDate = $scope.searchData.checkoutDate;
      originalSearchEdit.originalNumberOfAdults = $scope.searchData.numberOfAdults;
      originalSearchEdit.originalRange = $scope.searchData.range;
      originalSearchEdit.originalRateType = $scope.searchData.rateType;
      originalSearchEdit.rateType = $scope.specialRate.rateType;
      originalSearchEdit.pointRight = $scope.specialRate.pointRight;
      originalSearchEdit.showCodeField = $scope.specialRate.showCodeField;
      originalSearchEdit.originalPromoCode = $scope.searchData.promoCode;
      originalSearchEdit.originalOpenStatus = $scope.status.isopen;
    }

    $scope.init = function () {
      var searchD = searchParamModelService.data;

      $scope.searchData = searchD;
      $scope.editSearchCollapseFactory = editSearchCollapseFactory;
      $scope.editSearchPanelCollapseFactory = editSearchPanelCollapseFactory;
      $scope.editSearchAlertFactory = editSearchAlertFactory;

      $scope.adultPlural = $scope.searchData.numberOfAdults > 1;


      if ((typeof searchD.rateType !== 'undefined') && (searchD.rateType !== null) && (searchD.rateType !== '')) {
        if (searchD.rateType === 'CORPORATE') {
          $scope.specialRate.showCodeField = true;
          $scope.specialRate.promoCode = searchD.promoCode;
          $scope.searchData.promoCode = searchD.promoCode;
        }
        $scope.searchData.rateType = searchD.rateType;
        $scope.specialRate.rateType = CQ.I18n.getMessage($('#' + searchD.rateType).text());
      } else {
        $scope.specialRate.rateType = LQMobile.Util.getFirstItemText('.hotelSearchConfig .special-rates-list li');
        $scope.searchData.rateType = $('.hotelSearchConfig .special-rates-list li').first().attr('id');
        $scope.specialRate.showCodeField = false;
      }

      setOriginalValues();
    };

    $scope.increaseAdults = function (amount) {
            // $rootScope.$broadcast(events.noOfAdultsChange);
      satelliteTrackingModelService.noOfAdultsChange();
      if ($scope.searchData.numberOfAdults === 1) {
        $scope.adultPlural = true;
      }
      if ($scope.searchData.numberOfAdults < maxNumberOfGuests) {
        $scope.searchData.numberOfAdults += amount;
      }
    };

    $scope.decreaseAdults = function (amount) {
      satelliteTrackingModelService.noOfAdultsChange();
      if ($scope.searchData.numberOfAdults === 2) {
        $scope.adultPlural = false;
      }
      if ($scope.searchData.numberOfAdults > 1) {
        $scope.searchData.numberOfAdults -= amount;
      }
    };

    $scope.increaseRange = function (amount) {
            // $rootScope.$broadcast(events.searchRadiusChange);
      satelliteTrackingModelService.searchRadiusChange();
      if ($scope.searchData.range < 100) {
        $scope.searchData.range += amount;
      }
    };

    $scope.decreaseRange = function (amount) {
            // $rootScope.$broadcast(events.searchRadiusChange);
      satelliteTrackingModelService.searchRadiusChange();
      if ($scope.searchData.range > 10) {
        $scope.searchData.range -= amount;
      }
    };

    $scope.findHotels = function (searchData) {
      var valid = false;

      console.log('finding hotels');
      if (searchData.rateType === 'CORPORATE') {
        if ($scope.searchData.promoCode === '') {
          editSearchAlertFactory.clearAlerts();
          editSearchAlertFactory.addAlert('danger', 'Please enter a valid Promo/Corp Code or select an alternate rate type to continue.');
        } else {
          editSearchAlertFactory.clearAlerts();
          valid = true;
        }
      } else if (
        $scope.searchData.searchType !== 'location'
        && (typeof $scope.searchData.currentPlace === 'undefined'
            || $scope.searchData.currentPlace === null
            || $scope.searchData.currentPlace === '')
      ) {
        editSearchAlertFactory.addAlert('danger', 'Please enter a valid Destination.');
      } else if (searchData.rateType !== '') {
        valid = true;
        editSearchAlertFactory.clearAlerts();
      }

      if (valid) {
        console.log('valid search');
        console.log(searchData);

        editSearchAlertFactory.clearPersistentAlerts();
        searchParamModelService.updateSearchData(searchData);
        editSearchPanelCollapseFactory.toggleSearchEditPanel();
        $scope.doFindHotels();
      }
      satelliteTrackingModelService.findHotelsButtonClicked();
    };

    $scope.iconCalendarSelected = function () {
      satelliteTrackingModelService.calendarIconClick();
    };
  }]);
