/*jslint nomen: true */
/*global lqMobileModule, CQ, satellite*/
/*jshint maxparams: 11 */
lqMobileModule.factory('hotelSearchModelService', ['$rootScope',
  '$q',
  '$filter',
  'events',
  'hotelSearchEndpointService',
  'searchParamModelService',
  'googleMapsEndpointService',
  'editSearchPanelCollapseFactory',
  'editSearchAlertFactory',
  'HotelAvailabilityRequest',
  'satelliteTrackingModelService',
  'dateFormatterService',
  function (
    $rootScope,
    $q,
    $filter,
    events,
    hotelSearchEndpointService,
    searchParamModelService,
    googleMapsEndpointService,
    editSearchPanelCollapseFactory,
    editSearchAlertFactory,
    HotelAvailabilityRequest,
    satelliteTrackingModelService,
    dateFormatterService
  ) {
    'use strict';

    var modelService = {};
    var initDeferred;

    modelService.data = {
      initPromise: null,
      geocoder: null,
      hotels: null,
      hotelsLoaded: []
    };

    function getFormattedHotelData(hotelObj) {
      var result = hotelObj;

      result.hotelSummary.hotelType = $filter('hotelName')(hotelObj.hotelSummary.name, 0);
      result.hotelSummary.hotelTitle = $filter('hotelName')(hotelObj.hotelSummary.name, 1);
      result.hotelSummary.hotelDistanceFromPOI = $filter('distance')(hotelObj.range.range, 2, ',', '.', hotelObj.hotelSummary.address.countryCode);
      result.hotelSummary.fullHotelAddress = hotelObj.hotelSummary.address.street + ', ' + hotelObj.hotelSummary.address.city + ', ' + hotelObj.hotelSummary.address.stateProvince + ', ' + hotelObj.hotelSummary.address.countryCode;
    // Check if there is a dedicated mobile-web phone
      if (result.hotelSummary.phoneNumbers.MOBILE_WEB) {
        result.hotelSummary.fullHotelPhone = hotelObj.hotelSummary.primaryPhoneNumber.countryCode + '-' + result.hotelSummary.phoneNumbers.MOBILE_WEB.phoneNumber.number;
      } else {
        result.hotelSummary.fullHotelPhone = hotelObj.hotelSummary.primaryPhoneNumber.countryCode + '-' + hotelObj.hotelSummary.primaryPhoneNumber.areaCode + '-' + hotelObj.hotelSummary.primaryPhoneNumber.number;
      }
      result.hotelSummary.hotelLocation = hotelObj.hotelSummary.address.city + ', ' + hotelObj.hotelSummary.address.stateProvinceDisplay;
      if (hotelObj.hotelRate.availabilityStatus !== 'ERROR') {
        result.hotelSummary.minRateFormatted = $filter('lqcurrency')(hotelObj.hotelRate.minRate.amount, '$', 2, ',', '.');
        result.hotelSummary.minRateCurrencyCode = hotelObj.hotelRate.minRate.currencyCode;
        result.hotelSummary.loyaltyPointsFormatted = $filter('number')(hotelObj.hotelSummary.loyaltyPointsTier.points, 0);
      }

      return result;
    }

    modelService.loadHotelInfo = function (hotel) {
      var availabilityRequest = HotelAvailabilityRequest.buildRequestById(
        searchParamModelService.data, hotel.hotelId
      );

      return hotelSearchEndpointService.getAvailableHotels(availabilityRequest).then(
        function (response) {
          return googleMapsEndpointService.getDistanceFrom2Coords(
            hotel.geoLocation.latitude, hotel.geoLocation.longitude,
            searchParamModelService.data.latitude, searchParamModelService.data.longitude
          ).then(function (range) {
            var hotelObj = getFormattedHotelData(response.data[0]);
            response.data[0].range.range = range;
            modelService.data.hotelsLoaded[hotelObj.hotelRate.innNumber.id] = hotelObj;
          }
          );
        }, function (error) {
        throw error;
      });
    };

    modelService.getHotelsList = function (location) {
      var hotelCounter = 0;
      var availabilityRequest = HotelAvailabilityRequest.buildRequestByLocation(
        searchParamModelService.data, location
      );
      var oneDay = 1000 * 60 * 60 * 24;
      var now = new Date();
      var checkInTime = searchParamModelService.data.checkinDate.getTime();
      var checkoutTime = searchParamModelService.data.checkoutDate.getTime();
      var checkInCheckOutDiffMS = checkoutTime - checkInTime;
      var nowCheckInDiffMS = checkInTime - now.getTime();
      var dateDiffInDays = Math.round(checkInCheckOutDiffMS / oneDay);
      var daysUntilCheckIn = Math.round(nowCheckInDiffMS / oneDay);
      var daysToArrival = daysUntilCheckIn;
      var checkinDate = searchParamModelService.data.checkinDate;
      var checkoutDate = searchParamModelService.data.checkoutDate;
      var numberOfAdults = searchParamModelService.data.numberOfAdults;
      var numberOfChildren = searchParamModelService.data.numberOfChildren;
      var numberOfGuests = numberOfAdults + numberOfChildren;
      var numberOfNights = dateDiffInDays == 0 ? 1 : dateDiffInDays;
      var promoCode = searchParamModelService.data.promoCode;
      var rateType = searchParamModelService.data.rateType;
      var destinationPlace = searchParamModelService.data.currentPlace;
      var searchType = searchParamModelService.data.searchType;

      if (daysUntilCheckIn < 1) {
        daysToArrival = 0;
      }

      modelService.data.hotels = [];
      modelService.data.hotelsAvailable = false;

      return hotelSearchEndpointService.getAvailableHotels(availabilityRequest).then(
        function (response) {
          var hotels = response.data;
          var filteredHotels = [];
          var index;
          var hotelObj;
          // changing for in could have side effects
          for (index in hotels) { // eslint-disable-line
            try {
              hotelObj = getFormattedHotelData(hotels[index]);
              if (hotelObj.hotelRate.availabilityStatus !== 'ERROR') {
                modelService.data.hotelsAvailable = true;
                hotelCounter += 1;
              }

              filteredHotels.push(hotelObj);
              modelService.data.hotelsLoaded[hotelObj.hotelRate.innNumber.id] = hotelObj;
            } catch (err) {
              // do nothing
            }
          }
          modelService.data.hotels = hotels;
          satelliteTrackingModelService.hotelSearchCompleted(
            hotelCounter, false, numberOfNights, hotelCounter,
            dateFormatterService.getFormattedDate(checkinDate),
            dateFormatterService.getFormattedDate(checkoutDate),
            numberOfAdults, numberOfChildren, numberOfGuests,
            daysToArrival, promoCode, rateType, destinationPlace,
            searchType
          );
        }, function (error) {
        modelService.data.hotels = [];
        satelliteTrackingModelService.hotelSearchCompleted(
          hotelCounter, true, numberOfNights, hotelCounter,
          dateFormatterService.getFormattedDate(checkinDate),
          dateFormatterService.getFormattedDate(checkoutDate),
          numberOfAdults, numberOfChildren, numberOfGuests,
          daysToArrival, promoCode, rateType, destinationPlace, searchType
        );
        throw error;
      });
    };

    modelService.getHotelAvailability = function (hotelId) {
      var availabilityRequest = HotelAvailabilityRequest.buildRequestById(
        searchParamModelService.data, hotelId
      );

      hotelSearchEndpointService.getAvailableHotels(availabilityRequest).then(
        function () {
          // do nothing
        }, function (error) {
        editSearchPanelCollapseFactory.toggleSearchEditPanel('.hotelSearchResults');
        modelService.data.hotels = [];
        resultsLoaded.resolve();
        editSearchAlertFactory.addAlert('danger', CQ.I18n.getMessage(error.data.trim()));
      });
    };

    modelService.geolocateAddress = function (address) {
      var deferred = $q.defer();
      modelService.data.initPromise.then(function () {
        googleMapsEndpointService.geoLocateAddress(
          modelService.data.geocoder, address
        ).then(function (geolocatedAddressObject) {
          deferred.resolve(geolocatedAddressObject);
        });
      });
      return deferred.promise;
    };

    modelService.clearHotels = function () {
      modelService.data.hotels = {};
    };

    initDeferred = $q.defer();

    modelService.data.initPromise = initDeferred.promise;
    googleMapsEndpointService.loadGoogleMapsAPI().then(function () {
      modelService.data.geocoder = googleMapsEndpointService.addGeocoder();
      initDeferred.resolve();
    });

    return modelService;
  }]);
