/* global lqMobileModule, alert*/
/* jshint maxparams: 12 */
lqMobileModule.controller('instantHoldController', ['$scope',
  '$rootScope',
  'hotelInformationModel',
  'searchParamModelService',
  'bookingModelService',
  'instantHoldModelService',
  'routingService',
  'paymentTypes',
  'returnsUserProfileModel',
  'Profile',
  'events',
  '$timeout',
  'satelliteTrackingModelService',
  'dateFormatterService',

  function (
    $scope,
    $rootScope,
    hotelInformationModel,
    searchParamModelService,
    bookingModelService,
    instantHoldModelService,
    routingService,
    paymentTypes,
    returnsUserProfileModel,
    Profile,
    events,
    $timeout,
    satelliteTrackingModelService,
    dateFormatterService
  ) {
    'use strict';

    $scope.instantHold = instantHoldModelService.data;

    // used to prevent back button of the browser.
    if (bookingModelService.data.confirmationNumber !== '') {
      routingService.goToHomePage();
    }

    $scope.instantHoldError = '';
    $scope.instantHoldForm = {};
    bookingModelService.data.paymentType = paymentTypes.INSTANTHOLD;

    function forwardToConfirmation() {
      routingService.goToBookingConfirmation();
      $scope.loadingModal.close();
      $timeout(function () {
        trackHoldConfirmationLoaded();
      }, 500);
    }

    function trackHoldConfirmationLoaded() {
      var oneDay = 1000 * 60 * 60 * 24;
      var now = new Date();
      var checkInTime = searchParamModelService.data.checkinDate.getTime();
      var checkoutTime = searchParamModelService.data.checkoutDate.getTime();
      var checkInCheckOutDiffMS = checkoutTime - checkInTime;
      var nowCheckInDiffMS = checkInTime - now.getTime();
      var dateDiffInDays = Math.round(checkInCheckOutDiffMS / oneDay);
      var daysUntilCheckIn = Math.round(nowCheckInDiffMS / oneDay);
      var daysToArrival = daysUntilCheckIn;
      var checkinDate = searchParamModelService.data.checkinDate;
      var checkoutDate = searchParamModelService.data.checkoutDate;
      var numberOfNights = dateDiffInDays;
      var numberOfAdults = searchParamModelService.data.numberOfAdults;
      var promoCode = searchParamModelService.data.promoCode;
      var totalRate = searchParamModelService.data.selectedRoom.hotelRoomRate.totalRate.amount;
      var reservationRevenue =
        searchParamModelService.data.selectedRoom.hotelRoomRate.totalRateWithTaxes.amount;
      var totalTaxes = reservationRevenue - totalRate;
      var currencyCode =
        searchParamModelService.data.selectedRoom.hotelRoomRate.averageNightlyRate.currencyCode;
      var hotelId = hotelInformationModel.data.hotelDetails.hotelId;
      var roomTypeName =
        searchParamModelService.data.selectedRoom.hotelRoomRate.roomType.roomDescription;
      var guestPostalCode = bookingModelService.data.zipCode;
      var roomBookingType = 'mobile';
      var confirmationNum = bookingModelService.data.confirmationNumber;

      if (daysUntilCheckIn < 1) {
        daysToArrival = 0;
      }

      satelliteTrackingModelService.lqInstantHoldConfirmationLoaded(
        numberOfNights,
        totalRate,
        totalTaxes,
        currencyCode,
        hotelId,
        dateFormatterService.getFormattedDate(checkinDate),
        dateFormatterService.getFormattedDate(checkoutDate),
        numberOfAdults,
        daysToArrival,
        roomTypeName,
        guestPostalCode,
        promoCode,
        roomBookingType,
        confirmationNum,
        reservationRevenue
      );
    }

    function handleError(error) {
      if (typeof error !== 'undefined' && error.length > 0) {
        $scope.instantHoldError = CQ.I18n.getMessage(error.trim());
      }
      $scope.instantHoldForm.submitDisabled = false;
      $scope.loadingModal.close();
    }


    $scope.instantHoldForm.submitForm = function (isFormValid) {
      $scope.instantHoldForm.formSubmitted = true;
      $scope.instantHoldForm.submitDisabled = true;
      if (isFormValid) {
        $scope.instantHoldError = '';
        $scope.loadingModal = $scope.showLoadingModal();
        instantHoldModelService.completeBooking(
          $scope.bookingForm.mobilePhone
        ).then(forwardToConfirmation, handleError);
      } else {
        $scope.instantHoldForm.submitDisabled = false;
      }
    };

    satelliteTrackingModelService.lqInstantHoldInfoLoaded(
      hotelInformationModel.data.hotelDetails.hotelId
    );
  }]);
