/* global lqMobileModule, alert, LQMobile*/
/* jshint maxparams: 10 */
lqMobileModule.factory('instantHoldModelService',
  ['searchParamModelService', 'hotelInformationModel', 'instantHoldEndpointService', 'bookingModelService', 'ReservationRequest', 'RoomReservation', 'lodash', 'returnsUserProfileModel', 'reservationsModel',
    function (
      searchParamModelService,
      hotelInformationModel,
      instantHoldEndpointService,
      bookingModelService,
      ReservationRequest,
      RoomReservation,
      lodash,
      returnsUserProfileModel,
      reservationsModel
    ) {
      'use strict';

      var dataModel = {};

      /* Map the data from the various sources to the reservationRequest object */
      dataModel.buildReservationRequest = function (mobilePhone) {
        var reservationRequest = ReservationRequest.buildInstantHold();
        var roomReservation = RoomReservation.build();
        var hotelModel = hotelInformationModel.data;
        var searchModel = searchParamModelService.data;
        var arrivalDate;
        var checkoutDate;

        // Populate mainContant and paymentInfo with the phone number
        if (returnsUserProfileModel.isLoggedIn()) {
          reservationRequest.mainContact.returnsNumber =
            returnsUserProfileModel.data.userProfile.returnsNumber;
        }

        reservationRequest.mainContact.phoneNumbers[0].phoneNumber.number = mobilePhone;
        reservationRequest.paymentInfo[0].phoneNumber.number = mobilePhone;

        // Populate hotelData
        reservationRequest.innNumber.id = lodash.get(hotelModel, 'hotelDetails.hotelId', null);

        // Populate searchData
        arrivalDate = lodash.get(searchModel, 'checkinDate', null);
        reservationRequest.arrivalDate = LQMobile.Util.dateToString(arrivalDate);
        checkoutDate = lodash.get(searchModel, 'checkoutDate', null);
        reservationRequest.departureDate = LQMobile.Util.dateToString(checkoutDate);

        reservationRequest.quotedTotalRate.amount = lodash.get(searchModel, 'selectedRoom.hotelRoomRate.totalRate.amount', null);
        reservationRequest.quotedTotalRate.currencyCode = lodash.get(searchModel, 'selectedRoom.hotelRoomRate.totalRateWithTaxes.currencyCode', null);

        reservationRequest.quotedTotalRateWithTaxes.amount = lodash.get(searchModel, 'selectedRoom.hotelRoomRate.totalRateWithTaxes.amount', null);
        reservationRequest.quotedTotalRateWithTaxes.currencyCode = lodash.get(searchModel, 'selectedRoom.hotelRoomRate.totalRateWithTaxes.currencyCode', null);

        reservationRequest.rateQualifier.rateCode.crsCode = lodash.get(searchModel, 'selectedRoom.hotelRoomRate.rateQualifier.rateCode.crsCode');

        reservationRequest.roomReservations.push(roomReservation);

        roomReservation.numberOfAdults = lodash.get(searchModel, 'numberOfAdults', null);
        roomReservation.numberOfChildren = lodash.get(searchModel, 'numberOfChildren', null);
        roomReservation.numberOfRooms = lodash.get(searchModel, 'numberOfRooms', null);
        roomReservation.roomType.crsRoomTypeCode = lodash.get(searchModel, 'selectedRoom.hotelRoomRate.roomType.crsRoomTypeCode', null);
        roomReservation.roomType.roomCode = lodash.get(searchModel, 'selectedRoom.hotelRoomRate.roomType.roomCode', null);
        reservationRequest.roomReservations[0].nightlyRoomRates = lodash.get(searchModel, 'selectedRoom.hotelRoomRate.nightlyRates', null);

        return {
          ReservationRequest: reservationRequest
        };
      };

      dataModel.completeBooking = function (mobilePhone) {
        var isUserLoggedIn = returnsUserProfileModel.isLoggedIn();
        bookingModelService.setMobilePhone(mobilePhone);
        return instantHoldEndpointService.makeRequest(
          dataModel.buildReservationRequest(mobilePhone)
        ).then(
          function (result) {
            if (typeof result !== 'undefined' && result.data.length > 0) {
              bookingModelService.data.confirmationNumber =
                result.data[0].reservation.confirmationNumber.number;
              if (!isUserLoggedIn) {
                reservationsModel.addLocalReservation(bookingModelService.data.confirmationNumber, 'LQ', 'INSTANTHOLD');
              }
              bookingModelService.resetBookingData();
              bookingModelService.persistData();
              return result;
            }
            return 'Instant Hold Error';
          },
          function (error) {
            throw error.data;
          }
        );
      };

      return dataModel;
    }]);
