/* global lqMobileModule */
lqMobileModule.controller(
  'mobileCheckinController',
  [
    '$scope',
    '$rootScope',
    '$modal',
    'routingService',
    'activeStaysModel',
    'returnsUserProfileModel',
    'satelliteTrackingModelService',
    'touchIdService',
    'mobileCheckinResultModel',
    function (
      $scope,
      $rootScope,
      $modal,
      routingService,
      activeStaysModel,
      returnsUserProfileModel,
      satelliteTrackingModelService,
      touchIdService,
      mobileCheckinResultModel
    ) {
      /**
      * init
      *
      * Initialize the Mobile Checkout Controller
      */
      var modalInstance = null;
      var isOpening = false;

      /**
       * Get only rooms on a particular floor
       *
       * @param {(string|number)} floorNumber - number of floor
       * @param {string} roomNumber - already assigned room number
       * @returns {Array} all rooms on provided floor
       */
      function getRoomsForFloor(floorNumber, roomNumber) {
        var allRooms = activeStaysModel.availableRooms.reduce(function (array, room) {
          if (room.floor == floorNumber) array.push(room.roomNumber);
          return array;
        }, []);

        if (roomNumber && roomNumber.slice(0, -2) === floorNumber && allRooms.indexOf(roomNumber) < 0) allRooms.push(roomNumber);

        return allRooms.sort();
      }

      function init() {
        var notAlreadyOpening;

        //
        // What if we are not logged in?
        if (!returnsUserProfileModel.isLoggedIn()) {
          notAlreadyOpening = modalInstance === null && !isOpening;

          // check if user is using finger auth
          touchIdService.userIsUsingFingerAuth().then(
            function (result) {
              if (result.isUsing) {
                touchIdService.promptForFingerAuth().then(
                  function (creds) {
                    //
                    // Authenticate the User.
                    returnsUserProfileModel.authenticateUser(
                      creds.username, '', creds.lastname, creds.password
                    ).then(
                      function (authToken) {
                        //
                        // Persist the Auth Token
                        window.sessionStorage.setItem('token', authToken.token);

                        $scope.getActiveStayData();
                      },
                      function (error) {
                        // creds error , send to sign in screen with rootscope error
                        $rootScope.touchIdAuthError = error;
                        routingService.goToSignInPage();
                      }
                    );
                  },
                  function () {
                    // failed or canceled:
                    if (notAlreadyOpening) {
                      isOpening = true;
                      modalInstance = $modal.open({
                        templateUrl: 'loginModalCheckout.html',
                        controller: 'genericLoginModalInstanceController',
                        windowClass: 'app-modal-window',
                        resolve: {
                          showLoadingModal: function () {
                            isOpening = false;
                            return $scope.showLoadingModal;
                          }
                        }
                      });

                      $scope.setupModalInstanceCallbacks();
                    }
                  }
                );
              } else if (notAlreadyOpening) {
                isOpening = true;
                modalInstance = $modal.open({
                  templateUrl: 'loginModalCheckout.html',
                  controller: 'genericLoginModalInstanceController',
                  windowClass: 'app-modal-window',
                  resolve: {
                    showLoadingModal: function () {
                      isOpening = false;
                      return $scope.showLoadingModal;
                    }
                  }
                });

                $scope.setupModalInstanceCallbacks();
              }
            },
            function () {
              //
              // Open the sign in modal.
              if (notAlreadyOpening) {
                isOpening = true;
                modalInstance = $modal.open({
                  templateUrl: 'loginModalCheckout.html',
                  controller: 'genericLoginModalInstanceController',
                  windowClass: 'app-modal-window',
                  resolve: {
                    showLoadingModal: function () {
                      isOpening = false;
                      return $scope.showLoadingModal;
                    }
                  }
                });

                $scope.setupModalInstanceCallbacks();
              }
            }
          );
        // get active stay info and check type
        } else {
          $scope.getActiveStayData();
        }
      }

      $scope.showLoadingMessageModal = function () {
        return $modal.open({
          templateUrl: 'loadingMessageModalContent.html',
          controller: 'loadingMessageModalController',
          windowClass: 'loading-modal-window',
          backdrop: 'static',
          resolve: {
            loadingMessage: function () {
              return lqmobileConfigs.checkinModalLoadingMessage;
            }
          },
          keyboard: false
        });
      };

      $scope.setupModalInstanceCallbacks = function () {
        modalInstance.result.then(
          function () {
            console.log('modal close logged in');
            modalInstance.close();

            // get stay data
            $scope.getActiveStayData();
          },
          function () {
            console.log('modal dismissed');
            // send to homepage
            $scope.goToHomePage();
          }
        );
      };

      $scope.getActiveStayData = function () {
        if (!$scope.isCheckingActiveStays) {
          $scope.isCheckingActiveStays = true;
          $scope.loadingModal = $scope.showLoadingModal();

          // get active stays
          activeStaysModel.getActiveStay(
            returnsUserProfileModel.data.userProfile.returnsNumber
          ).then(
            function () {
              var roomNumber;
              var floorNumber;

              // check if active stay is a check-in
              if (!activeStaysModel.isActiveCheckIn()) {
                if (activeStaysModel.isCheckedIn()) {
                  // already checked in
                  mobileCheckinResultModel.setCheckinResultModelAlreadyCheckedIn();
                  satelliteTrackingModelService.mobileCheckInAlreadyDone();
                } else {
                  // generic unavailable
                  mobileCheckinResultModel.setCheckinResultModelFailure(
                    activeStaysModel.activeStay.hotel.address,
                    activeStaysModel.activeStay.hotel.phone
                  );
                }

                // no active stay, send to end screen
                routingService.goToMobileCheckinResult(true, true);
              } else if (activeStaysModel.availableRooms.length || activeStaysModel.getRoomNumber()) {
                if ($scope.loadingModal) $scope.loadingModal.close();

                // active check in stay found and rooms available OR already assigned a room
                // parse floors and rooms
                roomNumber = activeStaysModel.getRoomNumber();
                floorNumber = 'Select Floor';

                // set rooms
                if (roomNumber) {
                  // get floor and rooms
                  floorNumber = roomNumber.slice(0, -2);
                  $scope.allRooms = getRoomsForFloor(floorNumber, roomNumber);

                  // set selected
                  $scope.currentHotelFloor = floorNumber;
                  $scope.currentRoomNumber = roomNumber;
                } else {
                  $scope.allRooms = ['Select Room'];
                }

                $scope.allFloors = activeStaysModel.availableRooms.reduce(function (array, room) {
                  if (array.indexOf(room.floor) < 0) array.push(room.floor);
                  return array;
                }, []);
                $scope.allFloors.unshift('Select Floor');
              } else {
                // no rooms available or call failed
                // generic unavailable - send to end screen
                mobileCheckinResultModel.setCheckinResultModelFailure(
                  activeStaysModel.activeStay.hotel.address,
                  activeStaysModel.activeStay.hotel.phone
                );
                routingService.goToMobileCheckinResult(true, true);
              }

              $scope.isCheckingActiveStays = false;
            },
            function () {
              if ($scope.loadingModal) $scope.loadingModal.close();
              // complete failure to even fetch data
              mobileCheckinResultModel.setCheckinResultModelFailure('');
              // no active stay, send to end screen
              routingService.goToMobileCheckinResult(true, true);

              $scope.isCheckingActiveStays = false;
            }
          );
        }
      };

      /**
      * Object check in data
      */
      $scope.activeStaysModel = activeStaysModel;

      /**
      * String - fine print
      */
      $scope.MOBILE_CHECK_IN_REVIEW_FINE_PRINT = MOBILE_CHECK_IN_REVIEW_FINE_PRINT;

      /**
       * Dropdown data
       */
      $scope.allFloors = ['Select Floor'];
      $scope.currentHotelFloor = 'Select Floor';
      $scope.hotelFloorIsOpen = false;
      $scope.allRooms = ['Select Room'];
      $scope.currentRoomNumber = 'Select Room';
      $scope.roomNumberIsOpen = false;

      /**
      * checkIn
      *
      * Event handler for click of Complete Check In button.
      */
      $scope.checkIn = function () {
        // Loading message with label
        $scope.loadingModal = $scope.showLoadingMessageModal();

        //
        // Process the check in
        activeStaysModel.processCheckin($scope.currentRoomNumber).then(

          //
          // Success
          function (response) {
            // build successful check in for result model
            mobileCheckinResultModel.setCheckinResultModelSuccess(
              activeStaysModel.activeStay.hotel.address,
              activeStaysModel.activeStay.hotel.phone,
              response.roomNumber
            );
            routingService.goToMobileCheckinResult();
            $scope.loadingModal.close();
          },

          //
          // Error
          function (err) {
            console.log(err);
            mobileCheckinResultModel.setCheckinResultModelFailure(
              activeStaysModel.activeStay.hotel.address,
              activeStaysModel.activeStay.hotel.phone
            );
            routingService.goToMobileCheckinResult();
            $scope.loadingModal.close();
          }

        );
      };

      /**
      * cancel
      *
      * Event handler for click of Cancel link.
      */
      $scope.cancel = function () {
        $scope.goToHomePage();
      };

      $scope.goToHomePage = function () {
        routingService.goToHomePage();
      };

      /**
       * updateFloor
       *
       * Update the room data on floor change
       */
      $scope.updateFloor = function () {
        // filter rooms
        $scope.allRooms = getRoomsForFloor($scope.currentHotelFloor, activeStaysModel.getRoomNumber());
        if ($scope.allRooms.length === 0) $scope.allRooms = ['Select Room'];

        // set room if floor selected
        if ($scope.allRooms.length) {
          $scope.currentRoomNumber = $scope.allRooms[0];
        } else {
          $scope.currentRoomNumber = 'Select Room';
        }
      };

      $scope.$on('check-active-stays', $scope.getActiveStayData);
      init();
    }]
);
