/* global lqMobileModule */
/* jshint maxparams: 11 */
lqMobileModule.factory(
  'mobileCheckinResultModel',
  [
    'routingService',
    function (
      routingService
    ) {
      'use strict';

      var dataModel = {
        data: {}
      };

      function fillInHotelInfo(message, roomNumber, phoneNumber) {
        return message.replace('{ROOM_NUMBER}', roomNumber).replace('{PHONE_NUMBER}', phoneNumber);
      }

      function setActionAddresses(hotelAddress) {
        var address = typeof hotelAddress !== 'undefined' ? hotelAddress : 'current location';
        dataModel.data.actions.uberAddress = encodeURIComponent(address);
        dataModel.data.actions.mapsAddress = encodeURIComponent(address);
      }

      function formatHotelPhone(hotelPhone) {
        // strip all non-digit characters for easy tel: implementation
        if (typeof hotelPhone === 'string') dataModel.data.actions.phoneNumber = hotelPhone.replace(/\D/g, '');
      }

      dataModel.setCheckinResultModelSuccess = function (hotelAddress, hotelPhone, roomNumber) {
        var bodyText;
        dataModel.reset();

        if (typeof roomNumber === 'undefined') {
          bodyText = fillInHotelInfo(MOBILE_CHECK_IN_SUCCESS_MESSAGE_NO_ROOM, '', '');
        } else {
          bodyText = fillInHotelInfo(MOBILE_CHECK_IN_SUCCESS_MESSAGE_WITH_ROOM, roomNumber, '');
        }

        dataModel.data.displayActions = true;
        setActionAddresses(hotelAddress);
        formatHotelPhone(hotelPhone);

        dataModel.data.headerText = MOBILE_CHECK_IN_SUCCESS_HEADER;
        dataModel.data.bodyText = bodyText;
      };

      dataModel.setCheckinResultModelFailure = function (hotelAddress, hotelPhone) {
        var bodyText;
        dataModel.reset();

        if (typeof hotelPhone === 'undefined') {
          bodyText = fillInHotelInfo(MOBILE_CHECK_IN_FAILURE_MESSAGE_NO_PHONE, '', '');
        } else {
          bodyText = fillInHotelInfo(MOBILE_CHECK_IN_FAILURE_MESSAGE, '', hotelPhone);
        }

        dataModel.data.displayActions = true;
        setActionAddresses(hotelAddress);
        formatHotelPhone(hotelPhone);

        dataModel.data.headerText = MOBILE_CHECK_IN_FAILURE_HEADER;
        dataModel.data.bodyText = bodyText;
      };

      dataModel.setCheckinResultModelAlreadyCheckedIn = function () {
        dataModel.reset();

        dataModel.data.headerText = MOBILE_CHECK_IN_ALREADY_CHECKED_IN_HEADER;
        dataModel.data.bodyText = MOBILE_CHECK_IN_ALREADY_CHECKED_IN_MESSAGE;

        dataModel.data.displayStatusBox = true;
        dataModel.data.statusBox.headerText = 'Status';
        dataModel.data.statusBox.bodyText = 'Checked In';
      };

      dataModel.reset = function () {
        dataModel.data = {
          displayActions: false,
          actions: {
            uberAddress: '',
            mapsAddress: '',
            phoneNumber: ''
          },
          displayHeader: true,
          headerText: MOBILE_CHECK_IN_DEFAULT_HEADER,
          displayBody: true,
          bodyText: MOBILE_CHECK_IN_DEFAULT_MESSAGE,
          displayStatusBox: false,
          statusBox: {
            headerText: 'Status',
            bodyText: ''
          },
          displayPrimaryCTA: true,
          primaryCTA: {
            text: 'Back To Home',
            action: routingService.goToHomePage
          },
          displayLink: false,
          link: {
            text: 'cancel',
            action: routingService.goToHomePage
          }
        };
      };

      dataModel.reset();
      return dataModel;
    }
  ]
);
