/* global lqMobileModule */
lqMobileModule.controller(
  'mobileCheckoutController',
  [
    '$scope',
    '$q',
    '$modal',
    'routingService',
    'mobileCheckoutModel',
    'returnsUserProfileModel',
    'satelliteTrackingModelService',
    'touchIdService',
    'activeStaysModel',
    function (
      $scope,
      $q,
      $modal,
      routingService,
      mobileCheckoutModel,
      returnsUserProfileModel,
      satelliteTrackingModelService,
      touchIdService,
      activeStaysModel
    ) {
      /**
      * init
      *
      * Initialize the Mobile Checkout Controller
      */
      var modalInstance = null;
      var isOpening = false;

      function init() {
        var notAlreadyOpening;

        //
        // What if we are not logged in?
        if (!returnsUserProfileModel.isLoggedIn()) {
          notAlreadyOpening = modalInstance === null && !isOpening;

          // @todo:: if user is using touch id, offer that
          // if they fail or cancel, show the popup instead.
          // { isUsing: usesTouchId, isCurrent: isCurrentUser }
          touchIdService.userIsUsingFingerAuth().then(
            function (result) {
              if (result.isUsing) {
                touchIdService.promptForFingerAuth().then(
                  function (creds) {
                    //
                    // Authenticate the User.
                    returnsUserProfileModel.authenticateUser(
                      creds.username, '', creds.lastname, creds.password
                    ).then(
                      function (authToken) {
                        //
                        // Persist the Auth Token
                        window.sessionStorage.setItem('token', authToken.token);
                        //
                        // Mobile Checkout Eligibility
                        mobileCheckoutModel.checkEligibility().then(
                          // success
                          function () {
                            //
                            // Load User Profile
                            returnsUserProfileModel.getUserProfile(
                              authToken.token
                            ).then(
                              function () {
                                $rootScope.$broadcast(events.signInActionTaken);
                                if (!mobileCheckoutModel.isEligible()) {
                                  routingService.goToCheckoutUnavailable(true);
                                }
                              },
                              function () {
                                // auth was seccuessful, but no user profile available,
                                // i have no idea how to handle this situation... checkout
                                // unavailable i suppose....
                                routingService.goToCheckoutUnavailable(true);
                              }
                            );
                            if (!mobileCheckoutModel.isEligible()) {
                              routingService.goToCheckoutUnavailable(true);
                            }
                          },
                          // error
                          function () {
                            routingService.goToCheckoutUnavailable(true);
                          }
                        );
                      },
                      function (error) {
                        // creds error , send to sign in screen with rootscope error
                        $rootScope.touchIdAuthError = error;
                        routingService.goToSignInPage();
                      }
                    );
                  },
                  function () {
                    // failed or canceled:
                    if (notAlreadyOpening) {
                      isOpening = true;
                      modalInstance = $modal.open({
                        templateUrl: 'loginModalCheckout.html',
                        controller: 'checkoutLoginModalInstanceController',
                        windowClass: 'app-modal-window',
                        resolve: {
                          showLoadingModal: function () {
                            isOpening = false;
                            return $scope.showLoadingModal;
                          },
                          showLoadingMessageModal: function () {
                            isOpening = false;
                            return $scope.showLoadingMessageModal;
                          }
                        }
                      });

                      modalInstance.result.then(
                        function () {
                          modalInstance.close();
                        },
                        function () {
                           //
                        }
                      );
                    }
                  }
                );
              } else if (notAlreadyOpening) {
                isOpening = true;
                modalInstance = $modal.open({
                  templateUrl: 'loginModalCheckout.html',
                  controller: 'checkoutLoginModalInstanceController',
                  windowClass: 'app-modal-window',
                  resolve: {
                    showLoadingModal: function () {
                      isOpening = false;
                      return $scope.showLoadingModal;
                    },
                    showLoadingMessageModal: function () {
                      isOpening = false;
                      return $scope.showLoadingMessageModal;
                    }
                  }
                });

                modalInstance.result.then(
                  function () {
                    modalInstance.close();
                  },
                  function () {
                     //
                  }
                );
              }
            },
            function () {
              //
              // Open the sign in modal.
              if (notAlreadyOpening) {
                isOpening = true;
                modalInstance = $modal.open({
                  templateUrl: 'loginModalCheckout.html',
                  controller: 'checkoutLoginModalInstanceController',
                  windowClass: 'app-modal-window',
                  resolve: {
                    showLoadingModal: function () {
                      isOpening = false;
                      return $scope.showLoadingModal;
                    },
                    showLoadingMessageModal: function () {
                      isOpening = false;
                      return $scope.showLoadingMessageModal;
                    }
                  }
                });

                modalInstance.result.then(
                  function () {
                    modalInstance.close();
                  },
                  function () {
                     //
                  }
                );
              }
            }
          );

      // If we have not already established eiligibility, check for it now.
        } else if (!mobileCheckoutModel.isEligible()) {
          $scope.loadingModal = $scope.showLoadingModal();
          mobileCheckoutModel.checkEligibility().then(

            // success.
            function (resp) {
              $scope.loadingModal.close();

              //
              // If not eligible, send to Checkout Unavilable page.
              if (!resp.eligibility) {
                routingService.goToCheckoutUnavailable();
              }
            },

            // failure
            function () {
              routingService.goToCheckoutUnavailable();
              $scope.loadingModal.close();
            }
          );
        }
      }

      $scope.showLoadingMessageModal = function () {
        return $modal.open({
          templateUrl: 'loadingMessageModalContent.html',
          controller: 'loadingMessageModalController',
          windowClass: 'loading-modal-window',
          backdrop: 'static',
          resolve: {
            loadingMessage: function () {
              return lqmobileConfigs.checkoutModalLoadingMessage;
            }
          },
          keyboard: false
        });
      };


      /**
      * Bool checkoutAttempt
      */
      $scope.checkoutAttempt = false;


      /**
      * Object checkoutData
      */
      $scope.checkoutData = {};
      $scope.checkoutData.isEligible = mobileCheckoutModel.isEligible();


      /**
      * getLastUpdated
      *
      * Get the formatted date of the checkout data's last update.
      *
      * @return String
      */
      $scope.checkoutData.getLastUpdated = function () {
        return mobileCheckoutModel.lastUpdated;
      };

      /**
      * getBalanceDue
      *
      * Get the formatted balance due amount for the reservation.
      *
      * @return String
      */
      $scope.checkoutData.getBalanceDue = function () {
        var balance = parseFloat(mobileCheckoutModel.balanceDue);
        return isNaN(balance) ? '$--' : '$' + balance.toFixed(2);
      };

      $scope.payments = function () {
        return mobileCheckoutModel.paymentInfo;
      };

      $scope.balanceIsZero = function () {
        var balance = parseFloat(mobileCheckoutModel.balanceDue);
        return isNaN(balance) || balance === 0.0;
      };

      /**
      * isDirectBill
      *
      * Determine of the reservation is Direct Bill vs Credit Card.
      *
      * @return Boolean
      */
      $scope.isDirectBill = function () {
        return mobileCheckoutModel.hasDirectBill;
      };

      /**
       * isCreditCard
       *
       * @return Boolean
       */
      $scope.isCreditCard = function () {
        return mobileCheckoutModel.hasCreditCard;
      };


      /**
      * checkout
      *
      * Event handler for click of Complete Checkout button.
      */
      $scope.checkout = function () {
        var deferred = $q.defer();

        //
        // Progress indicator
        $scope.checkoutAttempt = true;

        // Loading message with label
        $scope.loadingModal = $scope.showLoadingMessageModal();


        //
        // Process the Checkout.
        activeStaysModel.processCheckout().then(

          //
          // Success
          function (resp) {
            routingService.goToCheckoutConfirmation(); // redirect the user to next step.
            $scope.loadingModal.close();
            deferred.resolve(resp);
          },

          //
          // Error
          function (err) {
            deferred.reject(CQ.I18n.getMessage(err));

            routingService.goToCheckoutUnavailable();
            $scope.loadingModal.close();
          }

        );

        return deferred.promise;
      };

      /**
      * cancel
      *
      * Event handler for click of Cancel link.
      */
      $scope.cancel = function () {
        routingService.goToHomePage();
      };

      init();
    }]
);
