/* global lqMobileModule, console, angular*/
lqMobileModule.factory('mobileCheckoutEndpointService', ['$window',
  '$q',
  '$http',
  'serviceEndPoints',
  'lodash',
  'satelliteTrackingModelService',
  function (
    $window,
    $q,
    $http,
    serviceEndPoints,
    lodash,
    satelliteTrackingModelService
  ) {
    'use strict';

    var endpointService = {};

    /**
     * checks if a user is eligible for mobile checkout by LQ_AUTH_KEY
     * this value should be provided when the user is authenticated as "token"
     *
     * @param token string
     * @returns status only
     */
    endpointService.getEligibility = function (token) {
      var serviceURL = serviceEndPoints.getMobileCheckoutEligibility();

      // serviceURL = './test/checkoutEligible.json';
      return $http({
        method: 'get',
        url: serviceURL,
        params: {},
        headers: {
          'Cache-Control': 'no-cache',
          LQ_AUTH_KEY: lodash.trim(token),
          apikey: serviceEndPoints.getApiKey()
        }

      }).then(
        // Success
        function (response) {
          return response;
        },
        // Error
        function (error) {
          console.error(JSON.stringify(error));
          throw error;
        }
      );
    };


    /**
     *
     * @param token string
     * @param confirmationNumber string
     * @returns {
          *       "elgibility": bool|bit, (unsure yet)
          *       "folioNumber": number
          *       "confirmationNumber": number
          *    }
     */
    endpointService.checkoutAction = function (token, confirmationNumber) {
      var serviceURL = serviceEndPoints.getMobileCheckoutAction(confirmationNumber);
      satelliteTrackingModelService.mobileCheckoutAttempted();

      console.log('attempting to checkout via: ' + serviceURL);
      // serviceURL = './test/checkout.json';
      return $http({
        method: 'post', // 'get', //
        url: serviceURL,
        params: {},
        headers: {
          'Cache-Control': 'no-cache',
          LQ_AUTH_KEY: lodash.trim(token),
          apikey: serviceEndPoints.getApiKey()
        }
      }).then(
        // Success
        function (response) {
          satelliteTrackingModelService.mobileCheckoutCompleted();
          console.log('checkout success');
          return response;
        },
        // Error
        function (error) {
          satelliteTrackingModelService.mobileCheckoutUnsuccessful();
          console.error(JSON.stringify(error));
          throw error;
        }
      );
    };


    return endpointService;
  }]);
