/* global lqMobileModule, CQ*/
/* jshint maxparams: 11 */
lqMobileModule.factory(

  'mobileCheckoutModel',

  [
    '$q',
    '$filter',
    '$rootScope',
    'storageModelService',
    'mobileCheckoutEndpointService',
    'satelliteTrackingModelService',
    'localStorageService',
    function (
      $q,
      $filter,
      $rootScope,
      storageModelService,
      mobileCheckoutEndpointService,
      satelliteTrackingModelService,
      localStorageService
    ) {
      'use strict';


    /**
    * Bool isEligible
    */
      var isEligible = false;
      var dtmEventFired = false;

      var isSuccessfulResponse = false;

    // Initialzie the data model and define public properties.
      var dataModel = {

        /**
        * Int lastCheck
        *
        * Timestamp of last service call to check eligibility.
        */
        lastCheck: null,

        /**
        * String confirmationNumber
        *
        * Reservation confirmation numbers, used to process the checkout action.
        */
        confirmationNumber: null,

        /**
        * String cardOnFile
        *
        * Last 4 of customer's credit card on file. Required if directBillAcct is empty.
        */
        cardOnFile: null,

        /**
        * Double balanceDue
        *
        * Open balance amount on the reservation.
        */
        balanceDue: null,

        /**
        * Object[] paymentInfo
        */
        paymentInfo: [],

        /**
        * Date lastUpdated
        *
        * Timestamp when the balance/account info was last updated as of.
        */
        lastUpdated: null,

        innNumber: null,
        hasCreditCard: false,
        hasDirectBill: false,
        hasFreeNight: false

      };


    /**
     * isEligible
     *
     * Returns the current mobile checkout status.
     *
     * @returns {boolean}
     */
      dataModel.isEligible = function () {
        return isEligible && (new Date().getHours() < 12); // Do not allow checkout after 12pm.
      };

    /*
       Based on DE2331

       When calling GET /ecommerce/centralreservation/reservations/departure/eligible
         If a 200 with the "eligibility": false - show the mobile checkout banner
         If a 200 with the "eligibility": true - show the mobile checkout banner
         If another status code (such as 422 or 567) is returned then do not show the banner
    * */
      dataModel.isSuccessfulMobileCheckoutResponse = function () {
        return isSuccessfulResponse;
      };


    /**
    * showMobileCheckoutBanner
    *
    * determine whether or not the Mobile Checkout CTA should be shown to the end user;
    *
    * @return Boolean
    */
      dataModel.showMobileCheckoutBanner = function () {
        // 200 response from Mobile Checkout web service
        // AND it is before 12pm device time
        // AND have NOT completed checkout within last 12 hours.
        if (dataModel.isSuccessfulMobileCheckoutResponse()
            && (new Date().getHours() < 12)
            && !dataModel.recentCheckoutComplete()) {
          // track analytics
          if (!dtmEventFired) {
            satelliteTrackingModelService.mobileCheckoutBannerShown(
              isEligible, dataModel.confirmationNumber
            );
            dtmEventFired = true;
          }
          return true;
        }

        return false;
      };


    /**
     * setEligibility
     *
     * Update the current mobile checkout status.
     *
     * @param eligibility {boolean}
     */
      dataModel.setEligibility = function (eligibility) {
        isEligible = eligibility;
      };

    /**
     * This function will build and return the this modal from the API
     * Service request
     *
     * @param response {Object}
     */
      dataModel.build = function (response) {
        var i;
        var paymentObject;
        var payInfo;
        var trimmed;
        var lastTwo;
        var departDate;
        var month;
        var day;
        var hr;
        var min;
        var meridian;
        var iMax;

        isEligible = response.eligibility === true;
        dataModel.confirmationNumber = response.confirmationNumber.number;
        dataModel.innNumber = response.innNumber.idWithoutZero;
        dataModel.balanceDue = response.folioBalance.amount;
        dtmEventFired = false;

      //
      // Setup the returned payment type objects
        dataModel.paymentInfo = [];
        for (i = 0, iMax = response.paymentInfo.length; i < iMax; i += 1) {
          paymentObject = {};
          payInfo = response.paymentInfo[i];
          switch (payInfo.type) {
            case 'CreditCard':
              dataModel.hasCreditCard = true;
              paymentObject.typeString = 'Card On File';
              paymentObject.detailString = 'xxxx-' + payInfo.creditCard.creditCardLastFour;
              break;
            case 'FreeNightCertificate':
              dataModel.hasFreeNight = true;
              paymentObject.typeString = 'Free Night';
              paymentObject.detailString = 'Remaining Points ' + payInfo.certificateNumber.points;
              break;
            case 'DirectBill':
              dataModel.hasDirectBill = true;
              paymentObject.typeString = 'Direct Bill';
              paymentObject.detailString = payInfo.directBillCustomerName;
              break;
            default:
              // shouldn't happen
          }
          dataModel.paymentInfo.push(paymentObject);
        }

      //
      // Parse the Last Modified Date
        trimmed = response.departureTime.replace(/\s/g, '');
        if (trimmed.indexOf('Z') == -1 && trimmed.indexOf('z') == -1) {
          lastTwo = trimmed.substr(trimmed.length - 2, 2);
          trimmed = trimmed.substr(0, trimmed.length - 2);
          trimmed += ':' + lastTwo;
        }

        departDate = new Date(trimmed);
        month = departDate.getMonth() + 1;
        day = departDate.getDate();
        hr = departDate.getHours();
        min = departDate.getMinutes();
        meridian = '';

        if (min < 10) {
          min = '0' + min;
        }
        if (hr < 12) {
          if (hr == 0) {
            hr = 12;
          }
          meridian = 'am';
        } else {
          if (hr > 12) {
            hr -= 12;
          }
          meridian = 'pm';
        }
        dataModel.lastUpdated = month + '/' + day + ' ' + hr + ':' + min + meridian;
      };

      dataModel.reset = function () {
        dataModel.lastCheck = null;
        dataModel.confirmationNumber = null;
        dataModel.cardOnFile = null;
        dataModel.balanceDue = null;
        dataModel.paymentInfo = [];
        dataModel.lastUpdated = null;
        // dataModel.innNumber = null;
        dataModel.hasCreditCard = false;
        dataModel.hasDirectBill = false;
        dataModel.hasFreeNight = false;
        isSuccessfulResponse = false;
        isEligible = false;
        dtmEventFired = false;
      };

    /**
    * supressEligibilityCheck
    *
    * Return true if our timeout suggests we should suppress the eligibility check.
    *
    * @return Boolean
    */
      dataModel.suppressEligibilityCheck = function () {
        //
        // Don't check more than once per hour.
        //  On success (or permanent failure), sleep for 12 hours.
        var t = new Date().getTime();
        var sleep = localStorageService.getValue('lqApp.Checkout.Timeout');

        // less than 1 hour since last check
        return (
          t < dataModel.lastCheck + (3600 * 1000)
          // less than 12 hours since last successful checkout
          || (sleep != null && t < sleep)
          // after 12pm local time.
          || (new Date().getHours() > 12)
        );
      };

    /**
    * recentCheckoutComplete
    *
    * Return true if the user has recently (within last 12 hours) successfully completed
    * mobile checkout.
    *
    * @return Boolean
    */
      dataModel.recentCheckoutComplete = function () {
        var t = new Date().getTime();
        var sleep = localStorageService.getValue('lqApp.Checkout.Timeout');
        return (sleep != null && t < sleep);
      };

    /**
    * checkEligibility
    *
    * Checks if the authenticated user is eligible for mobile checkout.
    */
      dataModel.checkEligibility = function () {
        //
        // Get auth token from session
        var authToken = window.sessionStorage.getItem('token');

        //
        // Check mobile checkout eligibility.
        console.log('Checking Mobile Checkout eligibility...');
        return mobileCheckoutEndpointService.getEligibility(authToken).then(

          //
          // Success
          function (eligibilityResponse) {
            var isValidResponse;
            dataModel.lastCheck = new Date().getTime();


            // Check Response is valid.
            if (eligibilityResponse.status == 200) {
              isSuccessfulResponse = true;
            }

            isValidResponse = eligibilityResponse.hasOwnProperty('data')
              && eligibilityResponse.data.hasOwnProperty('eligibility')
              && eligibilityResponse.data.eligibility == true;


            if (isValidResponse) {
              console.log('Eligible for Checkout...');
              dataModel.build(eligibilityResponse.data);
            }

            // return the user data same as previously
            return eligibilityResponse.data;
          },

          //
          // Failed to get eligibility, oh well return the user
          // and don't do anything else, we don't want to
          // interrupt the UX because of a service side error
          function (error) {
            dataModel.setEligibility(false);
            isSuccessfulResponse = false;
            return error;
          }

        );
      };

      return dataModel;
    }
  ]);
