/* global lqMobileModule, lqMobile*/
lqMobileModule.factory('Profile', ['lodash', 'PointsSummary',
  function (lodash, PointsSummary) {
    'use strict';

        /* Constructor */
    function Profile() {
      this.firstName = null;
      this.lastName = null;
      this.middleInitial = null;
      this.emailAddress = null;
      this.mobilePhone = null;
      this.secondPhone = null;
      this.streetAddress1 = null;
      this.streetAddress2 = null;
      this.city = null;
      this.state = null;
      this.country = null;
      this.zipCode = null;
      this.creditCard = {};
      this.creditCard.ccNumber = null;
      this.creditCard.ccMonth = lqMobile.Defaults.getDefaultCCMonth() + 1;
      this.creditCard.ccYear = lqMobile.Defaults.getDefaultCCYear();
      this.creditCard.ccType = null;
      this.companyName = null;
      this.returnsNumber = null;
      this.languagePrefCode = null;
      this.favoriteHotels = null;
      this.preferences = null;
      this.returnsTier = null;
      this.pointsSummary = PointsSummary.build();

      // JC: added with mobile checkout
      this.mobileCheckoutEligible = false;
      this.checkoutNumber = null;

      // 2017-02-06 JC: added for tracking military program
      this.enrolledPrograms = [];
      this.isMilitary = false;
    }

        /* Static call on object for a profile */
    Profile.build = function () {
      var profile = new Profile();
      Object.preventExtensions(profile);
      return profile;
    };

        /* Static call on object for a profile. Helps ensure correct object type.*/
    Profile.buildCopy = function (source) {
      var target = Profile.build();
      // sourceCopy eliminates reference issues
      var sourceCopy = JSON.parse(JSON.stringify(source));
      // copy all the fields

      // changing for in could have side effects
      for (var key in sourceCopy) { // eslint-disable-line
        target[key] = sourceCopy[key];
      }
      return target;
    };

    function formatMonth(month) {
      if (
        !lodash.isNull(month) && !lodash.isUndefined(month) && month.length === 2
        && !lodash.isUndefined(month.startsWith)
      ) {
        return month.startsWith('0') ? month.substring(1) : month;
      }
      return month;
    }

        /* jshint maxstatements: 40 */
    Profile.map = function (returns, creditCards) {
      var profile = Profile.build();
      var emailAddress;
      var mobilePhone;
      var homePhone;
      var creditCard;
      var expirationDate;
      var splitDate;
      var favHotelsArray = lodash.get(returns, 'favoriteHotels', []);
      var favHotelsArrayLength = favHotelsArray.length;
      var formattedFavArray = [];
      var index;
      var favHotel;
      var addressObj;
      var phoneObj;
      var hotelObj;
      var i;
      var iMax;
      var eachObj;
      var isUserActiveInProgram;
      var isMilitary;
      var isProgramActive;

      profile.firstName = returns.firstName;
      if (typeof profile.firstName === 'string') {
        profile.firstName = profile.firstName.trim();
      }
      profile.lastName = returns.lastName;
      if (typeof profile.lastName === 'string') {
        profile.lastName = profile.lastName.trim();
      }
      profile.middleInitial = returns.middleInitial;
      if (typeof profile.middleInitial === 'string') {
        profile.middleInitial = profile.middleInitial.trim();
      }
      profile.returnsNumber = lodash.get(returns, 'returnsNumber.id', null);
      profile.returnsTier = returns.returnsTier;
      profile.companyName = returns.employerCompanyName;
      profile.languagePrefCode = returns.languagePrefCode;

      profile.city = lodash.get(returns, 'address.city', null);
      profile.state = lodash.get(returns, 'address.stateProvince', null);
      profile.country = lodash.get(returns, 'address.countryCode', null);
      profile.streetAddress1 = lodash.get(returns, 'address.street', null);
      profile.streetAddress2 = lodash.get(returns, 'address.street2', null);
      profile.zipCode = lodash.get(returns, 'address.postalCode', null);

      emailAddress = lodash.findWhere(returns.storedEmails, {
        primary: true
      });
      profile.emailAddress = lodash.get(emailAddress, 'email.email', null);
      if (typeof profile.emailAddress === 'string') {
        profile.emailAddress = profile.emailAddress.trim();
      }

      mobilePhone = lodash.findWhere(returns.storedPhones, {
        phoneDesignation: 'MOBILE'
      });
      profile.mobilePhone = lodash.get(mobilePhone, 'phoneNumber.number', null);

      homePhone = lodash.findWhere(returns.storedPhones, {
        phoneDesignation: 'HOME'
      });
      profile.secondPhone = lodash.get(homePhone, 'phoneNumber.number', null);

      creditCard = lodash.findWhere(creditCards, {
        primary: true
      });
      if (!lodash.isUndefined(creditCard) && !lodash.isNull(creditCard)) {
        expirationDate = creditCard.creditCard.expirationDate;
        splitDate = expirationDate.split('-');
        profile.creditCard.ccNumber = lodash.get(creditCard, 'creditCard.cardNumber', null);
                // Remove any leading zeros from the month
        profile.creditCard.ccMonth = formatMonth(splitDate[1]);
        profile.creditCard.ccYear = parseInt(splitDate[0], 10);
        profile.creditCard.ccType = lodash.get(creditCard, 'creditCard.creditCardType', null);
      }

      for (index = 0; index < favHotelsArrayLength; index += 1) {
        favHotel = favHotelsArray[index];
        addressObj = favHotel.address;
        phoneObj = favHotel.phoneNumber;
        hotelObj = {
          name: favHotel.innName,
          id: favHotel.innNumber.id,
          address: addressObj.street + ', ' + addressObj.city + ', ' + addressObj.stateProvince + ', ' + addressObj.countryCode,
          phone: phoneObj.countryCode + '-' + phoneObj.areaCode + '-' + phoneObj.number,
          location: addressObj.city + ', ' + addressObj.stateProvinceDisplay
        };
        formattedFavArray.push(hotelObj);
      }

      profile.favoriteHotels = formattedFavArray;
      profile.preferences = lodash.get(returns, 'preferences', []);
      profile.pointsSummary.currentPointsBalance = lodash.get(returns, 'pointsSummary.currentPointsBalance', 0);
      profile.pointsSummary.nightsToNextMemberLevel = lodash.get(returns, 'pointsSummary.nightsToNextMemberLevel', 0);
      profile.pointsSummary.accountAdjustments = lodash.get(returns, 'pointsSummary.accountAdjustments', 0);
      profile.pointsSummary.pointsEarnedYTD = lodash.get(returns, 'pointsSummary.pointsEarnedYTD', 0);
      profile.pointsSummary.bonusPointsYTD = lodash.get(returns, 'pointsSummary.bonusPointsYTD', 0);
      profile.pointsSummary.pointsRedeemedYTD = lodash.get(returns, 'pointsSummary.pointsRedeemedYTD', 0);
      profile.pointsSummary.staysToNextMemberLevel = lodash.get(returns, 'pointsSummary.staysToNextMemberLevel', 0);

      profile.enrolledPrograms = lodash.get(returns, 'enrolledPrograms', []);

      for (i = 0, iMax = profile.enrolledPrograms.length; i < iMax; i += 1) {
        eachObj = profile.enrolledPrograms[i];
        isUserActiveInProgram = eachObj.hasOwnProperty('active') && (eachObj.active === true || eachObj.active === 'true');
        isMilitary = eachObj.hasOwnProperty('program') && eachObj.program.hasOwnProperty('name') && eachObj.program.name.toUpperCase() === 'MILITARY_REWARDS';
        isProgramActive = eachObj.hasOwnProperty('program') && eachObj.program.hasOwnProperty('active') && (eachObj.program.active === true || eachObj.program.active === 'true');
        if (isUserActiveInProgram && isMilitary && isProgramActive) {
          profile.isMilitary = true;
          break;
        }
      }

      return profile;
    };

        /**
        * @returns {boolean}
        */
    Profile.isMobileCheckoutEligible = function () {
      return this.mobileCheckoutEligible;
    };

    Profile.setMobileCheckoutEligibility = function (eligibility) {
      this.mobileCheckoutEligible = eligibility;
    };

        /**
         * Return the constructor function
         */
    return Profile;
  }]);
