/* global lqMobileModule*/
/* jshint maxstatements: 66 */
lqMobileModule.factory('ReservationFull', [
  function () {
    'use strict';

    /* Constructor */
    function Reservation() {
      this.cancelable = false;
      this.modifiable = false;
      this.roomType = null;
      this.subtotal = null;
      this.estimatedTax = null;
      this.estimatedTotal = null;
      this.totalPoints = 0;
      this.currency = null;
      this.adults = null;
      this.rateType = null;
      this.promoCode = null;
      this.firstName = null;
      this.lastName = null;
      this.zipCode = null;
      this.mobilePhone = null;
      this.emailAddress = null;
      this.selectedRoom = {
        hotelRoomRate: {
          totalRate: {
            amount: null,
            currencyCode: null
          },
          totalRateWithTaxes: {
            amount: null,
            currencyCode: null
          },
          roomType: {
            roomDescription: null,
            roomCode: null
          },
          nightlyRates: null,
          computedRates: {
            totalPoints: 0,
            totalRate: 0
          }
        }
      };
      this.hotelId = null;
      this.arrivalDate = null;
      this.departureDate = null;
      this.status = null;
      this.confirmationNumber = null;
      this.creditCard = {
        ccNumber: null,
        ccYear: null,
        ccMonth: null,
        ccType: null
      };
      this.rateQualifier = null;
      this.quotedTotalRate = null;
      this.quotedTotalRateWithTaxes = null;
      this.roomReservations = null;
      this.city = null;
      this.stateProvinceDisplay = null;
    }

    /* Static call on object for a reservation */
    Reservation.build = function () {
      return new Reservation();
    };

    function getComputedRates(ratesArray) {
      var ratesArrayLength = ratesArray.length;
      var result = {
        totalRate: 0,
        totalPoints: 0
      };
      var pos;
      var rateObj;
      for (pos = 0; pos < ratesArrayLength; pos += 1) {
        rateObj = ratesArray[pos];
        if (rateObj.hasOwnProperty('nightlyRate')) {
          result.totalRate += rateObj.nightlyRate.amount;
          result.totalPoints += rateObj.points;
        }
      }
      return result;
    }

    Reservation.map = function (reservationData) {
      var reservation = Reservation.build();
      var paymentInfoArray;
      var paymentInfoArrayLength;
      var index;
      var paymentInfo;
      var arrivalDateArray;
      var departureDateArray;
      var expirationDate;

      reservation.cancelable = reservationData.reservation.cancelable;
      reservation.modifiable = reservationData.reservation.modifiable;
      reservation.roomType =
        reservationData.reservation.roomReservations[0].roomType.roomDescription;
      reservation.subtotal = reservationData.reservation.quotedTotalRate.amount;
      reservation.estimatedTotal = reservationData.reservation.quotedTotalRateWithTaxes.amount;
      reservation.estimatedTax = reservation.estimatedTotal - reservation.subtotal;
      reservation.currency = reservationData.reservation.quotedTotalRate.currencyCode;
      reservation.adults = reservationData.reservation.roomReservations[0].numberOfAdults;
      reservation.rateType = reservationData.reservation.rateQualifier.rateCode.type === 'CORPORATE' ? reservationData.reservation.rateQualifier.rateCode.type : reservationData.reservation.rateQualifier.rateCode.code;
      reservation.firstName = reservationData.reservation.mainContact.firstName;
      reservation.lastName = reservationData.reservation.mainContact.lastName;
      reservation.zipCode = reservationData.reservation.mainContact.postalAddress.postalCode;
      reservation.mobilePhone =
        reservationData.reservation.mainContact.phoneNumbers[0].phoneNumber.number;
      reservation.emailAddress =
        reservationData.reservation.mainContact.emailAddress[0].email.email;

      reservation.city = reservationData.reservation.mainContact.postalAddress.city;
      reservation.country = reservationData.reservation.mainContact.postalAddress.countryCode;
      reservation.state = reservationData.reservation.mainContact.postalAddress.stateProvince;
      reservation.streetAddress1 = reservationData.reservation.mainContact.postalAddress.street;
      reservation.streetAddress2 = reservationData.reservation.mainContact.postalAddress.street2;

      reservation.selectedRoom.hotelRoomRate.totalRate =
        reservationData.reservation.quotedTotalRate;
      reservation.selectedRoom.hotelRoomRate.totalRateWithTaxes =
        reservationData.reservation.quotedTotalRateWithTaxes;
      reservation.selectedRoom.hotelRoomRate.roomType =
        reservationData.reservation.roomReservations[0].roomType;
      reservation.selectedRoom.hotelRoomRate.nightlyRates =
        reservationData.reservation.roomReservations[0].nightlyRoomRates;
      reservation.selectedRoom.hotelRoomRate.computedRates =
        getComputedRates(reservationData.reservation.roomReservations[0].nightlyRoomRates);

      if (reservation.rateType === 'CORPORATE') {
        reservation.promoCode = reservationData.reservation.rateQualifier.rateCode.code;
      }

      reservation.hotelId = reservationData.reservation.innNumber.id;
      if (reservationData.reservation.arrivalDate) {
        arrivalDateArray = reservationData.reservation.arrivalDate.split('-');
        if (arrivalDateArray.length === 3) {
          reservation.arrivalDate = new Date(
            arrivalDateArray[0], arrivalDateArray[1] - 1, arrivalDateArray[2]
          );
        }
      }
      if (reservationData.reservation.departureDate) {
        departureDateArray = reservationData.reservation.departureDate.split('-');
        if (departureDateArray.length === 3) {
          reservation.departureDate = new Date(
            departureDateArray[0], departureDateArray[1] - 1, departureDateArray[2]
          );
        }
      }
      reservation.status = reservationData.reservation.status.id;
      reservation.confirmationNumber = reservationData.reservation.confirmationNumber.number;


      paymentInfoArray = reservationData.reservation.paymentInfo;
      if (paymentInfoArray) {
        paymentInfoArrayLength = paymentInfoArray.length;
        for (index = 0; index < paymentInfoArrayLength; index += 1) {
          paymentInfo = paymentInfoArray[index];
          if (paymentInfo.type === 'FreeNightCertificate') {
            reservation.totalPoints = paymentInfo.certificateNumber.points;
          } else if (paymentInfo.type === 'CreditCard') {
            reservation.creditCard.ccNumber = paymentInfo.creditCard.cardNumber;
            if (paymentInfo.creditCard.expirationDate) {
              expirationDate = paymentInfo.creditCard.expirationDate.split('-');
              reservation.creditCard.ccMonth = parseInt(expirationDate[1], 10);
              reservation.creditCard.ccYear = parseInt(expirationDate[0], 10);
              reservation.creditCard.ccType = paymentInfo.creditCard.creditCardType;
            }
          }
        }
      }

      reservation.rateQualifier = reservationData.reservation.rateQualifier;
      reservation.quotedTotalRate = reservationData.reservation.quotedTotalRate;
      reservation.quotedTotalRateWithTaxes = reservationData.reservation.quotedTotalRateWithTaxes;
      reservation.roomReservations = reservationData.reservation.roomReservations;

      reservation.hotelName = reservationData.hotelSummary.name;
      reservation.hotelAddress = reservationData.hotelSummary.address.street + ', ' + reservationData.hotelSummary.address.city + ', ' + reservationData.hotelSummary.address.stateProvince;
      reservation.hotelStreetAddress1 = reservationData.hotelSummary.address.street;
      reservation.hotelStreetAddress2 = reservationData.hotelSummary.address.street2;
      reservation.hotelCity = reservationData.hotelSummary.address.city;
      reservation.hotelCountry = reservationData.hotelSummary.address.countryCode;
      reservation.hotelState = reservationData.hotelSummary.address.stateProvince;
      reservation.hotelStateProvinceDisplay =
        reservationData.hotelSummary.address.stateProvinceDisplay;
    // Check if hotel has a dedicated mobile-web phone
      if (reservationData.hotelSummary.phoneNumbers.MOBILE_WEB) {
        reservation.hotelPrimaryPhone = reservationData.hotelSummary.primaryPhoneNumber.countryCode + '-' + reservationData.hotelSummary.phoneNumbers.MOBILE_WEB.phoneNumber.number;
      } else {
        reservation.hotelPrimaryPhone = reservationData.hotelSummary.primaryPhoneNumber.countryCode + '-' + reservationData.hotelSummary.primaryPhoneNumber.areaCode + '-' + reservationData.hotelSummary.primaryPhoneNumber.number;
      }
      reservation.hotelLocation = reservationData.hotelSummary.location;

      return reservation;
    };


  /**
   * Return the constructor function
   */
    return Reservation;
  }]);
