/* global lqMobileModule*/
lqMobileModule.factory('ReservationSummary', [
  function () {
    'use strict';

        /* Constructor */
    function Reservation() {
      this.hotelName = null;
      this.hotelId = null;
      this.hotelAddress = null;
      this.street = null;
      this.city = null;
      this.stateProvinceDisplay = null;
      this.stateProvince = null;
      this.primaryPhone = null;
      this.arrivalDate = null;
      this.departureDate = null;
      this.status = null;
      this.location = null;
      this.confirmationNumber = null;
      this.firstName = null;
      this.lastName = null;
    }

        /* Static call on object for a reservation */
    Reservation.build = function () {
      var reservation = new Reservation();
      return reservation;
    };

    Reservation.map = function (reservationData) {
      var reservation = Reservation.build();
      var arrivalDateArray;
      var departureDateArray;

      reservation.hotelName = reservationData.hotelSummary.name;
      reservation.hotelId = reservationData.hotelSummary.innNumber.id;
      reservation.hotelAddress = reservationData.hotelSummary.address.street + ', ' + reservationData.hotelSummary.address.city + ', ' + reservationData.hotelSummary.address.stateProvince;
      reservation.hotelStreet = reservationData.hotelSummary.address.street;
      reservation.hotelCity = reservationData.hotelSummary.address.city;
      reservation.hotelStateProvince = reservationData.hotelSummary.address.stateProvince;
      reservation.hotelStateProvinceDisplay =
        reservationData.hotelSummary.address.stateProvinceDisplay;
            // Check if hotel has a dedicated mobile-web phone
      if (reservationData.hotelSummary.phoneNumbers.MOBILE_WEB) {
        reservation.hotelPrimaryPhone = reservationData.hotelSummary.primaryPhoneNumber.countryCode + '-' + reservationData.hotelSummary.phoneNumbers.MOBILE_WEB.phoneNumber.number;
      } else {
        reservation.hotelPrimaryPhone = reservationData.hotelSummary.primaryPhoneNumber.countryCode + '-' + reservationData.hotelSummary.primaryPhoneNumber.areaCode + '-' + reservationData.hotelSummary.primaryPhoneNumber.number;
      }
      if (reservationData.arrivalDate) {
        arrivalDateArray = reservationData.arrivalDate.split('-');
        if (arrivalDateArray.length === 3) {
          reservation.arrivalDate = new Date(
            arrivalDateArray[0], arrivalDateArray[1] - 1, arrivalDateArray[2]
          );
        }
      }
      if (reservationData.departureDate) {
        departureDateArray = reservationData.departureDate.split('-');
        if (departureDateArray.length === 3) {
          reservation.departureDate = new Date(
            departureDateArray[0], departureDateArray[1] - 1, departureDateArray[2]
          );
        }
      }

      reservation.status = reservationData.status.id;
      reservation.hotelLocation = reservationData.hotelSummary.location;
      reservation.confirmationNumber = reservationData.confirmationNumber.number;
      reservation.firstName = reservationData.firstName;
      reservation.lastName = reservationData.lastName;

      return reservation;
    };

        /**
         * Return the constructor function
         */
    return Reservation;
  }]);
