/* global lqMobileModule*/
/* jshint maxparams: 10 */
/* jshint maxstatements: 50 */
lqMobileModule.factory('Returns', [
  'EnrollmentMethod',
  'StoredEmail',
  'StoredPhone',
  'PostalAddress',
  'Profile',
  'ReturnsNumber',
  'lodash',
  function (
    EnrollmentMethod,
    StoredEmail,
    StoredPhone,
    PostalAddress,
    Profile,
    ReturnsNumber,
    lodash
  ) {
    'use strict';

        /* Constructor */
    function Returns() {
      this.address = null;
      this.employerCompanyName = null;
      this.enrollmentMethod = null;
      this.favoriteHotels = [];
      this.preferences = [];
      this.languagePrefCode = null;
      this.firstName = null;
      this.lastName = null;
      this.middleInitial = null;
      this.storedEmails = [];
      this.storedPhones = [];
    }

        /* Static call on object for a returns */
    Returns.build = function () {
      return new Returns();
    };

    Returns.map = function (data) {
      var returns = Returns.build();
      var enrollmentMethod = EnrollmentMethod.build();
      var storedEmail = StoredEmail.build();
      var homePhone = StoredPhone.build();
      var mobilePhone = StoredPhone.build();
      var postalAddress = PostalAddress.build();
      var returnsNumber;
      var favHotels = lodash.get(data, 'favoriteHotels', []);
      var favHotelsLength = favHotels.length;
      var formattedFavHotels = [];
      var index;
      var favHotel;
      var favHotelInn;
      var favHotelObject;

      if (!(data instanceof Profile)) {
        throw 'Unsupported mapping to Returns';
      }

      returns.storedEmails.push(storedEmail);

      if (typeof data.emailAddress === 'string') {
        data.emailAddress = data.emailAddress.trim();
      }
      storedEmail.email.email = data.emailAddress;

      if (StoredPhone.isNumberValid(data.secondPhone)) {
        returns.storedPhones.push(homePhone);
        homePhone.phoneNumber.number = data.secondPhone;
        homePhone.phoneDesignation = 'HOME';
        homePhone.primary = !StoredPhone.isNumberValid(data.mobilePhone);
      }

      if (StoredPhone.isNumberValid(data.mobilePhone)) {
        returns.storedPhones.push(mobilePhone);
        mobilePhone.phoneNumber.number = data.mobilePhone;
        mobilePhone.phoneDesignation = 'MOBILE';
        mobilePhone.primary = true;
      }

      returns.enrollmentMethod = enrollmentMethod;
      enrollmentMethod.channel = 'MOBILE';
      enrollmentMethod.source = 'UXP';
      enrollmentMethod.campaign.code = 'WEB';

      if (typeof data.firstName === 'string') {
        data.firstName = data.firstName.trim();
      }
      returns.firstName = data.firstName;
      if (typeof data.lastName === 'string') {
        data.lastName = data.lastName.trim();
      }
      returns.lastName = data.lastName;

      if (typeof data.middleInitial === 'string') {
        data.middleInitial = data.middleInitial.trim();
      }
      returns.middleInitial = data.middleInitial;

      returns.languagePrefCode = data.languagePrefCode;

      returnsNumber = lodash.get(data, 'returnsNumber', null);
      if (!lodash.isNull(returnsNumber)) {
        returns.returnsNumber = ReturnsNumber.build(returnsNumber);
      } else {
        returnsNumber = null;
      }

      returns.address = postalAddress;
      postalAddress.postalCode = data.zipCode;
      postalAddress.city = lodash.get(data, 'city', 'UNKNOWN');
      postalAddress.street = lodash.get(data, 'streetAddress1', 'UNKNOWN');
      postalAddress.street2 = lodash.get(data, 'streetAddress2', null);
      postalAddress.stateProvince = lodash.get(data, 'state', 'XX');
      postalAddress.countryCode = lodash.get(data, 'country', 'XX');

      returns.employerCompanyName = lodash.get(data, 'companyName', null);

      for (index = 0; index < favHotelsLength; index += 1) {
        favHotel = favHotels[index];
        favHotelInn = { id: favHotel.id };
        favHotelObject = { innNumber: favHotelInn, localPreferences: [] };
        formattedFavHotels.push(favHotelObject);
      }
      returns.favoriteHotels = formattedFavHotels;

      returns.preferences = lodash.get(data, 'preferences', []);

      return {
        Returns: returns
      };
    };

        /**
         * Return the constructor function
         */
    return Returns;
  }]);
