/* global lqMobileModule, LQMobile*/
lqMobileModule.factory('StoredCreditCard', ['Profile', 'lodash',
  function (Profile, lodash) {
    'use strict';

        /* Constructor */
    function StoredCreditCard() {
      this.creditCard = {
        cardNumber: null,
        creditCardLastFour: null,
        creditCardType: null,
        expirationDate: null,
        nameOnCard: null,
        billingAddress: {
          city: null,
          countryCode: null,
          postalCode: null,
          stateProvince: null,
          street: null,
          street2: null
        }
      };
      this.creditCardDesignation = 'PERSONAL';
      this.description = null;
      this.primary = true;
      this.buildRequest = function () {
        return {
          StoredCreditCard: this
        };
      };
      this.isCardNumberEmpty = function () {
        return lodash.isNull(this.creditCard.cardNumber) ||
                    lodash.isUndefined(this.creditCard.cardNumber) ||
                    this.creditCard.cardNumber.length === 0;
      };
      this.isCardTypeEmpty = function () {
        return lodash.isNull(this.creditCard.creditCardType) ||
                    lodash.isUndefined(this.creditCard.creditCardType) ||
                    this.creditCard.creditCardType.length === 0;
      };
      this.isValid = function () {
        return !this.isCardNumberEmpty() &&
                    !this.isCardTypeEmpty() &&
                    this.creditCard.creditCardType !== 'UNKNOWN';
      };
    }

        /* Static call on object for an cc */
    StoredCreditCard.build = function () {
      return new StoredCreditCard();
    };

    function formatMonth(month) {
      if (!lodash.isNull(month) && !lodash.isUndefined(month) && month.toString().length === 1) {
        return '0' + month;
      }
      return month;
    }

    StoredCreditCard.map = function (profile) {
      var storedCreditCard = StoredCreditCard.build();
      var cardNumber;

      if (!(profile instanceof Profile)) {
        throw 'Unsupported mapping to StoredCreditCard';
      }

      cardNumber = lodash.get(profile, 'creditCard.ccNumber', null);
      if (!lodash.isNull(cardNumber)) {
        storedCreditCard.creditCard.cardNumber = cardNumber;
        storedCreditCard.creditCard.creditCardLastFour = cardNumber.substr(cardNumber.length - 4);

        // If the cardNumber is NOT A NUMBER, then use the existing creditCardType on the profile.
        // Otherwise, get the number from the
        if (isNaN(cardNumber)) {
          storedCreditCard.creditCard.creditCardType = profile.creditCard.ccType;
        } else {
          storedCreditCard.creditCard.creditCardType = LQMobile.Util.findCCType(cardNumber);
        }
      }

      storedCreditCard.creditCard.expirationDate = lodash.get(profile, 'creditCard.ccYear', null) + '-' + formatMonth(lodash.get(profile, 'creditCard.ccMonth', null));
      storedCreditCard.creditCard.nameOnCard = profile.firstName + ' ' + profile.lastName;
      storedCreditCard.creditCard.billingAddress.city = lodash.get(profile, 'city', 'UNKNOWN');
      storedCreditCard.creditCard.billingAddress.countryCode = lodash.get(profile, 'country', 'XX');
      storedCreditCard.creditCard.billingAddress.postalCode = profile.zipCode;
      storedCreditCard.creditCard.billingAddress.stateProvince = lodash.get(profile, 'state', 'XX');
      storedCreditCard.creditCard.billingAddress.street = lodash.get(profile, 'streetAddress1', 'UNKNOWN');
      storedCreditCard.creditCard.billingAddress.street2 = lodash.get(profile, 'streetAddress2', null);
      storedCreditCard.creditCardDesignation = 'PERSONAL';
      storedCreditCard.description = null;
      storedCreditCard.primary = true;

      return storedCreditCard;
    };

        /**
         * Return the constructor function
         */
    return StoredCreditCard;
  }]);
