/* global lqMobileModule, LQMobile */
lqMobileModule.factory(
  'HotelAvailabilityRequest',
  [
    'currencyService',
    function (currencyService) {
      'use strict';

      /* Constructor */
      function HotelAvailabilityRequest() {
        this.arrivalDate = null;
        this.departureDate = null;

        this.numberOfAdults = null;
        this.numberOfChildren = null;
        this.numberOfRooms = null;
        this.currencyCode = currencyService.getActiveCurrency();

        this.rateCodes = [];
      }

        /* Static call on object for a plain reservation request */
      HotelAvailabilityRequest.build = function () {
        return new ReservationRequest();
      };

      function buildRequest(searchData) {
        var hotelAvailabilityRequest = new HotelAvailabilityRequest();
        var defaultRate = LQMobile.Util.getDefaultRate();

        hotelAvailabilityRequest.arrivalDate = LQMobile.Util.dateToString(searchData.checkinDate);
        hotelAvailabilityRequest.departureDate = LQMobile.Util.dateToString(searchData.checkoutDate);

        hotelAvailabilityRequest.numberOfAdults = searchData.numberOfAdults;
        hotelAvailabilityRequest.numberOfChildren = searchData.numberOfChildren;
        hotelAvailabilityRequest.numberOfRooms = searchData.numberOfRooms;

        if (searchData.rateType === 'CORPORATE') {
          hotelAvailabilityRequest.rateCodes.push({
            code: searchData.promoCode,
            type: 'CORPORATE'
          });
          hotelAvailabilityRequest.rateCodes.push({
            code: defaultRate,
            type: LQMobile.Util.getRateCode(defaultRate)
          });
        } else {
          hotelAvailabilityRequest.rateCodes.push({
            code: searchData.rateType,
            type: LQMobile.Util.getRateCode(searchData.rateType)
          });
        }

        return hotelAvailabilityRequest;
      }

      HotelAvailabilityRequest.buildRequestByLocation = function (searchData, location) {
        var hotelAvailabilityRequest = buildRequest(searchData);
        hotelAvailabilityRequest.proximities = [{
          location: {
            latitude: location.lat(),
            longitude: location.lng()
          },
          range: {
            range: searchData.range,
            unit: 'MILE'
          }
        }];

        return {
          LocationAvailabilityRequest: hotelAvailabilityRequest
        };
      };

      HotelAvailabilityRequest.buildRequestById = function (searchData, hotelId) {
        var hotelAvailabilityRequest = buildRequest(searchData);
        hotelAvailabilityRequest.propertyIds = [{
          id: hotelId
        }];

        return {
          LocationAvailabilityRequest: hotelAvailabilityRequest
        };
      };


        /**
         * Return the constructor function
         */
      return HotelAvailabilityRequest;
    }
  ]
);
