/* global lqMobileModule, LQMobile */
/* jshint maxparams: 10 */
/* jshint maxstatements: 50 */
lqMobileModule.factory('Reservation', function () {
  'use strict';

    /* Constructor */
  function Reservation() {
    this.channel = 'MOBILE_APP';
    this.arrivalDate = null;
    this.departureDate = null;
    this.iata = null;
    this.innNumber = {
      id: null
    };
    this.mainContact = {
      emailAddress: [],
      suffix: '',
      firstName: null,
      lastName: null,
      middleName: '',
      phoneNumbers: [],
      postalAddress: null,
      returnsNumber: null,
      guestSpecialRequest: null
    };
    this.paymentInfo = [];
    this.quotedTotalRate = {
      amount: null,
      currencyCode: null
    };
    this.quotedTotalRateWithTaxes = {
      amount: null,
      currencyCode: null
    };
    this.rateQualifier = null;
    this.roomReservations = [];
    this.specialRequests = null;
    this.language = {
      code: 'en'
    };
    this.confirmationNumber = {
      number: null
    };
  }

    /* Static call on object for a plain reservation request */
  Reservation.build = function () {
    return new ReservationRequest();
  };

    /* Static call on object for a reservation request for a normal booking path*/
  Reservation.buildReservationRequest = function (
    searchData, mainContact, paymentInfoArray, hotelData, specialRequests, selConfirmationNumber
  ) {
    var reservation = new Reservation();
    var roomReservation = {};

    reservation.arrivalDate = LQMobile.Util.dateToString(searchData.checkinDate);
    reservation.departureDate = LQMobile.Util.dateToString(searchData.checkoutDate);
    reservation.innNumber.id = hotelData.hotelDetails.hotelId;
    reservation.mainContact = mainContact;
    reservation.paymentInfo = paymentInfoArray;
    reservation.quotedTotalRate.amount = searchData.selectedRoom.hotelRoomRate.totalRate.amount;
    reservation.quotedTotalRate.currencyCode =
      searchData.selectedRoom.hotelRoomRate.totalRateWithTaxes.currencyCode;
    reservation.quotedTotalRateWithTaxes.amount =
      searchData.selectedRoom.hotelRoomRate.totalRateWithTaxes.amount;
    reservation.quotedTotalRateWithTaxes.currencyCode =
      searchData.selectedRoom.hotelRoomRate.totalRateWithTaxes.currencyCode;
    reservation.rateQualifier = searchData.selectedRoom.hotelRoomRate.rateQualifier;
    roomReservation.numberOfAdults = searchData.numberOfAdults;
    roomReservation.numberOfChildren = searchData.numberOfChildren;
    roomReservation.numberOfRooms = searchData.numberOfRooms;
    roomReservation.roomType = searchData.selectedRoom.hotelRoomRate.roomType;
    roomReservation.nightlyRoomRates = searchData.selectedRoom.hotelRoomRate.nightlyRates;
    reservation.roomReservations.push(roomReservation);
    reservation.confirmationNumber.number = selConfirmationNumber;
    reservation.specialRequests = specialRequests;

    return {
      Reservation: reservation
    };
  };

    /**
     * Return the constructor function
     */
  return Reservation;
});
