/* global lqMobileModule, LQMobile */
/* jshint maxparams: 10 */
/* jshint maxstatements: 50 */
lqMobileModule.factory('ReservationRequest', function (GuestEmail,
  GuestPhone,
  Country,
  PostalAddress,
  PaymentInfo,
  RateQualifier,
  RateCode,
  RatePolicy,
  PhoneGuarantee,
  Notification) {
  'use strict';

    /* Constructor */
  function ReservationRequest() {
    this.channel = 'MOBILE_APP';
    this.arrivalDate = null;
    this.departureDate = null;
    this.iata = null;
    this.innNumber = {
      id: null
    };
    this.mainContact = {
      emailAddress: [],
      suffix: '',
      firstName: null,
      lastName: null,
      middleName: '',
      phoneNumbers: [],
      postalAddress: null,
      returnsNumber: null,
      guestSpecialRequest: null
    };
    this.paymentInfo = [];
    this.quotedTotalRate = {
      amount: null,
      currencyCode: null
    };
    this.quotedTotalRateWithTaxes = {
      amount: null,
      currencyCode: null
    };
    this.rateQualifier = null;
    this.roomReservations = [];
    this.specialRequests = null;
    this.language = {
      code: 'en'
    };
  }

    /* Static call on object for a plain reservation request */
  ReservationRequest.build = function () {
    return new ReservationRequest();
  };

    /* Static call on object for a reservation request for a normal booking path*/
  ReservationRequest.buildReservationRequest = function (
    searchData, mainContact, paymentInfoArray, hotelData, specialRequests
  ) {
    var reservationRequest = new ReservationRequest();
    var roomReservation = {};

    reservationRequest.arrivalDate = LQMobile.Util.dateToString(searchData.checkinDate);
    reservationRequest.departureDate = LQMobile.Util.dateToString(searchData.checkoutDate);
    reservationRequest.innNumber.id = hotelData.hotelDetails.hotelId;
    reservationRequest.mainContact = mainContact;
    reservationRequest.paymentInfo = paymentInfoArray;
    reservationRequest.quotedTotalRate.amount =
      searchData.selectedRoom.hotelRoomRate.totalRate.amount;
    reservationRequest.quotedTotalRate.currencyCode =
      searchData.selectedRoom.hotelRoomRate.totalRateWithTaxes.currencyCode;
    reservationRequest.quotedTotalRateWithTaxes.amount =
      searchData.selectedRoom.hotelRoomRate.totalRateWithTaxes.amount;
    reservationRequest.quotedTotalRateWithTaxes.currencyCode =
      searchData.selectedRoom.hotelRoomRate.totalRateWithTaxes.currencyCode;
    reservationRequest.rateQualifier = searchData.selectedRoom.hotelRoomRate.rateQualifier;
    roomReservation.numberOfAdults = searchData.numberOfAdults;
    roomReservation.numberOfChildren = searchData.numberOfChildren;
    roomReservation.numberOfRooms = searchData.numberOfRooms;
    roomReservation.roomType = searchData.selectedRoom.hotelRoomRate.roomType;
    roomReservation.nightlyRoomRates = searchData.selectedRoom.hotelRoomRate.nightlyRates;
    reservationRequest.roomReservations.push(roomReservation);
    reservationRequest.specialRequests = specialRequests;

    return {
      ReservationRequest: reservationRequest
    };
  };

    /* Static call on object for an instant hold reservation request */
  ReservationRequest.buildInstantHold = function () {
    var instantHold = new ReservationRequest();
    var phoneNumber = GuestPhone.build();
    var country = Country.build();
    var email = GuestEmail.build();
    var postalAddress = PostalAddress.build();
    var phoneGuarantee = PhoneGuarantee.build();
    var rateQualifier = RateQualifier.build();
    var rateCode = RateCode.build();
    var depositPolicy = RatePolicy.build();
    var cancelPolicy = RatePolicy.build();
    var guarenteePolicy = RatePolicy.build();
    var notification = Notification.build();

        // set the channel
    instantHold.channel = 'MOBILE_APP';

        // set the email
    email.email = 'reservations@laquinta.com';
    email.emailDesignation = 'PERSONAL';
    email.primary = true;
    instantHold.mainContact.emailAddress.push(email);

        // set the main contact
    instantHold.mainContact.firstName = 'LQ';
    instantHold.mainContact.lastName = 'INSTANTHOLD';

        // set the phone number
    phoneNumber.phoneDesignation = 'MOBILE';
    country.code = 'US';
    country.description = 'United States';
    phoneNumber.phoneNumber.country = country;
    phoneNumber.phoneNumber.countryCode = '1';
    phoneNumber.primary = true;
    notification.code = 'InstantHold';
    phoneNumber.notifications.push(notification);
    instantHold.mainContact.phoneNumbers.push(phoneNumber);

        // set the postal address
    postalAddress.city = 'UNKNOWN';
    postalAddress.countryCode = 'XX';
    postalAddress.postalCode = 'UNKNOWN';
    postalAddress.stateProvince = 'XX';
    postalAddress.street = 'UNKNOWN';
    instantHold.mainContact.postalAddress = postalAddress;

        // set the payment info
    instantHold.paymentInfo.push(phoneGuarantee);

        // set the rate code
    rateCode.code = 'LQHOLD';
    rateCode.type = 'INSTANT_HOLD';
    rateQualifier.rateCode = rateCode;
    instantHold.rateQualifier = rateQualifier;

        // set the deposit policy
    depositPolicy.details = 'CREDIT CARD GUARANTEE';
    depositPolicy.id = 'CC';
    depositPolicy.category = 'DEPOSIT_POLICY';
    instantHold.rateQualifier.ratePolicy.push(depositPolicy);

        // set the cancel policy
    cancelPolicy.details = 'Reservations must be cancelled prior to the cancellation policy deadline specified at the time of booking to avoid penalty fees.';
    cancelPolicy.id = '6PM';
    cancelPolicy.category = 'CANCEL_POLICY';
    instantHold.rateQualifier.ratePolicy.push(cancelPolicy);

        // set the guarentee policy
    guarenteePolicy.details = 'CREDIT CARDS: AX CB DC DS MC VS VI UT CA';
    guarenteePolicy.id = 'CC';
    guarenteePolicy.category = 'GUARANTEE_POLICY';
    instantHold.rateQualifier.ratePolicy.push(guarenteePolicy);

    return instantHold;
  };

    /**
     * Return the constructor function
     */
  return ReservationRequest;
});
