/* global lqMobileModule, googleMapsModelService*/
/* jshint maxparams: 7 */
lqMobileModule.factory('omniBoxModelService', ['$q', 'googleMapsEndpointService', 'omniBoxEndpointService', 'searchParamModelService', 'persistentStorageModelService', 'googleFilterModel', 'localInfoModelService',
  function (
    $q,
    googleMapsEndpointService,
    omniBoxEndpointService,
    searchParamModelService,
    persistentStorageModelService,
    googleFilterModel,
    localInfoModelService
  ) {
    'use strict';

    var modelService = {};
    var initDeferred;

    function processPlacesSearchBox(predictions) {
      modelService.deferredPlaces.resolve(predictions.slice(0, 10));
    }

    function autoCompleteSearch(placeIn) {
      modelService.deferredPlaces = $q.defer();
      modelService.data.autocomplete.getPlacePredictions(placeIn, processPlacesSearchBox);
      return modelService.deferredPlaces.promise;
    }

    function addPreviousSearch(search) {
      localInfoModelService.addPreviousSearch(search);
    }

    modelService.data = {
      initPromise: null,
      autocomplete: null,
      geocoder: null,
      placesPredictions: {},
      placesPredictionsPromises: {},
      showedPlacesPredictions: []
    };

    modelService.data.localInfo = localInfoModelService.data;

    modelService.clearPredictions = function () {
      modelService.data.showedPlacesPredictions = [];
    };

    modelService.selectPlace = function (address) {
      var filteredAddress = googleFilterModel.filterTerm(address);
      modelService.clearPredictions();
      searchParamModelService.setCurrentPlace(filteredAddress);
      addPreviousSearch(filteredAddress);
    };

    modelService.selectHotel = function (hotel) {
      var deferred = $q.defer();
      var propId;

      if (hotel.propertyId) {
        propId = hotel.propertyId;
      } else {
        propId = hotel.property_id;
      }

      modelService.clearPredictions();
      searchParamModelService.setCurrentHotel(propId).then(function () {
        googleMapsModelService.zoomToHotel(propId);
        deferred.resolve();
      });

      return deferred.promise;
    };

    modelService.selectLocation = function (latitude, longitude) {
      searchParamModelService.setCurrentLocation(latitude, longitude);
    };

    modelService.loadPlacesSuggestions = function (query) {
      var placeIn = {
        input: query,
        componentRestrictions: {
          country: 'us'
        }
      };
      modelService.data.initPromise.then(function () {
        autoCompleteSearch(placeIn).then(function (predictions) {
          modelService.data.placesPredictions = predictions;
        });
      });
    };

    function filterLocations(filter) {
      var firstChar = filter.charAt(0).toUpperCase();
      var key;
      modelService.data.showedPlacesPredictions = [];
      // disable es lint for for in loop, as changing has side ffects
      for (key in modelService.data.placesPredictions[firstChar]) { // eslint-disable-line
        if (
            key !== 'jcr:primaryType' &&
            modelService.data.placesPredictions[firstChar].hasOwnProperty(key) &&
            modelService.data.placesPredictions[firstChar][key].NAME.toLowerCase()
              .indexOf(filter.toLowerCase()) === 0 &&
            modelService.data.showedPlacesPredictions.length < 12
        ) {
          modelService.data.showedPlacesPredictions.push(
            modelService.data.placesPredictions[firstChar][key]
          );
        }
      }
    }

    modelService.getPlacesSuggestions = function (query) {
      var firstLetter = query.charAt(0).toUpperCase();
      if (modelService.data.placesPredictionsPromises[firstLetter]) {
        modelService.data.placesPredictionsPromises[firstLetter].then(function (data) {
          modelService.data.placesPredictions[firstLetter] = data;
          filterLocations(query);
        });
      } else {
        modelService.data.placesPredictionsPromises[firstLetter] =
          omniBoxEndpointService.getDestinationsAutoComplete(firstLetter);
        modelService.data.placesPredictionsPromises[firstLetter].then(function (data) {
          modelService.data.placesPredictions[firstLetter] = data;
          filterLocations(query);
        });
      }
    };

    initDeferred = $q.defer();
    modelService.data.initPromise = initDeferred.promise;
    googleMapsEndpointService.loadGoogleMapsAPI().then(function () {
      modelService.data.autocomplete = googleMapsEndpointService.addAutocomplete();
      initDeferred.resolve();
    });

    return modelService;
  }]);
