/* global lqMobileModule, LQMobile, $, navigator, isEditMode */
/* jshint maxparams: 10 */
lqMobileModule.controller('omniboxController',
  [
    '$scope',
    '$rootScope',
    '$timeout',
    'omniBoxModelService',
    'routingService',
    'searchParamModelService',
    '$location',
    '$modal',
    'events',
    'editSearchPanelCollapseFactory',
    'geolocationService',
    'satelliteTrackingModelService',
    'lodash',
    function (
      $scope,
      $rootScope,
      $timeout,
      omniBoxModelService,
      routingService,
      searchParamModelService,
      $location,
      $modal,
      events,
      editSearchPanelCollapseFactory,
      geolocationService,
      satelliteTrackingModelService,
      lodash
    ) {
      'use strict';

      $scope.showSuggestions = false;
      $scope.boxCleared = false;
      $scope.showDefaultSuggestions = true;
      $scope.predictions = omniBoxModelService.data;
      $scope.location = null;

      $scope.init = function (nearByLabel) {
        var currentPlace = searchParamModelService.data.currentPlace;
        $scope.nearByLabel = nearByLabel;

        if ((typeof currentPlace !== 'undefined') && (currentPlace !== null) && (currentPlace !== '')) {
          $scope.omniSearchTerm = currentPlace;
        } else if (searchParamModelService.data.searchType === 'location') {
          $scope.omniSearchTerm = $scope.nearByLabel;
        }
      };

      $scope.$on('hideSuggestions', function () {
        $scope.hideSuggestionBox();
      });

      $scope.omniBoxSearch = function () {
        $timeout(function () {
          var searchTerm = $scope.omniSearchTerm;
          if ((typeof searchTerm !== 'undefined') && (searchTerm.length > 0) && /^[a-zA-Z]/.test(searchTerm[0])) {
            omniBoxModelService.clearPredictions();
            $scope.showDefaultSuggestions = false;
            omniBoxModelService.getPlacesSuggestions(searchTerm);
          } else {
            $scope.showDefaultSuggestions = true;
          }
        }, 200);
      };

      $scope.keyDownSearch = function () {
        var searchTerm = $scope.omniSearchTerm;
        $scope.hideSuggestionBox();
        $('#omnibox-input').blur();
        if ((typeof searchTerm !== 'undefined') && (searchTerm.length > 0)) {
          if (searchTerm !== $scope.nearByLabel) {
            $scope.placeSearch();
          } else {
            LQMobile.SearchDestination.hideOnSearchStyle();
            $scope.locationClick($scope.location);
          }
        }
      };

      $scope.clearPredictions = function () {
        $scope.showDefaultSuggestions = true;
        omniBoxModelService.clearPredictions();
        $scope.omniSearchTerm = '';
        $scope.boxCleared = true;
        $timeout(function () {
          $('#omnibox-input').focus();
        }, 200);
      };

      $scope.placeClick = function (place) {
        $scope.omniSearchTerm = place;
        $scope.placeSearch();
      };

      $scope.placeSearch = function () {
        LQMobile.SearchDestination.hideOnSearchStyle();
        omniBoxModelService.selectPlace($scope.omniSearchTerm);
        $scope.doPlaceSearch();
      };

      $scope.hotelClick = function (hotelId, location) {
        if (!editSearchPanelCollapseFactory.isCollapsed) {
          editSearchPanelCollapseFactory.toggleSearchEditPanel('.hotelSearchResults');
        }
        $scope.omniSearchTerm = location;
        $scope.loadHotelDetails(hotelId, location);
      };

      $scope.showSuggestionBox = function () {
        if (!isEditMode) {
          LQMobile.SearchDestination.showOnSearchStyle();
        }
        $('#signinbutton, #joinnowbutton, #membertierbanner, #lq-returns-logo').hide();
        $('body').addClass('omnibox-is-open');
        $scope.showSuggestions = true;
      };

      $scope.hideSuggestionBox = function () {
        $timeout(function () {
          if (!$scope.boxCleared) {
            if (!isEditMode) {
              $('#signinbutton, #joinnowbutton, #membertierbanner, #lq-returns-logo').show();
              $('body').removeClass('omnibox-is-open');
              LQMobile.SearchDestination.hideOnSearchStyle();
            }
            $scope.showSuggestions = false;
        // when focus is lost, the search box value is set except in the home
            if ($scope.isSetValueOnBlurActive && $scope.nearByLabel !== $scope.omniSearchTerm) {
              omniBoxModelService.selectPlace($scope.omniSearchTerm);
            }
          } else {
            $scope.boxCleared = false;
          }
        }, 250);
      };

      $scope.locationIconClick = function (location) {
        satelliteTrackingModelService.searchByCurrentLocationIcon();
        $scope.hideSuggestionBox();
        $scope.locationClick(location);
      };

      $scope.locationLinkClick = function (location) {
        satelliteTrackingModelService.searchByCurrentLocationLink();
        $scope.locationClick(location);
      };

      $scope.locationClick = function (location) {
        lodash.debounce(function () {
          if (navigator.geolocation) {
            $scope.location = location;
            geolocationService.getCurrentPositionPromise(true).then(
              $scope.setPosition,
              $scope.showError
            );
          } else {
            $scope.open('Geolocation is not supported by this device.');
          }
        }, 50, { leading: true })();
      };

      $scope.setPosition = function (position) {
        $timeout(function () {
          $scope.omniSearchTerm = $scope.location;
        }, 5);
        omniBoxModelService.selectLocation(position.coords.latitude, position.coords.longitude);
        $scope.doLocationSearch();
      };

      $scope.showError = function (error) {
        var showSettings = (typeof cordova.plugins.settings.open !== 'undefined');
        switch (error.code) {
          case error.PERMISSION_DENIED:
            $scope.open('Please enable location services in your device settings.', showSettings);
            searchParamModelService.setNoPermission();
            break;
          case error.POSITION_UNAVAILABLE:
            $scope.open('Location information is unavailable.', false);
            break;
          case error.TIMEOUT:
            $scope.open('The request has timed out, please ensure location services are enabled in your device settings.', showSettings);
            break;
          case error.UNKNOWN_ERROR:
            $scope.open('An unknown error occurred.', false);
            break;
          default:
            // do nothing
        }
      };


  /**
  * open
  *
  * Open the Location Error modal to display an error message to the user.
  *
  * @param String errorMessage
  * @param Boolean showSettings
  */
      $scope.open = function (errorMessage, showSettings) {
        var modalInstance;

        showSettings = (typeof showSettings == 'undefined') ? false : showSettings;

        //
        // Setup the Location Error Modal.
        modalInstance = $modal.open({
          templateUrl: 'locationErrorModalContent.html',
          controller: 'ModalInstanceCtrl',
          windowClass: 'app-modal-window',
          resolve: {
            message: function () {
              return errorMessage;
            },
            showSettings: function () {
              return showSettings;
            }
          }
        });

        //
        // ???
        modalInstance.result.then(function () {
          // here is when the user clicks ok
        }, function () {
          // here is when the user clicks no or closes de modal
        });
      };
    }]);


/**
* ModalInstanceCtrl
*
* Modal controller to manage the Location Error Modal.
*/
lqMobileModule.controller('ModalInstanceCtrl', function ($scope, $modalInstance, message, showSettings) {
  'use strict';

  $scope.errorMessage = message;
  $scope.showSettings = showSettings;


  /**
  * settings
  *
  * Bind to "Settings" button click. Attempts to open Device's native
  * settings panel.
  */
  $scope.settings = function () {
    if (typeof cordova.plugins.settings.open !== 'undefined') {
      //
      // Attempt to open the Device's settings panel.
      cordova.plugins.settings.open(
        'settings',

        // Success.
        function () {
          //
          // Dismiss the modal.
          $scope.ok();
        },
        // Fail.
        function () {
          // Change the modal error, hide the Settings button.
          $scope.errorMessage = 'Unable to open device settings.';
          $scope.showSettings = false;
        }
      );
    }
  };

  $scope.ok = function () {
    $modalInstance.close();
  };

  $scope.cancel = function () {
    $modalInstance.dismiss('cancel');
  };
});
