/* global lqMobileModule */
lqMobileModule.controller(
  'bannerController',
  [
    '$rootScope',
    '$scope',
    '$route',
    '$modal',
    '$q',
    'routingService',
    'returnsUserProfileModel',
    'freeNightModel',
    'reservationsModel',
    'geolocationService',
    'touchIdService',
    'satelliteTrackingModelService',
    'lodash',
    function (
      $rootScope,
      $scope,
      $route,
      $modal,
      $q,
      routingService,
      returnsUserProfileModel,
      freeNightModel,
      reservationsModel,
      geolocationService,
      touchIdService,
      satelliteTrackingModelService,
      lodash
    ) {
      $scope.customBannerText = typeof CUSTOM_BANNER_TEXT !== 'undefined' ? CUSTOM_BANNER_TEXT : 'Important program update for members';

      function bannerInitLoggedInUser() {
        if (returnsUserProfileModel.isLoggedIn()) {
          console.log('is logged in!');

          // Does user have a free night to redeem?
          // @todo: Instant Free Night check here.
          if (!freeNightModel.built) {
            freeNightModel.checkEligibility();
          }

          // fetch current reservations:
          reservationsModel.initializeReservations();
        }
      }

      function displayTouchIdWizard() {
        touchIdService.showFingerprintWizard(
          function (dismissModal) {
            lodash.debounce(function () {
              returnsUserProfileModel.retrieveCredentials().then(
                function (creds) {
                  touchIdService.registerWithFingerAuthentication(
                    creds.userid,
                    creds.lastname,
                    creds.password
                  ).then(function () {
                    dismissModal();
                  }, function () {
                    // failed auth or cancel, set wizard to appear again
                    touchIdService.setIfSeenWizard(false);
                    touchIdService.setUsingFingerAuth(false);
                    touchIdService.setUsingFingerAuth();
                    dismissModal();
                  });
                },
                function () {
                  // welp, nothing to do here :-\
                }
              );
            }, 400, { leading: true, trailing: false })();
          },
          function (dismissModal) {
            dismissModal();
          }
        );
      }


      /**
      * showMobileCheckoutBanner
      *
      * Determine if the user is eligible for mobile checkout.
      *
      * @return Boolean
      */
      $scope.showMobileCheckoutBanner = function () {
        // per moving to active stays model - banner never shows
        return false;
      };

      $scope.showCustomBanner = function () {
        return CUSTOM_BANNER_SHOW === 'true';
      };

      $scope.showFreeNightBanner = function () {
        return freeNightModel.isEligible()
          && !$scope.showMobileCheckoutBanner() && !$scope.showCustomBanner();
      };

      $scope.goToFreeNightInfo = function () {
        routingService.goToFreeNightInfo();
      };

      $scope.goToCustomPage = function () {
        routingService.goToCustomPage();
      };

      $scope.showFreeNightInfo = function () {
        if (!lqApp.StartupComplete) {
          return false;
        }

        if (!returnsUserProfileModel.isLoggedIn() && !$scope.showCustomBanner()) {
          return true;
        }

        return lqApp.StartupComplete && freeNightModel.built
          && !$scope.showMobileCheckoutBanner() && !$scope.showFreeNightBanner() && !$scope.showCustomBanner();
      };

      $scope.goToFreeNight = function () {
        satelliteTrackingModelService.freeNightBannerClicked(freeNightModel.innNumber);
        freeNightModel.userSelectedFreeNight = true;
        routingService.goToFreeNightRoomSelection(freeNightModel.innNumber);
      };


      //
      // init
      $scope.$on('login-complete', bannerInitLoggedInUser);
      $scope.$on('show-fingerprint-wizard', displayTouchIdWizard);
      bannerInitLoggedInUser();
    }]
);
