/* global lqMobileModule*/
/* jshint maxparams: 11 */
lqMobileModule.controller(
  'bookingPartialController', [
    '$rootScope',
    '$scope',
    '$window',
    'searchParamModelService',
    'hotelInformationModel',
    'routingService',
    'bookingModelService',
    'events',
    'reservationsModel',
    'paymentTypes',
    'satelliteTrackingModelService',
    'returnsUserProfileModel',
    function (
      $rootScope,
      $scope,
      $window,
      searchParamModelService,
      hotelInformationModel,
      routingService,
      bookingModelService,
      events,
      reservationsModel,
      paymentTypes,
      satelliteTrackingModelService,
      returnsUserProfileModel
    ) {
      'use strict';

      $scope.searchInfo = searchParamModelService.data;
      $scope.hotelInfo = hotelInformationModel.data.hotelDetails;
      $scope.creatingUserProfilePromess = null;

      // not sure why this isn't getting populated, but
      // we'll just repopulate it here when a user is logged in
      if (returnsUserProfileModel.isLoggedIn() &&
        bookingModelService.data.firstName == '') {
        bookingModelService.updateBookingDataFromProfile();
      }

      $scope.bookingForm = bookingModelService.data;

      // this function doesn't seem to be used, but might be useful later
      function isCheckinDateInLessThanFourMonths() { // eslint-disable-line
        var date = new Date();
        date.setHours(0, 0, 0, 0);
        date.setDate(date.getDate() + 120);
        if (searchParamModelService.data.checkinDate < date) {
          return true;
        }
        return false;
      }

      $scope.cancelBooking = function () {
        $window.history.back();
        routingService.turnOnBackButton();
      };

      $scope.bookAgain = function () {
        routingService.goToHotelDetails(hotelInformationModel.data.hotelDetails.hotelId);
      };

      $scope.goToGuestInfo = function () {
        routingService.goToBookingGuest();
      };

      $scope.goToBookingRecap = function () {
        routingService.goToBookingRecap();
      };

      $scope.bookingForm.getFormFieldCssClass = function (formSubmitted, isFieldValid) {
        if (formSubmitted && !isFieldValid) {
          return 'form-control has-error';
        }
        return 'form-control';
      };

      $scope.featuredLinkClick = function (linkLabel) {
        satelliteTrackingModelService.additionalFeaturesLinkSelected(linkLabel);
      };

      $scope.editMyReservation = function () {
        var firstName = bookingModelService.data.firstName;
        var lastName = bookingModelService.data.lastName;

        satelliteTrackingModelService.editMyReservationLinkSelected();
        $scope.loadingModal = $scope.showLoadingModal();
        if (bookingModelService.data.previousPaymentType === paymentTypes.INSTANTHOLD) {
          firstName = 'LQ';
          lastName = 'INSTANTHOLD';
        }
        reservationsModel.getSelectedReservationInfo(
          bookingModelService.data.confirmationNumber, firstName, lastName
        ).then(function (response) {
          reservationsModel.setFullReservation(response);
          $scope.loadingModal.close();
          routingService.goToReservationDetails();
        }, function (error) {
          $scope.generalErrorMsg = error.data;
          $scope.loadingModal.close();
        });
      };
    }
  ]);
