/* global lqMobileModule, $*/
lqMobileModule.controller(
  'homePartialController',
  [
    '$scope',
    '$timeout',
    'routingService',
    'returnsUserProfileModel',
    'events',
    'satelliteTrackingModelService',
    'mobileCheckoutModel',
    'activeStaysModel',
    'lodash',
    function (
      $scope,
      $timeout,
      routingService,
      returnsUserProfileModel,
      events,
      satelliteTrackingModelService,
      mobileCheckoutModel,
      activeStaysModel,
      lodash
    ) {
      'use strict';

      $scope.activeStaysModel = activeStaysModel;
      $scope.isSetValueOnBlurActive = false;
      $scope.isCheckingActiveStays = false;
      routingService.turnOffBackButton();

      $scope.initializeHomePageBottomBar = function () {
        $scope.returnsUser = returnsUserProfileModel;

        $scope.militaryClass = ($scope.returnsUser.data.userProfile.isMilitary) ? 'MILITARY' : '';
        if ($scope.returnsUser.data.userProfile.returnsTier) {
          $scope.memberTier = $scope.returnsUser.data.userProfile.returnsTier.charAt(0)
            + $scope.returnsUser.data.userProfile.returnsTier.slice(1).toLowerCase();
          if ($scope.returnsUser.data.userProfile.isMilitary) $scope.memberTier = 'Military ' + $scope.memberTier;
        }
      };

      $scope.doPlaceSearch = function () {
        routingService.goToHotelSearch();
      };

      $scope.doLocationSearch = function () {
        routingService.goToHotelSearch();
      };

      $scope.signIntoLQ = function () {
        satelliteTrackingModelService.homeSignInButtonClicked();
        if (returnsUserProfileModel.isLoggedIn()) {
          routingService.goToAccountSummary();
        } else {
          routingService.goToSignInPage();
        }
      };

      $scope.joinLQReturns = function () {
        satelliteTrackingModelService.joinLQButtonSelected();
        routingService.goToSingUp();
      };

      /**
      * requestUber
      *
      * Call global uber request method
      */
      $scope.requestUber = function () {
        $scope.executeUberRequest(encodeURIComponent(activeStaysModel.activeStay.hotel.address));
      };

      /**
      * getDirections
      *
      * Call global get directions method
      */
      $scope.getDirections = function () {
        $scope.executeGetDirections(encodeURIComponent(activeStaysModel.activeStay.hotel.address));
      };

      /**
      * callFrontDesk
      *
      * Call global call Hotel method
      */
      $scope.callFrontDesk = function () {
        $scope.executePhoneCall(activeStaysModel.getPhoneNumber());
      };

      /**
      * Begin the mobile checkin process.
      */
      $scope.goToMobileCheckin = function () {
        satelliteTrackingModelService.mobileCheckInButtonClicked();
        if (activeStaysModel.hasActiveStay()) {
          routingService.goToMobileCheckin();
        } else {
          routingService.goToCheckinUnavailable();
        }
      };

      /**
      * Begin the mobile checkout process.
      */
      $scope.goToMobileCheckout = function () {
        satelliteTrackingModelService.mobileCheckoutBannerClicked();
        if (mobileCheckoutModel.isEligible()) {
          routingService.goToMobileCheckout();
        } else {
          routingService.goToCheckoutUnavailable();
        }
      };

      /**
       * start functionality for current active stay swipe up shade
       */
      $scope.initActiveStayCurrent = function () {
        // drag testing
        $scope.activeStayObject.max = 0;
        $scope.activeStayObject.move = 20;
        $scope.activeStayObject.min = 0;
        $scope.activeStayObject.touchStart = null;
        $scope.activeStayObject.bottomStart = null;
        $scope.activeStayObject.currentBottom = 0;
        $scope.activeStayObject.isOpen = false;
        $scope.activeStayObject.swipeContainer = document.querySelector('[data-js-swipe]');
        $scope.activeStayObject.swipeHandle = document.querySelector('[data-js-swipe-handle]');
        $scope.activeStayObject.swipeContent = document.querySelector('[data-js-swipe-content]');

        $($scope.activeStayObject.swipeHandle).on('touchstart', function (e) {
          // get max
          $scope.activeStayObject.max = $scope.activeStayObject.swipeContent.offsetHeight;

          // store touch start event and bottom start position
          $scope.activeStayObject.touchStart = e.originalEvent.touches[0].clientY;
          $scope.activeStayObject.bottomStart = $scope.activeStayObject.swipeContainer.parentElement.getBoundingClientRect().bottom - $scope.activeStayObject.swipeContainer.getBoundingClientRect().bottom;

          $('body').on('touchmove.banner', $scope.activeStayObject.onTouchMove);
        });

        $($scope.activeStayObject.swipeHandle).on('touchend', function () {
          var newBottom = $scope.activeStayObject.min;
          $('body').off('touchmove.banner', $scope.activeStayObject.onTouchMove);

          if ($scope.activeStayObject.isOpen) {
            if ($scope.activeStayObject.currentBottom <= $scope.activeStayObject.max - $scope.activeStayObject.move) {
              $scope.activeStayObject.isOpen = false;
            } else {
              newBottom = $scope.activeStayObject.max;
            }
          } else if ($scope.activeStayObject.currentBottom >= $scope.activeStayObject.move) {
            $scope.activeStayObject.isOpen = true;
            newBottom = $scope.activeStayObject.max;
          }

          $scope.activeStayObject.updateContainer(newBottom);
        });

        $($scope.activeStayObject.swipeHandle).on('click', function (e) {
          lodash.debounce(function () {
            var newBottom = $scope.activeStayObject.currentBottom > $scope.activeStayObject.min ? $scope.activeStayObject.min : $scope.activeStayObject.max;
            if ($scope.activeStayObject.currentBottom > $scope.activeStayObject.min) $scope.activeStayObject.isOpen = false;

            $scope.activeStayObject.updateContainer(newBottom);
          }, 400, { leading: true, trailing: false })();
        });

        $scope.activeStayObject.onTouchMove = function (e) {
          $scope.activeStayObject.currentBottom = $scope.activeStayObject.getBottom(e);
          $scope.activeStayObject.swipeContainer.style.bottom = $scope.activeStayObject.currentBottom + 'px';
        };

        $scope.activeStayObject.getBottom = function (e) {
          var bottom = Math.min($scope.activeStayObject.bottomStart + ($scope.activeStayObject.touchStart - e.originalEvent.touches[0].clientY), $scope.activeStayObject.max);
          bottom = Math.max(bottom, $scope.activeStayObject.min);
          return bottom;
        };

        $scope.activeStayObject.updateContainer = function (bottom) {
          $scope.activeStayObject.currentBottom = bottom;
          $scope.activeStayObject.swipeContainer.style.bottom = bottom + 'px';
          if ($scope.activeStayObject.currentBottom === $scope.activeStayObject.max) {
            $scope.activeStayObject.swipeContainer.classList.add('is-open');
          } else {
            $scope.activeStayObject.swipeContainer.classList.remove('is-open');
          }
        };
      };

      $scope.initLoggedInUser = function () {
        // if user is logged in - look for active stays
        if (returnsUserProfileModel.isLoggedIn() && !$scope.isCheckingActiveStays) {
          $scope.isCheckingActiveStays = true;
          $scope.initializeHomePageBottomBar();

          // get active stays
          activeStaysModel.getActiveStay(
            returnsUserProfileModel.data.userProfile.returnsNumber
          ).then(
            function () {
              $scope.isCheckingActiveStays = false;

              // we have an active stay - check if currently staying
              if (activeStaysModel.isActiveStay()) {
                // set up functionality
                $scope.activeStayObject = {};
                $scope.initActiveStayCurrent();
                // // set up functionality
                // $scope.activeStayObject = {};
                // $scope.initActiveStayCurrent();
              }

              // send some events
              if (activeStaysModel.isActiveCheckIn()) {
                satelliteTrackingModelService.mobileCheckInDisplayed();
              } else if (activeStaysModel.isActiveCheckOut()) {
                satelliteTrackingModelService.mobileCheckoutBannerShown(
                  mobileCheckoutModel.isEligible(), mobileCheckoutModel.confirmationNumber
                );
              }
            },
            function () {
              $scope.isCheckingActiveStays = false;
              // error .. hide popup... or something?
            }
          );
        }
      };

      $scope.$on('login-complete', $scope.initLoggedInUser);
      $scope.initLoggedInUser();
      satelliteTrackingModelService.homeLoaded();
    }
  ]
);
