/* global lqMobileModule, $*/
/* jshint maxparams: 14 */
lqMobileModule.controller('hotelSearchPartialController',
  [
    '$q',
    '$scope',
    'routingService',
    'searchParamModelService',
    'hotelSearchModelService',
    'editSearchPanelCollapseFactory',
    'editSearchAlertFactory',
    '$location',
    'googleMapsModelService',
    '$timeout',
    'HotelSummary',
    'googleMapsEndpointService',
    '$routeParams',
    'reservationsModel',
    function (
      $q,
      $scope,
      routingService,
      searchParamModelService,
      hotelSearchModelService,
      editSearchPanelCollapseFactory,
      editSearchAlertFactory,
      $location,
      googleMapsModelService,
      $timeout,
      HotelSummary,
      googleMapsEndpointService,
      $routeParams,
      reservationsModel
  ) {
      'use strict';

      var editParameter = $location.search().editSearch;

      if (searchParamModelService.data.resultsViewType === 'list') {
        $scope.mapLoaded = false;
        $scope.modeLabel = 'map';
        $scope.defaultView = 'list';
        $scope.listmode = true;
      } else {
        $scope.mapLoaded = true;
        $scope.modeLabel = 'list';
        $scope.defaultView = 'map';
        $scope.listmode = false;
      }

      reservationsModel.clearModel();

      hotelSearchModelService.clearHotels();
      $scope.switchPages = function () {
        routingService.goToHomePage();
      };

      $scope.searchData = searchParamModelService.data;
      $scope.hotelSearchResults = hotelSearchModelService.data;

      $scope.selectHotel = function (hotelId) {
        searchParamModelService.setHotel(hotelId);
        routingService.goToHotelDetails(hotelId);
      };

      $scope.doLocationSearch = function () {
      // do nothing for now
      };

      $scope.$on('$routeUpdate', function () {
        var resultView = $routeParams.resultView;
        if ((!resultView && searchParamModelService.data.resultsViewType !== $scope.defaultView) ||
        (resultView && searchParamModelService.data.resultsViewType !== resultView)) {
          $scope.switchMode();
        }
      });

      $scope.switchMode = function () {
        if ($scope.listmode) {
          $scope.listmode = false;
          searchParamModelService.setResultsViewType('map');
          $scope.modeLabel = 'list';
          $location.search('resultView', 'map');
          if (!$scope.mapLoaded) {
            $scope.mapLoaded = true;
            $timeout(function () {
              googleMapsModelService.refreshMap();
            }, 5);
          }
        } else {
          $location.search('resultView', 'list');
          $scope.listmode = true;
          $scope.modeLabel = 'map';
          searchParamModelService.setResultsViewType('list');
        }
      };


      function showSearchEditPanel() {
        editSearchPanelCollapseFactory.toggleSearchEditPanel('.hotelSearchResults');
      }

      function getLocationObject() {
        var location = {
          latitude: searchParamModelService.data.latitude,
          longitude: searchParamModelService.data.longitude,
          lat: function () {
            return this.latitude;
          },
          lng: function () {
            return this.longitude;
          }
        };
        return location;
      }

      function processHotelResults() {
        editSearchAlertFactory.clearAlerts();
        if (!hotelSearchModelService.data.hotelsAvailable) {
          showSearchEditPanel();
          editSearchAlertFactory.addAlert('danger', 'We\'re sorry, no locations were found within ' + searchParamModelService.data.range + '-miles of your current location.');
        } else {
          googleMapsModelService.refreshMap();
          $location.search('editSearch', null);
        }
        $('div.hideSearchWhileLoading').removeClass('hideSearchWhileLoading');
        $scope.loadingModal.close();
      }

      function handleSearchErrors(error) {
        showSearchEditPanel();
        editSearchAlertFactory.addAlert('danger', CQ.I18n.getMessage(error.data.trim()));
        $('div.hideSearchWhileLoading').removeClass('hideSearchWhileLoading');
        $scope.loadingModal.close();
      }


      $scope.hotelFromMap = [];
      function displaySelectedHotelInfo(hotel) {
        googleMapsModelService.selectMarker(hotel.hotelSummary);
        $timeout(function () {
          $scope.hotelFromMap.push(hotel);
        }, 5);
      }

      function dismissMarkerListener() {
        googleMapsModelService.dismissSelectedMarker();
        $timeout(function () {
          LQMobile.Util.clearArray($scope.hotelFromMap);
        }, 5);
      }

      function markerListener(hotel) {
        if ($scope.hotelSearchResults.hotelsLoaded[hotel.hotelId]) {
          displaySelectedHotelInfo($scope.hotelSearchResults.hotelsLoaded[hotel.hotelId]);
        } else {
          $scope.loadingModal = $scope.showLoadingModal();
          hotelSearchModelService.loadHotelInfo(hotel).then(
          function () {
            displaySelectedHotelInfo($scope.hotelSearchResults.hotelsLoaded[hotel.hotelId]);
            $('div.hideSearchWhileLoading').removeClass('hideSearchWhileLoading');
            $scope.loadingModal.close();
          },
          function () {
            // display the hotel info from the summary, with no rates
            var hotelObj = HotelSummary.build(hotel);
            googleMapsEndpointService.getDistanceFrom2Coords(
              hotelObj.hotelSummary.location.latitude, hotelObj.hotelSummary.location.longitude,
              searchParamModelService.data.latitude, searchParamModelService.data.longitude
            ).then(function (range) {
              hotelObj.range.range = range;
              displaySelectedHotelInfo(hotelObj);
              $('div.hideSearchWhileLoading').removeClass('hideSearchWhileLoading');
              $scope.loadingModal.close();
            });
          });
        }
      }

      function getHotelResults(locationObj) {
        $scope.loadingModal = $scope.showLoadingModal();
        hotelSearchModelService.getHotelsList(locationObj).then(
          processHotelResults, handleSearchErrors
        );
      }

      function getCurrentPlaceLocation() {
        var searchData = searchParamModelService.data;
        return hotelSearchModelService.geolocateAddress(searchData.currentPlace).then(
          function (geolocatedAddressObject) {
            var searchLocation = geolocatedAddressObject.geometry.location;
            searchData.latitude = searchLocation.lat();
            searchData.longitude = searchLocation.lng();
            return searchLocation;
          },
          function (error) {
            throw error;
          }
        );
      }

    /*
     * if the user doesn't provide location access or no coordinates are available,
     * display search panel, else if available, search with user location
     * else if available, get coordinates from current location and search with it
     * else, display search panel
     */
      function loadHotelResults() {
        var searchData = searchParamModelService.data;
        dismissMarkerListener();

        if ((searchData.searchType === 'nopermission') ||
        (searchData.searchType === 'location' && (!searchData.latitude || !searchData.longitude))) {
          showSearchEditPanel();
        } else if (searchData.searchType === 'location') {
          getHotelResults(getLocationObject());
        } else if (searchData.currentPlace) {
          getCurrentPlaceLocation().then(function (geoLocation) {
            getHotelResults(geoLocation);
          });
        } else {
          showSearchEditPanel();
        }
      }

      $scope.doFindHotels = function () {
        loadHotelResults();
        if (searchParamModelService.data.resultsViewType === 'list') {
          $scope.mapLoaded = false;
        }
      };


      $scope.hotelObj = null;
      if (editParameter) {
        editSearchPanelCollapseFactory.expandPanel();
      } else {
      // collapse the edit search panel when the search starts.
        editSearchPanelCollapseFactory.collapsePanel();
        loadHotelResults();
        googleMapsModelService.attachMap('map-canvas', markerListener, dismissMarkerListener).then(function () {
          googleMapsModelService.zoom();
        });
      }

      // if the user makes a search, we clear the model in case he has
      // previously selected to change a reservations. A new search,
      // means a new reservation is done.
      editSearchAlertFactory.clearAlerts();


      $scope.showEverything = function () {
        $('div.hideSearchWhileLoading').removeClass('hideSearchWhileLoading');
        return true;
      };
    }]);
