/* global lqMobileModule*/
/* jshint maxparams: 9 */
lqMobileModule.controller('reservationsDetailsController',
  ['$scope',
    '$modal',
    '$rootScope',
    'reservationsModel',
    'routingService',
    'searchParamModelService',
    'hotelInformationModel',
    'bookingModelService',
    'events',
    'satelliteTrackingModelService',
    'dateFormatterService',
    'currencyService',
    function ($scope,
      $modal,
      $rootScope,
      reservationsModel,
      routingService,
      searchParamModelService,
      hotelInformationModel,
      bookingModelService,
      events,
      satelliteTrackingModelService,
      dateFormatterService,
      currencyService
    ) {
      'use strict';

      var selectedReservation = reservationsModel.getSelectedReservation();

      $scope.reservationDetails = null;
      $scope.cancellationNumber = null;
      $scope.error = '';
      $scope.loading = true;

      function trackAnalytics() {
        var oneDay = 1000 * 60 * 60 * 24;
        var now = new Date();
        var arrivalTime = $scope.reservationDetails.arrivalDate.getTime();
        var departureTime = $scope.reservationDetails.departureDate.getTime();
        var nowCheckInDiffMS = arrivalTime - now.getTime();
        var checkInCheckOutDiffMS = departureTime - arrivalTime;
        var numberOfNights = Math.round(checkInCheckOutDiffMS / oneDay);
        var daysToArrival = Math.round(nowCheckInDiffMS / oneDay);

        if (daysToArrival < 1) {
          daysToArrival = 0;
        }

        satelliteTrackingModelService.editReservationLoaded(
          $scope.reservationDetails.hotelId,
          numberOfNights,
          dateFormatterService.getFormattedDate($scope.reservationDetails.arrivalDate),
          dateFormatterService.getFormattedDate($scope.reservationDetails.departureDate),
          $scope.reservationDetails.adults,
          daysToArrival,
          $scope.reservationDetails.promoCode,
          $scope.reservationDetails.rateType
        );
      }

      if (selectedReservation !== null && selectedReservation.hasOwnProperty('estimatedTotal')) {
        $scope.reservationDetails = selectedReservation;
        reservationsModel.setFullReservation(selectedReservation);
        trackAnalytics();
        $scope.loading = false;
      } else {
        reservationsModel.getSelectedReservationInfo(
          selectedReservation.confirmationNumber, selectedReservation.firstName,
          selectedReservation.lastName
        ).then(function (response) {
          $scope.reservationDetails = response;
          reservationsModel.setFullReservation(response);
          trackAnalytics();
          $scope.loading = false;
        }, function (error) {
          $scope.error = error;
          $scope.loading = false;
        });
      }

      $scope.cancel = function () {
        satelliteTrackingModelService.cancelReservationButtonSelected();
        $scope.cancellationNumber = $scope.reservationDetails.confirmationNumber;
        $modal.open({
          templateUrl: 'cancelReservationModal.html',
          controller: 'cancelReservationModalController',
          windowClass: 'app-modal-window border-radius confirmation-modal',
          resolve: {
            showLoadingModal: function () {
              return $scope.showLoadingModal;
            }
          }
        });
      };

      $scope.editReservation = function () {
        satelliteTrackingModelService.editReservationButtonSelected();
        searchParamModelService.updateDataFromSelectedReservation();

        bookingModelService.setMobilePhone($scope.reservationDetails.mobilePhone);
        if ($scope.reservationDetails.status === 'CONFIRMED') {
          bookingModelService.updateBookingDataFromReservation();
          routingService.goToHotelDetails($scope.reservationDetails.hotelId);
        } else if ($scope.reservationDetails.status === 'ONHOLD') {
          searchParamModelService.setSelectedRoom($scope.reservationDetails.selectedRoom);
          $scope.loadingModal = $scope.showLoadingModal();
          hotelInformationModel.loadHotelDetails(
            $scope.reservationDetails.hotelId
          ).then(
            function () {
              // clear the confirmation number (from possible previous reservation)
              bookingModelService.resetConfirmationNumber();
              routingService.goToBookingGuest();
              $scope.loadingModal.close();
            },
            function (error) {
              $scope.loadingModal.close();
              $scope.error = error;
            }
          );
        }
      };
    }]);

lqMobileModule.controller('cancelReservationModalController', ['$scope', '$modalInstance', 'reservationsModel', 'showLoadingModal', 'routingService',
  function ($scope, $modalInstance, reservationsModel, showLoadingModal, routingService) {
    'use strict';

    $scope.showConfirmation = true;
    $scope.cancelOk = false;
    $scope.cancellationNumber = null;

    $scope.cancelReservation = function () {
      $scope.loadingModal = showLoadingModal();
      reservationsModel.cancelReservation().then(function (response) {
        $scope.cancellationNumber = response;
        $scope.showConfirmation = false;
        $scope.cancelOk = true;
        $scope.loadingModal.close();
      }, function (error) {
        $scope.showConfirmation = false;
        $scope.cancelOk = false;
        $scope.loadingModal.close();
        throw error;
      });
    };

    $scope.close = function () {
      $modalInstance.dismiss();
      routingService.goToMyReservations();
    };
  }]);
