/* global lqMobileModule, console, angular*/
lqMobileModule.factory('reservationsEndpointService', ['$window',
  '$q',
  '$http',
  'serviceEndPoints',
  function (
    $window,
    $q,
    $http,
    serviceEndPoints
  ) {
    'use strict';

    var endpointService = {};

    endpointService.getReservations = function (returnsId, startDate, endDate, authToken) {
      return $http({
        method: 'get',
        url: serviceEndPoints.getReservationsEndPoint() + '/' + returnsId,
        params: {
          apikey: serviceEndPoints.getApiKey(),
          startDate: startDate,
          endDate: endDate,
          random: Math.random().toString(36).substring(7)
        },
        headers: {
          LQ_AUTH_KEY: authToken
        }
      }).then(
        // Success
        function (response) {
          return response;
        },
        // Error
        function (error) {
          console.error(JSON.stringify(error));
          throw error;
        }
      );
    };

    endpointService.getReservationInfo = function (confirmationNumber, firstName, lastName) {
      if (typeof firstName === 'string') {
        firstName = firstName.trim();
      }
      if (typeof lastName === 'string') {
        lastName = lastName.trim();
      }
      return $http({
        method: 'get',
        url: serviceEndPoints.getReservationsEndPoint(),
        params: {
          apikey: serviceEndPoints.getApiKey(),
          confirmationNumber: confirmationNumber,
          firstName: firstName,
          lastName: lastName,
          random: Math.random().toString(36).substring(7)
        }
      }).then(
        // Success
        function (response) {
          return response;
        },
        // Error
        function (error) {
          console.error(JSON.stringify(error));
          throw error;
        }
      );
    };

    endpointService.cancelReservation = function (confirmationNumber) {
      return $http({
        method: 'delete',
        url: serviceEndPoints.getReservationsEndPoint() + '/' + confirmationNumber,
        params: {
          apikey: serviceEndPoints.getApiKey(),
          random: Math.random().toString(36).substring(7)
        }
      }).then(
        // Success
        function () {
          return confirmationNumber;
        },
        // Error
        function (error) {
          throw error;
        }
      );
    };

    return endpointService;
  }]);
