/* global lqMobileModule, console, CQ*/
/* jshint maxparams: 11 */
lqMobileModule.factory('reservationsModel', ['$q', '$filter',
  'storageModelService',
  'reservationsEndpointService',
  'ReservationSummary',
  'ReservationFull',
  'persistentStorageModelService',
  'returnsUserProfileModel',
  'lodash',
  function ($q, $filter,
    storageModelService,
    reservationsEndpointService,
    ReservationSummary,
    ReservationFull,
    persistentStorageModelService,
    returnsUserProfileModel,
    lodash) {
    'use strict';

    var dataModel = {
      data: {
        // reservations is the JSON data from the server
        reservations: [],
        selectedReservation: null,
        selectedFullReservation: null,
        error: ''
      },
      localData: {
        localReservations: null
      }
    };

    persistentStorageModelService.getLocalReservations().then(
      function (result) {
        dataModel.localData.localReservations = result;
      },
      function () {
        dataModel.localData.localReservations = null;
      }
    );

    function getFormattedReservations(data) {
      var index;
      var reservations = [];
      var mappedReservation;
      for (index = 0; index < data.length; index += 1) {
        mappedReservation = ReservationSummary.map(data[index]);
        reservations.push(mappedReservation);
      }
      return reservations;
    }

    function getIndividualReservation(confNumber, firstName, lastName) {
      var promise = reservationsEndpointService.getReservationInfo(
        confNumber, firstName, lastName
      ).then(function (response) {
        dataModel.data.reservations.push(ReservationFull.map(response.data[0]));
      }, function () {
        dataModel.data.error = 'We are sorry, we were not able to retrieve all of your reservations . Please call 1-855-250-6469 for assistance.';
      });
      return promise;
    }

    function loadReservationsFromCache() {
      var deferred = $q.defer();
      persistentStorageModelService.getLocalReservations().then(
        function (storedReservations) {
          var localReservations = dataModel.localData.localReservations;
          var promises = [];
          var index;
          var seen;
          var reservationLength;
          var savedReservation;
          var promise;

          if (lodash.isArray(storedReservations) && storedReservations.length > 0) {
            localReservations = localReservations.concat(storedReservations);
          }

          if (localReservations.length > 0) {
            seen = {};
            // clear out any reservations that have duplicate confNumbers
            localReservations = localReservations.filter(function (item) {
              var k = item.confNumber;
              return seen.hasOwnProperty(k) ? false : (seen[k] = true);
            });
          }
          reservationLength = localReservations.length;

          for (index = 0; index < reservationLength; index += 1) {
            savedReservation = localReservations[index];
            promise = getIndividualReservation(
              savedReservation.confNumber, savedReservation.firstName, savedReservation.lastName
            );
            promises.push(promise);
          }

          $q.all(promises).then(function () {
            deferred.resolve();
          });
        },
        function () {
          // same as above w/ no stored Reservations
          var localReservations = dataModel.localData.localReservations;
          var promises = [];
          var index;
          var seen;
          var reservationLength;
          var savedReservation;
          var promise;

          if (localReservations.length > 0) {
            seen = {};
            // clear out any reservations that have duplicate confNumbers
            localReservations = localReservations.filter(function (item) {
              var k = item.confNumber;
              return seen.hasOwnProperty(k) ? false : (seen[k] = true);
            });
          }
          reservationLength = localReservations.length;

          for (index = 0; index < reservationLength; index += 1) {
            savedReservation = localReservations[index];
            promise = getIndividualReservation(
              savedReservation.confNumber, savedReservation.firstName, savedReservation.lastName
            );
            promises.push(promise);
          }

          $q.all(promises).then(function () {
            deferred.resolve();
          });
        }
      );
      return deferred.promise;
    }

    function loadReservationsFromServices() {
      var returnsId = returnsUserProfileModel.data.userProfile.returnsNumber;
      var authToken = window.sessionStorage.getItem('token');
      var date = new Date();
      var endDate;
      var startDate;
      var deferred = $q.defer();

      date.setFullYear(date.getFullYear() + 1);
      endDate = $filter('date')(date, 'yyyy-MM-dd');
      date.setFullYear(date.getFullYear() - 1);
      date.setMonth(date.getMonth() - 1);
      startDate = $filter('date')(date, 'yyyy-MM-dd');

      reservationsEndpointService.getReservations(
        returnsId, startDate, endDate, authToken
      ).then(function (response) {
        dataModel.data.reservations = getFormattedReservations(response.data);
        deferred.resolve();
      }, function (error) {
        dataModel.data.error = error.data;
        deferred.reject(error.data);
      });

      return deferred.promise;
    }

    dataModel.initializeReservations = function () {
      dataModel.clearModel();
      if (returnsUserProfileModel.isLoggedIn()) {
        return loadReservationsFromServices();
      }
      return loadReservationsFromCache();
    };

    function getReservationsFromLocalStorage() {
      var reservationsFromSessionStorage = storageModelService.getReservations();
      // Checking that we have a valid reservation in the storage model service,
      // by looking at a property
      if (reservationsFromSessionStorage && reservationsFromSessionStorage.hasOwnProperty('confNumber')) {
        return reservationsFromSessionStorage;
      }
      return '';
    }

    function getReservationsFromCookie() {
      var defer = $q.defer();
      persistentStorageModelService.getLocalReservations().then(
        function (reservationsFromCookie) {
          // Checking that we have a valid reservation in the storage model service,
          // by looking at a property
          if (reservationsFromCookie && reservationsFromCookie.length > 0) {
            defer.resolve(reservationsFromCookie);
          } else {
            defer.resolve('');
          }
        },
        function () {
          defer.resolve('');
        }
      );
      return defer.promise;
    }

    dataModel.setSelectedReservation = function (reservation) {
      dataModel.data.selectedReservation = reservation;
      storageModelService.setReservations(dataModel.data);
    };

    dataModel.getSelectedReservationInfo = function (confirmationNumber, firstName, lastName) {
      return reservationsEndpointService.getReservationInfo(
        confirmationNumber, firstName, lastName
      ).then(function (response) {
        return ReservationFull.map(response.data[0]);
      }, function (error) {
        throw error;
      });
    };

    dataModel.setFullReservation = function (reservation) {
      dataModel.data.selectedFullReservation = reservation;
      dataModel.data.selectedReservation = reservation;
      storageModelService.setReservations(dataModel.data);
    };

    // appears unused but might be useful in future
    function updateReservationModel() { // eslint-disable-line
      var reservations = getReservationsFromLocalStorage();
      if (reservations) {
        dataModel.data = reservations;
      }

      getReservationsFromCookie().then(
        function (savedReservations) {
          if (savedReservations) {
            dataModel.localData.localReservations = savedReservations;
          } else {
            dataModel.localData.localReservations = null;
          }
        },
        function () {
          dataModel.localData.localReservations = null;
        }
      );
    }

    dataModel.isFncReservation = function (reservation) {
      var rateType = lodash.get(reservation, 'rateType', null);

      if (lodash.isNull(rateType)) {
        return false;
      }
      return (rateType.indexOf('FNC') !== -1);
    };

    dataModel.cancelReservation = function () {
      var points = lodash.get(dataModel, 'data.selectedFullReservation.totalPoints', 0);

      return reservationsEndpointService.cancelReservation(
        dataModel.data.selectedFullReservation.confirmationNumber
      ).then(function (response) {
        // Instead of requerying webservices for the profile,
        // just restore the points here in the data model.
        if (
          returnsUserProfileModel.isLoggedIn()
          && dataModel.isFncReservation(dataModel.data.selectedFullReservation)
        ) {
          returnsUserProfileModel.restorePoints(points);
        }
        return response;
      }, function (error) {
        throw error;
      });
    };

    dataModel.addLocalReservation = function (confNumber, firstName, lastName) {
      var localReservations = dataModel.localData.localReservations;
      var reservationLength = localReservations.length;
      var reservationObj = {
        confNumber: confNumber,
        firstName: firstName,
        lastName: lastName
      };
      var index;
      var savedReservation;
      for (index = 0; index < reservationLength; index += 1) {
        savedReservation = localReservations[index];
        if (savedReservation && savedReservation.confNumber === confNumber) {
          localReservations.splice(index, 1);
        }
      }
      localReservations.unshift(reservationObj);
      if (localReservations.length >= 15) {
        localReservations = localReservations.slice(0, 14);
      }
      persistentStorageModelService.setLocalReservations(localReservations);
    };

    dataModel.clearModel = function () {
      dataModel.data.error = '';
      dataModel.localData.localReservationsError = false;
      dataModel.data.selectedFullReservation = null;
      dataModel.data.reservations = [];
      dataModel.data.selectedReservation = null;
      dataModel.data.localReservations = [];
    };

    dataModel.isReservationBeingModified = function () {
      if (dataModel.data.selectedFullReservation) {
        return true;
      }
      return false;
    };

    dataModel.getSelectedReservation = function () {
      return dataModel.data.selectedReservation;
    };

    dataModel.getSelectedFullReservation = function () {
      return dataModel.data.selectedFullReservation;
    };


    return dataModel;
  }]);
