/* global lqMobileModule, CQ*/
/* jshint maxparams: 13 */
/* jshint maxstatements: 40 */
lqMobileModule.controller('loginModalInstanceController',
  [
    '$scope',
    '$rootScope',
    '$modalInstance',
    'bookingModelService',
    'returnsUserProfileModel',
    'routingService',
    'showLoadingModal',
    'events',
    '$timeout',
    'reservationsModel',
    'satelliteTrackingModelService',
    'localStorageService',
    'secureStorageService',
    function (
      $scope,
      $rootScope,
      $modalInstance,
      bookingModelService,
      returnsUserProfileModel,
      routingService,
      showLoadingModal,
      events,
      $timeout,
      reservationsModel,
      satelliteTrackingModelService,
      localStorageService,
      secureStorageService
    ) {
      'use strict';

      var username;
      var lastname;
      var userId;
      var email;
      var lastName;
      var password;


      $scope.showForgotPasswordForm = false;

      $scope.enableForgotPasswordForm = function () {
        satelliteTrackingModelService.forgotPwdLinkSelected();
        $scope.showForgotPasswordForm = true;
        $scope.signInForm.formSubmitted = false;
        satelliteTrackingModelService.changePasswordModalLoaded();
      };

      function handleSignInError(error) {
        if (typeof error !== 'undefined' && error.data.length > 0) {
          $scope.signInForm.servicesError = CQ.I18n.getMessage(error.data);
        }
        $scope.signInForm.submitDisabled = false;
        $scope.signInForm.signInError = true;
        $scope.signInForm.loadingModal.close();
      }

      $scope.signInForm = {};
      $scope.signInForm.userId = '';
      $scope.signInForm.userLastName = '';
      $scope.signInForm.password = '';
      $scope.signInForm.rememberMe = false;
      $scope.signInForm.signInError = false;
      $scope.signInForm.servicesError = '';

      $scope.signInForm.formSubmitted = false;
      $scope.signInForm.submitDisabled = false;


      //
      // if user has indicated they want their userId saved...
      if (localStorageService.getValue('lqApp.User.remember') !== 'false') {
        username = localStorageService.getValue('lqApp.User.username');
        lastname = localStorageService.getValue('lqApp.User.lastname');
        if (username != undefined && username != '') {
          $scope.signInForm.userId = username;
          $scope.signInForm.userLastName = lastname;
          $scope.signInForm.rememberMe = true;
        } else {
          $scope.signInForm.rememberMe = false;
        }
      }


      $scope.signInForm.signUserIn = function (isUserIdValid, isLastNameValid, isUserPwdValid) {
        $scope.signInForm.formSubmitted = true;
        $scope.signInForm.submitDisabled = true;
        $scope.signInForm.signInError = false;
        $scope.signInForm.servicesError = '';

        //
        // Are basic format-based validations passed?
        if (isUserIdValid && isLastNameValid && isUserPwdValid) {
          //
          // Show loader
          $scope.signInForm.loadingModal = showLoadingModal();


          //
          // Setup parameters for authentication
          userId = '';
          email = '';
          lastName = $scope.signInForm.userLastName;
          password = $scope.signInForm.password;

          if (typeof $scope.signInForm.userId === 'string') {
            if ($scope.signInForm.userId.indexOf('@') === -1) {
              userId = $scope.signInForm.userId;
            } else {
              email = $scope.signInForm.userId;
            }
          } else {
            userId = '';
            email = '';
          }

          //
          // Authenticate the User.
          returnsUserProfileModel.authenticateUser(
            userId, email, lastName, password
          ).then(function (authToken) {
            //
            // Persist the Auth Token
            window.sessionStorage.setItem('token', authToken.token);

            //
            // Load User Profile
            returnsUserProfileModel.getUserProfile(authToken.token).then(
              function () {
                $rootScope.$broadcast(events.signInActionTaken);

                // forward user to Booking form
                routingService.goToBookingGuest();

                // Close modal & loader
                $scope.signInForm.loadingModal.close();
                $modalInstance.close();
              },
              handleSignInError
            );

            //
            // Remember the User/Store credentials
            returnsUserProfileModel.rememberUser(
              userId, email, lastName, password, $scope.signInForm.rememberMe
            );
          }, handleSignInError);
        } else {
          $scope.signInForm.submitDisabled = false;
          $scope.signInForm.signInError = true;
        }
      };

      $scope.signInForm.continueAsGuest = function () {
        bookingModelService.resetPaymentData();
        if (!reservationsModel.isReservationBeingModified()) {
          bookingModelService.resetGuaranteeData();
        }
        routingService.goToBookingGuest();
        satelliteTrackingModelService.continueAsGuestOptionSelected();
        $modalInstance.close();
      };

      $scope.signInForm.rememberMeStateChage = function () {
        if ($scope.signInForm.rememberMe) {
          secureStorageService.isAvailable().then(function () {
            satelliteTrackingModelService.rememberMeOptionSelected();
          }, function () {
            $scope.signInForm.rememberMe = false;
          });
        } else {
          satelliteTrackingModelService.rememberMeOptionDeselected();
        }
      };


      $scope.resetForm = {};

      $scope.cleanResetForm = function () {
        $scope.resetForm.userId = '';
        $scope.resetForm.firstName = '';
        $scope.resetForm.lastName = '';
        $scope.resetForm.mobilePhone = '';
        $scope.resetForm.formSubmitted = false;
        $scope.resetForm.formSubmittedOk = false;
        $scope.resetForm.submitDisabled = false;
        $scope.resetForm.isFormValid = false;
        $scope.resetForm.servicesError = '';
      };

      $scope.cleanResetForm();
      $scope.resetForm.submitForm = function (isFormvalid) {
        $scope.resetForm.isFormValid = isFormvalid;
        $scope.resetForm.formSubmitted = true;
        $scope.resetForm.submitDisabled = true;
        $scope.resetForm.servicesError = '';
        if (isFormvalid) {
          $scope.resetForm.loadingModal = showLoadingModal();
          returnsUserProfileModel.resetUserPassword(
            $scope.resetForm.userId, $scope.resetForm.firstName,
            $scope.resetForm.lastName, $scope.resetForm.mobilePhone
          ).then(function () {
            $scope.resetForm.formSubmittedOk = true;
            $scope.resetForm.formSubmitted = false;
            $scope.resetForm.loadingModal.close();
          }, function (error) {
            $scope.resetForm.submitDisabled = false;
            $scope.resetForm.servicesError = CQ.I18n.getMessage(error);
            $scope.resetForm.loadingModal.close();
          });
        } else {
          $scope.resetForm.submitDisabled = false;
        }
      };

      $scope.getFormFieldCssClass = function (formSubmitted, isFieldValid) {
        if (formSubmitted && !isFieldValid) {
          return 'has-error';
        }
        return '';
      };

      $scope.returnToSignIn = function () {
        $scope.showForgotPasswordForm = false;
        $scope.resetForm.formSubmitted = false;
        $scope.resetForm.formSubmittedOk = false;
      };

      $scope.cancel = function () {
        if ($scope.showForgotPasswordForm) {
          $scope.showForgotPasswordForm = false;
          $scope.cleanResetForm();
        } else {
          $modalInstance.dismiss('cancel');
        }
      };

      $scope.goToHomePage = function () {
        routingService.goToHomePage();
      };
    }]);
