/* global lqMobileModule, console, lqMobile, CQ*/
/* jshint maxparams: 8 */
/* jshint maxstatements: 30 */
lqMobileModule.controller('returnsEditProfileController', ['$scope',
  '$modal',
  '$rootScope',
  '$locale',
  'returnsUserProfileModel',
  'Profile',
  'lodash',
  'events',
  'satelliteTrackingModelService',
  'touchIdService',
  function (
    $scope,
    $modal,
    $rootScope,
    $locale,
    returnsUserProfileModel,
    Profile,
    lodash,
    events,
    satelliteTrackingModelService,
    touchIdService
  ) {
    'use strict';

    var i;
    var actualYear = new Date().getFullYear();
    var saveData;
    var userData = Profile.buildCopy(returnsUserProfileModel.data.userProfile);
    var initTouchIDState;

    $scope.months = $locale.DATETIME_FORMATS.MONTH;
    $scope.years = [];

    // Subtract a month from the credit cards so that the correct month displays
    if (
      !lodash.isNull(userData.creditCard.ccMonth)
      && !lodash.isUndefined(!userData.creditCard.ccMonth)
    ) {
      userData.creditCard.ccMonth -= 1;
    }

    for (i = 0; i < 12; i += 1) {
      $scope.years.push(actualYear + i);
    }

    console.log('lqMobile.Defaults.getDefaultCCMonth() = ' + lqMobile.Defaults.getDefaultCCMonth());

    $scope.creditCardExpirationError = false;
    $scope.minCCMonth = lqMobile.Defaults.getDefaultCCMonth();
    $scope.minCCYear = lqMobile.Defaults.getDefaultCCYear();

    $scope.selectYear = function (year) {
      $scope.editForm.userData.creditCard.ccYear = year;
      $scope.ccYear.isopen = !$scope.ccYear.isopen;
    };

    $scope.selectMonth = function (month) {
      $scope.editForm.userData.creditCard.ccMonth = month;
      $scope.ccMonth.isopen = !$scope.ccMonth.isopen;
    };

    $scope.editForm = {};
    $scope.editForm.userData = userData;
    $scope.editForm.formSubmitted = false;
    $scope.editForm.submitDisabled = false;
    $scope.editForm.serviceErrors = '';
    $scope.editForm.showEnableFingerPrint = false;

    $scope.editForm.fingerPrintStateChange = function () {
      var returnsId;
      if (
        returnsUserProfileModel.hasOwnProperty('data')
        && returnsUserProfileModel.data.hasOwnProperty('userProfile')
        && returnsUserProfileModel.data.userProfile.hasOwnProperty('returnsNumber')
      ) {
        returnsId = returnsUserProfileModel.data.userProfile.returnsNumber;
      } else {
        returnsId = 'unknown';
      }
      satelliteTrackingModelService.fingerprintLoginStateToggled(
        'edit-profile', !!$scope.editForm.enableFingerPrint, returnsId
      );
    };

    $scope.editForm.submitForm = function (isFormValid) {
      var useFingerprint = false;
      $scope.editForm.formSubmitted = true;
      $scope.editForm.submitDisabled = true;
      $scope.editForm.serviceErrors = '';

      if (isFormValid) {
        $scope.loadingModal = $scope.showLoadingModal();

            // Make a copy just before saving because we need to manipulate the ccMonth
        saveData = Profile.buildCopy($scope.editForm.userData);
        if (
          !lodash.isNull(saveData.creditCard.ccMonth)
          && !lodash.isUndefined(saveData.creditCard.ccMonth)
        ) {
          saveData.creditCard.ccMonth += 1;
        }
        // Force the country to be a non US,CA or MX value when the state is *
        if (saveData.state === '*') {
          saveData.country = 'XX';
        }

        if ($scope.editForm.showEnableFingerPrint) {
          useFingerprint = !!$scope.editForm.enableFingerPrint;
        }

        returnsUserProfileModel.editUserProfile(saveData).then(
          function (userProfile) {
            if (
              $scope.editForm.showEnableFingerPrint &&
              useFingerprint !== initTouchIDState
            ) {
              touchIdService.setUsingFingerAuth(useFingerprint);
              if (useFingerprint) {
                returnsUserProfileModel.retrieveCredentials().then(
                  function (creds) {
                    touchIdService.registerWithFingerAuthentication(
                      userProfile.returnsNumber,
                      userProfile.lastName,
                      creds.password
                    ).then(
                      function () {
                        // do nothing
                        touchIdService.touchIdEnabledMessage();
                      },
                      function () {
                        // user failed to auth touchID
                        touchIdService.setUsingFingerAuth(false);
                        touchIdService.touchIdFailureMessage();
                      }
                    );
                  },
                  function () {
                    // welp, nothing to do here :-\
                  }
                );
              } else {
                // user had touch id, and no longer wants it:
                touchIdService.setUsingFingerAuth(false);
                touchIdService.touchIdDisabledMessage();
              }
            }
            $scope.loadingModal.close();
            $scope.openSuccessEditionModal();
            $scope.editForm.submitDisabled = false;
          },
          function (error) {
            console.error('edit profile error');
            console.error(error);
            $scope.editForm.submitDisabled = false;
            $scope.loadingModal.close();
            $scope.editForm.serviceErrors = CQ.I18n.getMessage(error);
            $('.navbar-brand')[0].scrollIntoView();
          }
        );
      } else {
        $scope.editForm.submitDisabled = false;
        $('.navbar-brand')[0].scrollIntoView();
      }
    };

    $scope.openSuccessEditionModal = function () {
      $modal.open({
        templateUrl: 'returnsProfileUpdatedModal.html',
        controller: 'returnsProfileUpdatedModalController',
        windowClass: 'app-modal-window border-radius'
      });
    };

    $scope.ccMonth = {
      isopen: false
    };

    $scope.ccYear = {
      isopen: false
    };

    satelliteTrackingModelService.returnsEditProfileLoaded();

    // check for fingerprint enabling
    touchIdService.userIsUsingFingerAuth(userData.returnsNumber).then(
      function (data) {
        if (data.isCurrent) {
          $scope.editForm.showEnableFingerPrint = true;
          $scope.editForm.enableFingerPrint = data.isUsing;
          initTouchIDState = !!data.isUsing;
        } else {
          $scope.editForm.showEnableFingerPrint = !data.isUsing;
          initTouchIDState = false;
        }
      },
      function () {
        $scope.editForm.showEnableFingerPrint = false;
        initTouchIDState = false;
      }
    );
  }]
);


lqMobileModule.controller('returnsProfileUpdatedModalController', ['$scope', '$modalInstance', 'routingService',
  function ($scope, $modalInstance, routingService) {
    'use strict';

    $scope.ok = function () {
      $modalInstance.dismiss('ok');
      routingService.goToAccountSummary();
    };
  }]);
