/* global lqMobileModule, $, console*/
/* jshint maxparams: 6 */
lqMobileModule.controller('returnsLoginModalController',
  [
    '$scope',
    '$modal',
    '$rootScope',
    'events',
    'returnsUserProfileModel',
    'routingService',
    'hotelInformationModel',
    'satelliteTrackingModelService',
    'touchIdService',
    function (
      $scope,
      $modal,
      $rootScope,
      events,
      returnsUserProfileModel,
      routingService,
      hotelInformationModel,
      satelliteTrackingModelService,
      touchIdService
    ) {
      'use strict';

      $scope.userLoggedIn = returnsUserProfileModel.isLoggedIn();
      $scope.hotelInfo = hotelInformationModel.data.hotelDetails;

      function openModal() {
        // init touch id
        satelliteTrackingModelService.bookingInfoModalLoaded($scope.hotelInfo.hotelId);
        $modal.open({
          templateUrl: 'loginModal.html',
          controller: 'loginModalInstanceController',
          windowClass: 'app-modal-window',
          resolve: {
            showLoadingModal: function () {
              return $scope.showLoadingModal;
            }
          }
        });
      }

      $scope.open = function () {
        touchIdService.userIsUsingFingerAuth().then(
          function (data) {
            if (!data.isUsing) {
              openModal();
            } else {
              touchIdService.promptForFingerAuth().then(
                function (creds) {
                  var loadingModal = $modal.open({
                    templateUrl: 'loadingModalContent.html',
                    windowClass: 'loading-modal-window',
                    backdrop: 'static',
                    keyboard: false
                  });
                  returnsUserProfileModel.authenticateUser(
                    creds.username, '', creds.lastname, creds.password
                  ).then(
                    function (authToken) {
                      //
                      // Persist the Auth Token
                      window.sessionStorage.setItem('token', authToken.token);

                      //
                      // Remember the User/Store credentials
                      returnsUserProfileModel.rememberUser(
                        creds.username, '', creds.lastname, creds.password, false
                      );

                      //
                      // Load User Profile
                      returnsUserProfileModel.getUserProfile(authToken.token).then(
                        function () {
                          loadingModal.close();
                          $rootScope.$broadcast(events.signInActionTaken);
                          // forward user to Booking form
                          routingService.goToBookingGuest();
                        },
                        function () {
                          // @todo: figure out how to display errors
                          // user was able to auth, but we were unable to
                          // retrieve their profile
                        }
                      );
                    },
                    function (error) {
                      loadingModal.close();
                      $rootScope.touchIdAuthError = error;
                      routingService.goToSignInPage();
                    }
                  );
                },
                function () {
                  openModal();
                }
              );
            }
          },
          function () {
            openModal();
          }
        );
      };

      $scope.goToGuestInfo = function () {
        routingService.goToBookingGuest();
      };
    }
  ]);
