/* global lqMobileModule, $timeout, $*/
/* jshint maxparams: 11 */
lqMobileModule.controller('returnsSignInController',
  [
    '$scope',
    '$modal',
    '$rootScope',
    'returnsUserProfileModel',
    'returnsUserDeviceModel',
    'routingService',
    '$timeout',
    '$q',
    'events',
    '$location',
    'satelliteTrackingModelService',
    'persistentStorageModelService',
    'secureStorageService',
    'touchIdService',
    function (
      $scope,
      $modal,
      $rootScope,
      returnsUserProfileModel,
      returnsUserDeviceModel,
      routingService,
      $timeout,
      $q,
      events,
      $location,
      satelliteTrackingModelService,
      persistentStorageModelService,
      secureStorageService,
      touchIdService
    ) {
      'use strict';

      var forceTouchPasswordReset = false;

      function handleError(error) {
        if (typeof error !== 'undefined') {
          $scope.errorMessage = error.data;
        }
        $scope.signInForm.submitDisabled = false;
        $scope.signInForm.signInError = true;
        if ($scope.hasOwnProperty('loadingModal') && $scope.loadingModal.hasOwnProperty('close')) {
          $scope.loadingModal.close();
        }
        $('.navbar-brand')[0].scrollIntoView();
      }

      // $scope.timeoutError = $location.search().iddleTimeOut;
      // if ($scope.timeoutError) {
      //   $location.search('iddleTimeOut', null);
      // }

      if (returnsUserProfileModel.isLoggedIn()) {
        routingService.goToAccountSummary();
      }

      $scope.signInForm = {};
      $scope.signInForm.userId = '';
      $scope.signInForm.userLastName = '';
      $scope.signInForm.password = '';
      $scope.signInForm.rememberMe = false;
      $scope.signInForm.signInError = false;
      $scope.signInForm.showEnableFingerPrint = false;

      $scope.signInForm.formSubmitted = false;
      $scope.signInForm.submitDisabled = false;

  //
  // if user has indicated they want their userId saved...
      persistentStorageModelService.getValue('rememberMe').then(
        function (value) {
          var credPromises = [];
          if (value == true || value == 'true') {
            credPromises.push(
              secureStorageService.getValue('loginName').then(
                function (storedUserName) {
                  secureStorageService.getValue('lastname').then(
                    function (storedLastName) {
                      $scope.signInForm.userId = storedUserName;
                      $scope.signInForm.userLastName = storedLastName;
                      $scope.signInForm.rememberMe = true;
                    },
                    function () {
                      $scope.signInForm.userId = '';
                      $scope.signInForm.userLastName = '';
                      $scope.signInForm.rememberMe = false;
                    }
                  );
                },
                function () {
                  $scope.signInForm.userId = '';
                  $scope.signInForm.userLastName = '';
                  $scope.signInForm.rememberMe = false;
                }
              )
            );
          } else {
            $scope.signInForm.userId = '';
            $scope.signInForm.userLastName = '';
            $scope.signInForm.rememberMe = false;
          }
        },
        function () {
          $scope.signInForm.userId = '';
          $scope.signInForm.userLastName = '';
          $scope.signInForm.rememberMe = false;
        }
      );


      $scope.signInForm.signUserIn = function (isUserIdValid, isLastNameValid, isUserPwdValid) {
        var deferred = $q.defer();
        var email = '';
        var userId = '';
        var lastName = '';
        var password = '';
        var requestingFingerprint = false;
        var signInName = '';

        $scope.signInForm.formSubmitted = true;
        $scope.signInForm.submitDisabled = true;
        $scope.signInForm.signInError = false;
        $scope.errorMessage = '';
        $scope.timeoutError = false;

        //
        // Are basic format-based validations passed?
        if (isUserIdValid && isLastNameValid && isUserPwdValid) {
          $scope.loadingModal = $scope.showLoadingModal();

          //
          // Setup parameters for authentication
          lastName = $scope.signInForm.userLastName;
          password = $scope.signInForm.password;

          requestingFingerprint = !!$scope.signInForm.enableFingerPrint;

          if (typeof $scope.signInForm.userId === 'string') {
            if ($scope.signInForm.userId.indexOf('@') === -1) {
              userId = $scope.signInForm.userId;
            } else {
              email = $scope.signInForm.userId;
            }
          } else {
            userId = '';
            email = '';
          }
          signInName = $scope.signInForm.userId;

          //
          // call Auth method on Returns User model.
          returnsUserProfileModel.authenticateUser(userId, email, lastName, password).then(

          // on Success
          function (authToken) {
            // Persist the Auth Token
            window.sessionStorage.setItem('token', authToken.token);

            // Load the profile
            returnsUserProfileModel.getUserProfile(authToken.token).then(
                function (userProfile) {
                  if (forceTouchPasswordReset) {
                    touchIdService.userIsUsingFingerAuth(userProfile.returnsNumber).then(
                      function (data) {
                        if (data.isUsing && data.isCurrent) {
                          touchIdService.registerWithFingerAuthentication(
                            userProfile.returnsNumber,
                            userProfile.lastName,
                            password
                          ).then(
                            function () {
                              forceTouchPasswordReset = false;
                            }
                          );
                        } else {
                          // touch id saved for a different user account
                          // don't update touch id storage
                        }
                      },
                      function () {
                        //  touch id is unavailable, and you shouldn't ever be here anyway
                      }
                    );
                  }

                  if (requestingFingerprint && !forceTouchPasswordReset) {
                    touchIdService.setUsingFingerAuth(true);
                    touchIdService.registerWithFingerAuthentication(
                      userProfile.returnsNumber,
                      userProfile.lastName,
                      password
                    ).then(
                      function () {
                        //
                        // Remember user and store credentials.
                        returnsUserProfileModel.rememberUser(
                          userProfile.returnsNumber,
                          userProfile.emailAddress,
                          userProfile.lastName,
                          password,
                          $scope.signInForm.rememberMe,
                          signInName
                        );

                        touchIdService.touchIdEnabledMessage();
                      },
                      function () {
                        //
                        // Remember user and store credentials.
                        returnsUserProfileModel.rememberUser(
                          userProfile.returnsNumber,
                          userProfile.emailAddress,
                          userProfile.lastName,
                          password,
                          $scope.signInForm.rememberMe,
                          signInName
                        );

                        // user failed to auth touchID
                        touchIdService.setUsingFingerAuth(false);
                        touchIdService.touchIdFailureMessage();
                      }
                    );
                  } else {
                    //
                    // Remember user and store credentials.
                    returnsUserProfileModel.rememberUser(
                      userProfile.returnsNumber,
                      userProfile.emailAddress,
                      userProfile.lastName,
                      password,
                      $scope.signInForm.rememberMe,
                      signInName
                    );
                  }

                  $rootScope.$broadcast(events.signInActionTaken);

                  //
                  // Trigger the custom LQ
                  returnsUserProfileModel.setDeviceUser();


                  //
                  // Init Push Notifications
                  returnsUserDeviceModel.initPushNotifications(userProfile.returnsNumber);

                  //
                  // Load Account Summary
                  routingService.goToAccountSummary();
                  satelliteTrackingModelService.setLoginStatus(true);
                  satelliteTrackingModelService.setMemberID(userProfile.returnsNumber);
                  satelliteTrackingModelService.setMemberTier(userProfile.returnsTier);

                  $scope.loadingModal.close();
                  deferred.resolve(userProfile);
                },
                function (error) {
                  $scope.signInForm.formSubmitted = true;
                  handleError(error);
                  deferred.reject(error);
                }
            );
          },
          function (error) {
            $scope.signInForm.formSubmitted = true;
            handleError(error);
            deferred.reject(error);
            satelliteTrackingModelService.setLoginStatus(false);
            satelliteTrackingModelService.setMemberID('');
            satelliteTrackingModelService.setMemberTier('');
          }
      );
        } else {
          $scope.signInForm.submitDisabled = false;
          $scope.signInForm.signInError = true;

            // TODO - this message should be internationalized
          deferred.reject('Invalid credentials');
        }
        return deferred.promise;
      };

      $scope.getFormFieldCssClass = function (formSubmitted, isFieldValid) {
        if (formSubmitted && !isFieldValid) {
          return 'has-error';
        }
        return '';
      };

      $scope.joinLQReturns = function () {
        satelliteTrackingModelService.joinLQButtonSelected();
        routingService.goToSingUp();
      };

      $scope.open = function () {
        satelliteTrackingModelService.forgotPwdLinkSelected();
        $modal.open({
          templateUrl: 'passwordModal.html',
          controller: 'forgotPassModalInstanceController',
          windowClass: 'app-modal-window',
          resolve: {
            showLoadingModal: function () {
              return $scope.showLoadingModal;
            },
            $timeout: function () {
              return $timeout;
            }
          }
        });
      };

      $scope.clearId = function () {
        $scope.signInForm.userId = '';
        $scope.boxCleared = true;
        $timeout(function () {
          $('#signInUserId').focus();
        }, 200);
      };

      $scope.signInForm.rememberMeStateChage = function () {
        if ($scope.signInForm.rememberMe) {
          secureStorageService.isAvailable().then(function () {
            satelliteTrackingModelService.rememberMeOptionSelected();
          }, function () {
            $scope.signInForm.rememberMe = false;
          });
        } else {
          satelliteTrackingModelService.rememberMeOptionDeselected();
        }
      };

      $scope.signInForm.fingerPrintStateChange = function () {
        var returnsId;
        if ($.trim($scope.signInForm.userId) !== '') {
          returnsId = $.trim($scope.signInForm.userId);
        } else {
          returnsId = 'unknown';
        }

        satelliteTrackingModelService.fingerprintLoginStateToggled(
          'sign-in', !!$scope.signInForm.enableFingerPrint, returnsId
        );
      };

      function authenticateUserFromTouchID(userID, lastName, password) {
        var deferred = $q.defer();

        //
        // call Auth method on Returns User model.
        returnsUserProfileModel.authenticateUser(userID, '', lastName, password).then(

          // on Success
          function (authToken) {
            // Persist the Auth Token
            window.sessionStorage.setItem('token', authToken.token);

            // Load the profile
            returnsUserProfileModel.getUserProfile(authToken.token).then(
                function (userProfile) {
                  $rootScope.$broadcast(events.signInActionTaken);

                  //
                  // Trigger the custom LQ
                  returnsUserProfileModel.setDeviceUser();

                  //
                  // Remember user and store credentials.
                  returnsUserProfileModel.rememberUser(
                    userProfile.returnsNumber, '', userProfile.lastName, password, $scope.signInForm.rememberMe
                  );

                  //
                  // Init Push Notifications
                  returnsUserDeviceModel.initPushNotifications(userProfile.returnsNumber);

                  //
                  // Load Account Summary
                  routingService.goToAccountSummary();
                  satelliteTrackingModelService.setLoginStatus(true);
                  satelliteTrackingModelService.setMemberID(userProfile.returnsNumber);
                  satelliteTrackingModelService.setMemberTier(userProfile.returnsTier);

                  $scope.loadingModal.close();
                  deferred.resolve(userProfile);
                },
                function (error) {
                  $scope.signInForm.formSubmitted = true;
                  handleError(error);
                  deferred.reject(error);
                  $scope.loadingModal.close();
                }
            );
          },
          function (error) {
            deferred.reject(error);
          }
        );

        return deferred.promise;
      }

      $scope.showFingerPrintPrompt = function () {
        touchIdService.promptForFingerAuth().then(
          function (creds) {
            $scope.loadingModal = $scope.showLoadingModal();
            authenticateUserFromTouchID(
              creds.username,
              creds.lastname,
              creds.password
            ).then(
              function () {
                $scope.loadingModal.close();
              },
              function (error) {
                $scope.signInForm.formSubmitted = true;
                handleError(error);
                satelliteTrackingModelService.setLoginStatus(false);
                satelliteTrackingModelService.setMemberID('');
                satelliteTrackingModelService.setMemberTier('');
                forceTouchPasswordReset = true;
                $scope.signInForm.showEnableFingerPrint = false;
                $scope.signInForm.showOneTouchLogin = false;
              }
            );
          }
        );
      };

      function initTouchID(fromError) {
        if (typeof fromError === 'undefined' || fromError === false) {
          touchIdService.userIsUsingFingerAuth().then(
            function (data) {
              $scope.signInForm.showEnableFingerPrint = !data.isUsing && !forceTouchPasswordReset;
              $scope.signInForm.showOneTouchLogin = data.isUsing && !forceTouchPasswordReset;
            },
            function () {
              $scope.signInForm.showEnableFingerPrint = false;
              $scope.signInForm.showOneTouchLogin = false;
            }
          );
        } else {
          $scope.signInForm.showEnableFingerPrint = false;
          $scope.signInForm.showOneTouchLogin = false;
        }
      }


      satelliteTrackingModelService.returnsIndexLoaded();
      if (!returnsUserProfileModel.skipNextFingerprint && !returnsUserProfileModel.isLoggedIn()) {
        if (typeof $rootScope.touchIdAuthError !== 'undefined' && $rootScope.touchIdAuthError !== false) {
          initTouchID($rootScope.touchIdAuthError);
        } else {
          initTouchID();
        }
      }

      if (returnsUserProfileModel.skipNextFingerprint) {
        returnsUserProfileModel.skipNextFingerprint = false;
      }

      if (typeof $rootScope.touchIdAuthError !== 'undefined' && $rootScope.touchIdAuthError !== false) {
        $scope.signInForm.formSubmitted = true;
        handleError($rootScope.touchIdAuthError);
        forceTouchPasswordReset = true;
        $rootScope.touchIdAuthError = false;
        delete $rootScope.touchIdAuthError;
      }
    }]);
