/* global lqMobileModule, console, CQ*/
/* jshint maxparams: 9 */
lqMobileModule.controller('returnsSignUpController',
  ['$scope', '$modal', '$rootScope', '$q', 'events', 'returnsUserProfileModel', 'routingService', 'Profile', 'favoriteHotelsModel', 'bookingModelService', 'satelliteTrackingModelService', 'touchIdService',
    function (
      $scope,
      $modal,
      $rootScope,
      $q,
      events,
      returnsUserProfileModel,
      routingService,
      Profile,
      favoriteHotelsModel,
      bookingModelService,
      satelliteTrackingModelService,
      touchIdService
    ) {
      'use strict';

      // if the user is already logged, redirect them to the account summary page
      if (returnsUserProfileModel.isLoggedIn()) {
        routingService.goToAccountSummary();
      }

      $scope.signUpForm = {};
      $scope.signUpForm.error = '';
      $scope.signUpForm.firstName = '';
      $scope.signUpForm.lastName = '';
      $scope.signUpForm.emailAddress = '';
      $scope.signUpForm.mobilePhone = '';
      $scope.signUpForm.zipCode = '';
      $scope.signUpForm.password = '';
      $scope.signUpForm.formSubmitted = false;
      $scope.signUpForm.showEnableFingerPrint = false;

      $scope.signUpForm.submitForm = function (isFormValid) {
        var profile;
        var deferred = $q.defer();
        var useFingerprint = false;

        $scope.signUpForm.formSubmitted = true;
        $scope.signUpForm.submitDisabled = true;
        $scope.signUpForm.error = '';
        if (isFormValid) {
          $scope.loadingModal = $scope.showLoadingModal();

          useFingerprint = !!$scope.signUpForm.enableFingerPrint;

          profile = Profile.build();
          profile.firstName = $scope.signUpForm.firstName;
          if (typeof profile.firstName === 'string') {
            profile.firstName = profile.firstName.trim();
          }
          profile.lastName = $scope.signUpForm.lastName;
          //
          profile.zipCode = $scope.signUpForm.zipCode;
          profile.mobilePhone = $scope.signUpForm.mobilePhone;
          profile.emailAddress = $scope.signUpForm.emailAddress;
          //

          return returnsUserProfileModel.createUserProfile(
            profile, $scope.signUpForm.password
          ).then(
            function (response) {
              $scope.loadingModal.close();

              if (useFingerprint) {
                touchIdService.setUsingFingerAuth(true);
                touchIdService.registerWithFingerAuthentication(
                  response.returnsNumber,
                  profile.lastName,
                  $scope.signUpForm.password
                ).then(function () {
                  //
                  // Remember user and store credentials.
                  returnsUserProfileModel.storeCredentials(
                    response.returnsNumber,
                    profile.emailAddress,
                    profile.lastName,
                    $scope.signUpForm.password
                  );

                  $scope.open();
                  deferred.resolve(response);
                  satelliteTrackingModelService.fingerprintLoginStateToggled(
                    'join-lq-returns', true, response.returnsNumber
                  );
                  touchIdService.touchIdEnabledMessage();
                }, function () {
                  //
                  // Remember user and store credentials.
                  returnsUserProfileModel.storeCredentials(
                    response.returnsNumber,
                    profile.emailAddress,
                    profile.lastName,
                    $scope.signUpForm.password
                  );

                  // user failed to auth touchID
                  touchIdService.setUsingFingerAuth(false);
                  touchIdService.touchIdFailureMessage();

                  $scope.open();
                  deferred.resolve(response);
                });
              } else {
                //
                // Remember user and store credentials.
                returnsUserProfileModel.storeCredentials(
                  response.returnsNumber,
                  profile.emailAddress,
                  profile.lastName,
                  $scope.signUpForm.password
                );

                $scope.open();
                deferred.resolve(response);
              }
              favoriteHotelsModel.updateHotelList();
              bookingModelService.updateBookingDataFromProfile();
            },
            function (error) {
              $scope.signUpForm.submitDisabled = false;
              $scope.signUpForm.error = CQ.I18n.getMessage(error);
              $scope.loadingModal.close();
              deferred.reject($scope.signUpForm.error);
            }
         );
        }
        $scope.signUpForm.submitDisabled = false;
        // TODO - this message should be internationalized
        deferred.reject('Invalid form state');
        return deferred.promise;
      };

      $scope.open = function () {
        var modalInstance = $modal.open({
          templateUrl: 'welcomeReturnsModal.html',
          controller: 'welcomeReturnsModalController',
          windowClass: 'app-modal-window border-radius',
          resolve: {
            returnsUserProfileModel: function () {
              return returnsUserProfileModel;
            }
          }
        });

        // if the modal is closed using the button or by clicking outside the modal,
        // we redirect the user
        modalInstance.result.then(function () {
          routingService.goToAccountSummary();
        }, function () {
          routingService.goToAccountSummary();
        });
      };

      // check for fingerprint enabling
      touchIdService.userIsUsingFingerAuth().then(
        function (data) {
          $scope.signUpForm.showEnableFingerPrint = !data.isUsing;
        },
        function () {
          $scope.signUpForm.showEnableFingerPrint = false;
        }
      );

      satelliteTrackingModelService.returnsJoinLoaded();
    }]);

lqMobileModule.controller('welcomeReturnsModalController', ['$scope', '$modalInstance', 'returnsUserProfileModel', 'satelliteTrackingModelService',
  function ($scope, $modalInstance, returnsUserProfileModel, satelliteTrackingModelService) {
    'use strict';

    $scope.userProfile = returnsUserProfileModel.data.userProfile;
    satelliteTrackingModelService.setLoginStatus(true);
    satelliteTrackingModelService.setMemberID(
      returnsUserProfileModel.data.userProfile.returnsNumber
    );
    satelliteTrackingModelService.setMemberTier(
      returnsUserProfileModel.data.userProfile.returnsTier
    );

    $scope.ok = function () {
      $modalInstance.dismiss('ok');
    };
  }]);
