/* global lqMobileModule, console, angular*/
lqMobileModule.factory(

  'returnsUserDeviceEndpointService',

  ['$window',
    '$q',
    '$http',
    'serviceEndPoints',

    function ($window, $q, $http, serviceEndPoints) {
      'use strict';

      var endpointService = {};


    /**
    * registerDevice
    *
    * Register the user's current device to receive Push Notifications from LQ.
    *
    * @param Strong token
    * @param String deviceId
    * @return Promise
    */
      endpointService.registerDevice = function (token, jsonObj) {
      //
      // Send Request
        return $http({
          method: 'POST',
          url: serviceEndPoints.getUserDeviceEndPoint(),
          params: {
            apikey: serviceEndPoints.getApiKey()
          },
          data: jsonObj,
          headers: {
            LQ_AUTH_KEY: token
          }
        }).then(
        // Success
        function (response) {
          return response;
        },
        // Error
        function (error) {
          console.error(JSON.stringify(error));
          throw error;
        }
      );
      };


    /**
    * unregisterDevice
    *
    * Unregister the user's current device so as to no longer receieve Push Notification from LQ.
    *
    * @param Strong token
    * @param String deviceId
    * @return Promise
    */
      endpointService.unregisterDevice = function (token, jsonObj) {
      //
      // Send Request
        return $http({
          method: 'DELETE',
          url: serviceEndPoints.getUserDeviceEndPoint(),
          params: {
            apikey: serviceEndPoints.getApiKey()
          },
          data: jsonObj,
          headers: {
            LQ_AUTH_KEY: token
          }
        }).then(
        // Success
        function (response) {
          return response;
        },
        // Error
        function (error) {
          console.error(JSON.stringify(error));
          throw error;
        }
      );
      };

      return endpointService;
    }]);

