/* global lqMobileModule, console, angular*/
lqMobileModule.factory('returnsUserProfileEndpointService', ['$window',
  '$q',
  '$http',
  'serviceEndPoints',
  'lodash',
  function ($window,
    $q,
    $http,
    serviceEndPoints,
    lodash) {
    'use strict';

    var endpointService = {};

    endpointService.resetUserPassword = function (userId, firstName, lastName, phone) {
      var serviceURL = serviceEndPoints.getUserProfileEndPoint() + '/password/reset';
      var params = {};

      if (userId.indexOf('@') !== -1) {
        params = {
          emailAddress: lodash.trim(userId),
          apikey: serviceEndPoints.getApiKey()
        };
      } else {
        params = {
          returnsId: lodash.trim(userId),
          apikey: serviceEndPoints.getApiKey()
        };
      }
      return $http({
        method: 'get',
        url: serviceURL,
        params: params,
        headers: {
          'Cache-Control': 'no-cache'
        }
      }).then(
        // Success
        function (response) {
          return response;
        },
        // Error
        function (error) {
          console.error(JSON.stringify(error));
          throw error;
        }
      );
    };

    endpointService.createUserProfile = function (jsonObj, password) {
      return $http({
        method: 'post',
        url: serviceEndPoints.getUserProfileEndPoint(),
        data: jsonObj,
        params: {
          apikey: serviceEndPoints.getApiKey()
        },
        headers: {
          LQ_AUTH_PASSWORD: password
        }
      }).then(
        // Success
        function (userProfile) {
          return userProfile;
        },
        // Error
        function (error) {
          console.error(JSON.stringify(error));
          throw error;
        }
      );
    };

    endpointService.editUserProfile = function (jsonObj, authToken) {
      return $http({
        method: 'put',
        url: serviceEndPoints.getUserProfileEndPoint(),
        params: {
          apikey: serviceEndPoints.getApiKey()
        },
        data: jsonObj,
        headers: {
          LQ_AUTH_KEY: authToken
        }
      }).then(
        // Success
        function (response) {
          return response;
        },
        // Error
        function (error) {
          console.error(JSON.stringify(error));
          throw error;
        }
      );
    };

    endpointService.authenticateUser = function (jsonObj) {
      return $http({
        method: 'post',
        url: serviceEndPoints.getReturnsAuthenticationEndPoint(),
        params: {
          apikey: serviceEndPoints.getApiKey()
        },
        data: jsonObj
      }).then(
        // Success
        function (response) {
          return response;
        },
        // Error
        function (error) {
          console.error(JSON.stringify(error));
          throw error;
        }
      );
    };

    endpointService.getUserProfile = function (token) {
      return $http({
        method: 'get',
        url: serviceEndPoints.getUserProfileEndPoint(),
        params: {
          apikey: serviceEndPoints.getApiKey()
        },
        headers: {
          LQ_AUTH_KEY: token,
          'Cache-Control': 'no-cache'
        }
      }).then(
        // Success
        function (response) {
          return response;
        },
        // Error
        function (error) {
          console.error(JSON.stringify(error));
          throw error;
        }
      );
    };

    endpointService.getUserActivities = function (token, startDate, endDate) {
      return $http({
        method: 'get',
        url: serviceEndPoints.getAccountActivityUrlEndPoint(),
        params: {
          apikey: serviceEndPoints.getApiKey(),
          startdate: startDate,
          enddate: endDate
        },
        headers: {
          LQ_AUTH_KEY: token,
          'Cache-Control': 'no-cache'
        }
      }).then(
        function (response) {
          return response;
        },
        function (error) {
          throw error;
        }
      );
    };

    endpointService.addCreditCard = function (jsonObj, token) {
      return $http({
        method: 'POST',
        url: serviceEndPoints.getCreditCardsEndPoint(),
        params: {
          apikey: serviceEndPoints.getApiKey()
        },
        data: jsonObj,
        headers: {
          LQ_AUTH_KEY: token
        }
      }).then(
        // Success
        function (response) {
          return response;
        },
        // Error
        function (error) {
          console.error(JSON.stringify(error));
          throw error;
        }
      );
    };

    endpointService.replacePrimaryCreditCard = function (jsonObj, token) {
      return $http({
        method: 'POST',
        url: serviceEndPoints.getCreditCardsReplaceEndPoint(),
        params: {
          apikey: serviceEndPoints.getApiKey()
        },
        data: jsonObj,
        headers: {
          LQ_AUTH_KEY: token
        }
      }).then(
        // Success
        function (response) {
          return response;
        },
        // Error
        function (error) {
          console.error(JSON.stringify(error));
          throw error;
        }
      );
    };

    endpointService.deleteCreditCard = function (lastFour, cardType, token) {
      return $http({
        method: 'DELETE',
        url: serviceEndPoints.getCreditCardsEndPoint(),
        params: {
          lastFour: lastFour,
          cardType: cardType,
          apikey: serviceEndPoints.getApiKey()
        },
        headers: {
          LQ_AUTH_KEY: token
        }
      }).then(
        // Success
        function (response) {
          return response;
        },
        // Error
        function (error) {
          console.error(JSON.stringify(error));
          throw error;
        }
      );
    };

    endpointService.modifyCreditCard = function (jsonObj, token) {
      return $http({
        method: 'PUT',
        url: serviceEndPoints.getCreditCardsEndPoint(),
        params: {
          apikey: serviceEndPoints.getApiKey()
        },
        data: jsonObj,
        headers: {
          LQ_AUTH_KEY: token
        }
      }).then(
        // Success
        function (response) {
          return response;
        },
        // Error
        function (error) {
          console.error(JSON.stringify(error));
          throw error;
        }
      );
    };

    endpointService.getCreditCards = function (token) {
      return $http({
        method: 'GET',
        url: serviceEndPoints.getCreditCardsEndPoint(),
        params: {
          apikey: serviceEndPoints.getApiKey()
        },
        headers: {
          LQ_AUTH_KEY: token,
          'Cache-Control': 'no-cache'
        }
      }).then(
        // Success
        function (response) {
          return response;
        },
        // Error
        function (error) {
          console.error(JSON.stringify(error));
          throw error;
        }
      );
    };

    endpointService.changePassword = function (returnsId, password, newPassword, authToken) {
      return $http({
        method: 'GET',
        url: serviceEndPoints.getChangePasswordEndPoint(returnsId),
        params: {
          apikey: serviceEndPoints.getApiKey()
        },
        headers: {
          LQ_AUTH_PASSWORD: password,
          LQ_AUTH_NEW_PASSWORD: newPassword,
          LQ_AUTH_KEY: authToken
        }
      }).then(
        // Success
        function (response) {
          return response;
        },
        // Error
        function (error) {
          console.error(JSON.stringify(error));
          throw error;
        }
      );
    };


    /**
    * registerDevice
    *
    * Register the user's current device to receive Push Notifications from LQ.
    *
    * @param Strong token
    * @param String deviceId
    * @return Promise
    */
    endpointService.registerDevice = function (token, deviceId) {
      //
      // Create JSON payload
      var jsonObject = JSON.stringify({
        device: deviceId
      });

      //
      // Send Request
      return $http({
        method: 'POST',
        url: serviceEndPoints.getUserDeviceEndPoint(),
        params: {
          apikey: serviceEndPoints.getApiKey()
        },
        data: jsonObject,
        headers: {
          LQ_AUTH_KEY: token
        }
      }).then(
        // Success
        function (response) {
          return response;
        },
        // Error
        function (error) {
          console.error(JSON.stringify(error));
          throw error;
        }
      );
    };


    /**
    * unregisterDevice
    *
    * Unregister the user's current device so as to no longer receieve Push Notification from LQ.
    *
    * @param Strong token
    * @param String deviceId
    * @return Promise
    */
    endpointService.unregisterDevice = function (token, deviceId) {
      //
      // Create JSON payload
      var jsonObject = JSON.stringify({
        device: deviceId
      });

      //
      // Send Request
      return $http({
        method: 'DELETE',
        url: serviceEndPoints.getUserDeviceEndPoint(),
        params: {
          apikey: serviceEndPoints.getApiKey()
        },
        data: jsonObject,
        headers: {
          LQ_AUTH_KEY: token
        }
      }).then(
        // Success
        function (response) {
          return response;
        },
        // Error
        function (error) {
          console.error(JSON.stringify(error));
          throw error;
        }
      );
    };


    return endpointService;
  }]);
