/* global lqMobileModule, CQ*/
/* jshint maxparams: 11 */
lqMobileModule.factory(

  'returnsUserProfileModel',

  ['$q',
    '$filter',
    '$rootScope',
    'storageModelService',
    'returnsUserProfileEndpointService',
    'Returns',
    'ReturnsNumber',
    'StoredCreditCard',
    'lodash',
    'Profile',
    'UserActivity',
    'events',
    'ccTokenizationService',
    'mobileCheckoutModel',
    'returnsUserDeviceModel',
    'freeNightModel',
    'satelliteTrackingModelService',
    'persistentStorageModelService',
    'secureStorageService',
    'localStorageService',
    'activeStaysModel',
    function (
      $q,
      $filter,
      $rootScope,
      storageModelService,
      returnsUserProfileEndpointService,
      Returns,
      ReturnsNumber,
      StoredCreditCard,
      lodash,
      Profile,
      UserActivity,
      events,
      ccTokenizationService,
      mobileCheckoutModel,
      returnsUserDeviceModel,
      freeNightModel,
      satelliteTrackingModelService,
      persistentStorageModelService,
      secureStorageService,
      localStorageService,
      activeStaysModel
    ) {
      'use strict';

      //
      // Initialzie the data model.
      var dataModel = {
        data: {
        //
        // userProfile is the mapped domain entity object of the profile (keep)
          userProfile: {},
          userActivities: []
        }
      };


      /**
      * String errorMsg
      */
      var errorMsg = 'We are unable to update your La Quinta Returns account information at this time. Please call 1-800-642-4258 for assistance.';

      /**
      * Strig magicToken
      */
      var magicToken = '888888888888';

      /**
      * Function credentialsCallback
      */
      dataModel.credentialsCallback = null;

      /**
      * Object storedCredentials
      */
      dataModel.storedCredentials = {
        userid: null,
        email: null,
        lastname: null,
        password: null,
        skipNextFingerprint: false
      };

      function buildCardObject(creditCard) {
        return {
          ccNumber: creditCard.cardNumber,
          ccExpMonth: creditCard.expirationDate.substring(5, 7),
          ccExpYear: creditCard.expirationDate.substring(0, 4)
        };
      }

      function isMasked(ccNumber) {
        return ccNumber.indexOf('*') > -1;
      }

      function handleMLError(deferred, error) {
        if (error.errorCode === 567) {
          deferred.reject(errorMsg);
        } else {
          deferred.reject(error.errorDescription);
        }
      }

      function addCreditCardIfValid(deferred, newCard, authToken) {
        var newCardRequest = newCard.buildRequest();

        if (newCard.isValid()) {
          ccTokenizationService.tokenize(buildCardObject(newCard.creditCard)).then(
        function (response) {
          newCard.creditCard.cardNumber = response.token;


          if (lodash.startsWith(response.token, magicToken)) {
            deferred.reject(errorMsg);
          } else {
            returnsUserProfileEndpointService.addCreditCard(newCardRequest, authToken).then(
                function (iResponse) {
                  deferred.resolve(iResponse);
                },
                function (error) {
                  deferred.reject(error.data);
                }
              );
          }
        },
        function (error) {
          handleMLError(deferred, error);
        }
      );
        } else {
          deferred.resolve({});
        }
      }

      function deleteCreditCard(deferred, lastFour, cardType, authToken) {
        returnsUserProfileEndpointService.deleteCreditCard(lastFour, cardType, authToken).then(
        function (response) {
          deferred.resolve(response);
        },
        function (error) {
          deferred.reject(error.data);
        }
    );
      }

      function modifyCreditCard(deferred, newCard, authToken) {
        var newCardRequest = newCard.buildRequest();

        if (isMasked(newCardRequest.StoredCreditCard.creditCard.cardNumber)) {
          returnsUserProfileEndpointService.modifyCreditCard(newCardRequest, authToken).then(
        function (response) {
          deferred.resolve(response);
        },
        function (error) {
          deferred.reject(error.data);
        }
      );
        } else {
          ccTokenizationService.tokenize(buildCardObject(newCard.creditCard)).then(
        function (response) {
          newCard.creditCard.cardNumber = response.token;
          if (lodash.startsWith(response.token, magicToken)) {
            console.log('rejections...' + errorMsg);
            deferred.reject(errorMsg);
          } else {
            returnsUserProfileEndpointService.modifyCreditCard(newCardRequest, authToken).then(
                function (iResponse) {
                  deferred.resolve(iResponse);
                },
                function (error) {
                  deferred.reject(error.data);
                }
              );
          }
        },
        function (error) {
          handleMLError(deferred, error);
        });
        }
      }

      function replaceCreditCardIfValid(deferred, newCard, authToken) {
        var newCardRequest = newCard.buildRequest();

        if (newCard.isValid()) {
          if (isMasked(newCardRequest.StoredCreditCard.creditCard.cardNumber)) {
            returnsUserProfileEndpointService.replacePrimaryCreditCard(
              newCardRequest, authToken
            ).then(
              function (response) {
                deferred.resolve(response);
              },
              function (error) {
                deferred.reject(error.data);
              }
            );
          } else {
            ccTokenizationService.tokenize(
              buildCardObject(newCard.creditCard)
            ).then(
              function (response) {
                newCard.creditCard.cardNumber = response.token;
                if (lodash.startsWith(response.token, magicToken)) {
                  deferred.reject(errorMsg);
                } else {
                  returnsUserProfileEndpointService.replacePrimaryCreditCard(
                    newCardRequest, authToken
                  ).then(
                    function (iResponse) {
                      deferred.resolve(iResponse);
                    },
                    function (error) {
                      deferred.reject(error.data);
                    }
                  );
                }
              },
              function (error) {
                handleMLError(deferred, error);
              }
            );
          }
        } else {
          deferred.resolve({});
        }
      }

      function areCardsSame(existingCard, newCard) {
        if (!lodash.has(existingCard, 'creditCard.creditCardLastFour') ||
        !lodash.has(existingCard, 'creditCard.creditCardType') ||
        !lodash.has(existingCard, 'creditCard.expirationDate')) {
          throw 'Incorrect type for parameter existingCard';
        }
        if (!(newCard instanceof StoredCreditCard)) {
          throw 'Incorrect type for parameter newCard';
        }

        return existingCard.creditCard.creditCardLastFour === newCard.creditCard.creditCardLastFour
            && existingCard.creditCard.creditCardType === newCard.creditCard.creditCardType;
      }

      function findPrimaryCard(existingCards) {
        var existingCard;
        if (!lodash.isNull(existingCards)) {
          existingCard = lodash.findWhere(existingCards, {
            primary: true
          });
        }

        return existingCard;
      }

      function updateCreditCardInformation(profile, existingCards, authToken) {
        var deferred = $q.defer();
        var existingCard = findPrimaryCard(existingCards);
        var newCard = StoredCreditCard.map(profile);

        if (!(profile instanceof Profile)) {
          throw 'Incorrect type for parameter profile';
        }

        if (lodash.isNull(existingCard) || lodash.isUndefined(existingCard)) {
          addCreditCardIfValid(deferred, newCard, authToken);
        } else if (newCard.isCardNumberEmpty()) {
          deleteCreditCard(
            deferred, existingCard.creditCard.creditCardLastFour,
            existingCard.creditCard.creditCardType, authToken
          );
        } else if (areCardsSame(existingCard, newCard)) {
          modifyCreditCard(deferred, newCard, authToken);
        } else {
          replaceCreditCardIfValid(deferred, newCard, authToken);
        }

        return deferred.promise;
      }


      /**
      * setUserProfile
      *
      * Setup the profile, store in Storage.
      *
      * @param {Profile} userProfile
      */
      function setUserProfile(userProfile) {
        // Validate mapping
        if (!(userProfile instanceof Profile)) {
          throw 'Unsupported mapping to Returns';
        }

        // Store in model state
        dataModel.data.userProfile = userProfile;

        // Persist data in Storage or other data Store
        storageModelService.setUserProfile(dataModel.data.userProfile);
        satelliteTrackingModelService.setMemberID(userProfile.returnsNumber);
        satelliteTrackingModelService.setMemberTier(userProfile.returnsTier);
      }

      /**
      * getUserProfile
      *
      * Retrieve User Profile data from Store
      *
      * @return Profile
      */
      function getUserProfile() {
        var userProfileFromSessionStorage = storageModelService.getUserProfile();

        //
        // Checking that we have a valid profile in the storage model service,
        // by looking at a property
        if (userProfileFromSessionStorage && userProfileFromSessionStorage.hasOwnProperty('returnsNumber')) {
          return userProfileFromSessionStorage;
        }

        return ''; // @todo why are we returning a String?
      }

      /**
      * updateUserProfile
      *
      * @todo What is the ultimate purpose of this method?
      */
      function updateUserProfile() {
        var userProfile = getUserProfile();

        if (userProfile) {
        //
        // Make a new copy to ensure correct object type.
          dataModel.data.userProfile = Profile.buildCopy(userProfile);
        }

        storageModelService.setUserProfile(dataModel.data.userProfile);
        satelliteTrackingModelService.setMemberID(userProfile.returnsNumber);
        satelliteTrackingModelService.setMemberTier(userProfile.returnsTier);
      }


      /**
      * buildAuthenticateJSON
      *
      * Build an object for the authentication web service.
      *
      * @param String userid
      * @param String email
      * @param String lastname
      * @param String password
      * @return Object
      */
      function buildAuthenticateJSON(userid, email, lastname, password) {
        var hasUserId = typeof userid === 'string' && userid !== '';
        var hasEmail = typeof email === 'string' && email !== '';
        if (hasUserId && hasEmail) {
          email = '';
        }

        if (typeof email === 'string') {
          email = email.trim();
        }

        if (typeof lastname === 'string') {
          lastname = lastname.trim();
        }

        return {
          LoyaltyServiceAuthenticateRequest: {
            email: email,
            lastName: lastname,
            returnsId: userid,
            password: password
          }
        };
      }


      /**
      * isLoggedIn
      *
      * Determine if the app user is logged in to Returns account.
      *
      * @return Boolean
      */
      dataModel.isLoggedIn = function () {
        var userProfileFromSessionStorage;

        if (dataModel.data.userProfile.returnsNumber !== undefined
            && dataModel.data.userProfile.returnsNumber !== null) {
          return true;
        }
        userProfileFromSessionStorage = storageModelService.getUserProfile();
        if (userProfileFromSessionStorage !== '' && !lodash.isEmpty(userProfileFromSessionStorage)) {
          updateUserProfile();
          return true;
        }
        return false;
      };

      /**
      * logoutUser
      *
      * Process the user's sign out request. Remove all local data
      * of the user.
      */
      dataModel.logoutUser = function () {
        //
        // empty out userProfile obj
        dataModel.data.userProfile = { };
        storageModelService.setUserProfile({ });

        //
        // Remove the Auth token from session
        window.sessionStorage.removeItem('token');

        //
        // Update all items dependent on the User
        mobileCheckoutModel.reset();
        freeNightModel.reset();
        activeStaysModel.reset();

        persistentStorageModelService.getValue('rememberMe').then(
          function (value) {
            if (typeof value === 'undefined' || value === 'false' || value === false || value === null) {
              persistentStorageModelService.removeValue('rememberMe');
              secureStorageService.removeValue('loginName');
            }
          },
          function () {
            // consider this a false response:
            persistentStorageModelService.removeValue('rememberMe');
            secureStorageService.removeValue('loginName');
          }
        );

        satelliteTrackingModelService.setLoginStatus(false);
        satelliteTrackingModelService.setMemberID('');
        satelliteTrackingModelService.setMemberTier('');
        //
        // Clear credentias from local storage and KeyChain.
        dataModel.clearCredentials();
      };


      /**
      * authenticateUser
      *
      * Authenticate the user usig the LQ web service.
      *
      * @param String userid
      * @param String email
      * @param String lastname
      * @param String password
      * @param String rememberme
      * @return Promise
      */
      dataModel.authenticateUser = function (userid, email, lastname, password) {
        //
        // Pass the auth JSON to the service call.
        var json = buildAuthenticateJSON(userid, email, lastname, password);
        return returnsUserProfileEndpointService.authenticateUser(json).then(

        //
        // Authentication Success!
        function (response) {
          return response.data;
        },

        // Auth error :-(
        function (error) {
          throw error;
        }

      );
      };

      /**
      * getUserProfile
      *
      * Load the User's profile from the LQ Web Service
      *
      * @param String token
      * @return Promise
      */
      dataModel.getUserProfile = function (token) {
        var deferred = $q.defer();
        var domainProfile;

        //
        // Get the profile from services
        returnsUserProfileEndpointService.getUserProfile(token).then(

        // Profile Success!
        function (response) {
          //
          // Then call the credit card from services
          returnsUserProfileEndpointService.getCreditCards(token).then(

            // Credit Card Success!
            function (storedCreditCards) {
              //
              // Create the Profile object and add it to our Returns user Model.
              domainProfile = Profile.map(response.data, storedCreditCards.data);
              setUserProfile(domainProfile);

              deferred.resolve(domainProfile);
            },

            // Credit Card error.
            function (error) {
              deferred.reject(error);
            }
          );
        },

        // Profile error.
        function (error) {
          deferred.reject(error);
        }

      );

        return deferred.promise;
      };

      /**
      * createUserProfile
      *
      * Enroll the user in the LQ Returns program. Utilizes account creation web service.
      *
      * @param Object data
      * @param String password
      * @return Promise
      */
      dataModel.createUserProfile = function (data, password) {
        //
        // Special case where we need to use a different promise.
        // This is needed because we need to wait until the response
        // from the nested, asychronous authenticate comes back before returning
        var deferred = $q.defer();
        var authenticateJSON;
        var domainProfile;

        //
        // Crete profile service call
        returnsUserProfileEndpointService.createUserProfile(Returns.map(data), password).then(

        // Profile Creation success!
        function (response) {
          //
          // using the response, set the profile
          domainProfile = Profile.map(response.data, null);
          setUserProfile(domainProfile);

          //
          // Then call the authenticate service and save the token
          authenticateJSON = buildAuthenticateJSON(response.data.returnsNumber.id, '', response.data.lastName, password);
          returnsUserProfileEndpointService.authenticateUser(authenticateJSON).then(

            // Auth Success!
            function (tokenResponse) {
              window.sessionStorage.setItem('token', tokenResponse.data.token);
              deferred.resolve(domainProfile);
            },

            // Auth failed.
            function (error) {
              deferred.reject(error);
            }

          );
        },

        // Profile Creation failed.
        function (error) {
          deferred.reject(error.data);
        }

      );

        return deferred.promise;
      };


      /**
      * editUserProfile
      *
      * Update the user's profile. Utilizes LQ web services to persist changes to User's account.
      *
      * @param Profile profile
      * @return Promise
      */
      dataModel.editUserProfile = function (profile) {
        var deferred = $q.defer();
        var authToken = window.sessionStorage.getItem('token');

        //
        // Call editUserProfile service
        returnsUserProfileEndpointService.editUserProfile(Returns.map(profile), authToken).then(

        // Edit Profile success!
        function () {
          //
          // Get stored credit cards.
          returnsUserProfileEndpointService.getCreditCards(authToken).then(

            // Credit Card Success!
            function (storedCreditCards) {
              //
              // Update Stored Credit Cards
              updateCreditCardInformation(
                profile, storedCreditCards.data, authToken
              ).then(function () { // Update Credit Card Success!
                //
                // Requery services so the credit cards appear masked and other fields are sync'd.
                dataModel.getUserProfile(authToken).then(
                  // Profile success!
                  function () {
                    deferred.resolve(profile);
                  },
                  // Profile error.
                  function (error) {
                    deferred.reject(error);
                  }
                );
              },
              // Update credit card error.
              function (error) {
                deferred.reject(error);
              }
              );
            },
            // Credit Card error.
            function (error) {
              deferred.reject(error);
            }
          );
        },
        // Edit Profile error.
        function (error) {
          deferred.reject(error.data);
        }

      );

        return deferred.promise;
      };


      /**
      * resetUserPassword
      *
      * Begin the reset password workflow for the Returns user.
      *
      * @param String userId
      * @param String firstName
      * @param String lastName
      * @param String phone
      */
      dataModel.resetUserPassword = function (userId, firstName, lastName, phone) {
        //
        // Call Reset Password web service
        return returnsUserProfileEndpointService.resetUserPassword(
          userId, firstName, lastName, phone
        ).then(function (response) {
          return response;
        }, function (error) {
          throw CQ.I18n.getMessage(error.data);
        });
      };


      /**
      * changeUserPassword
      *
      * Submit request to Change the Returns user's password.
      *
      * @param String returnsId
      * @param String oldPassword
      * @param String newPassword
      * @param String authToken
      */
      dataModel.changeUserPassword = function (returnsId, oldPassword, newPassword, authToken) {
        //
        // Call Change Password web service
        return returnsUserProfileEndpointService.changePassword(
          returnsId, oldPassword, newPassword, authToken
        ).then(function (response) {
          return response;
        }, function (error) {
          throw CQ.I18n.getMessage(error.data);
        });
      };


      /**
      * rememberUser
      *
      * Authenticate the user usig the LQ web service.
      *
      * @param String userid
      * @param String email
      * @param String lastname
      * @param String password
      * @param String rememberme
      * @return Promise
      */
      dataModel.rememberUser = function (
        userid, email, lastname, password, rememberMe, signInName
      ) {
        //
        // flag that userId should be saved
        if (typeof signInName !== 'undefined' && lodash.trim(signInName) !== '') {
          persistentStorageModelService.setValue('rememberMe', JSON.stringify(rememberMe)).then(
            function () {
              if (rememberMe && signInName !== '') {
                secureStorageService.setValue('loginName', signInName).then(
                  function () {
                    secureStorageService.setValue('lastname', lastname);
                  },
                  function (msg) {
                    console.log(JSON.stringify(msg) + ' -- error saving username');
                  });
              }
            },
            function (msg) {
              console.log(JSON.stringify(msg) + ' -- error saving remember');
            }
          );
        }

        //
        // when you authenticate the user, save their auth in localStorage
        // auth data is only removed when they log out

        // window.localStorage.setItem('lqApp.User.lastname', lastname);

        //
        // Pass credential to the device for Sedure Storage
        dataModel.storeCredentials(userid, email, lastname, password);
      };


      /**
      * authenticateUserFromStorage
      *
      * Authenticate the user from credentials stored in localStorage. This is
      * done to persist the user login and not require entering of crednetials
      * every time the app starts.
      */
      dataModel.authenticateUserFromStorage = function () {
        //
        // Retrieve credentials from Secure Storage
        return dataModel.retrieveCredentials().then(

        //
        // Successfully retrieved credentials.
        function (creds) {
          //
          // Attempt to authenticate the user
          return dataModel.authenticateUser(
            creds.userid, creds.email, creds.lastname, creds.password
          ).then(

            //
            // Success
            function (authToken) {
              //
              // Store the Token.
              window.sessionStorage.setItem('token', authToken.token);

              //
              // Load the profile.
              return dataModel.getUserProfile(authToken.token).then(

                // Profile retrieved
                function (userProfile) {
                  var primaryUser;

                  satelliteTrackingModelService.signInActionTaken();

                  //
                  // Trigger the custom LQ
                  dataModel.setDeviceUser();

                  //
                  // Init Push Notifictions (if no previous user)
                  primaryUser = localStorageService.getValue(
                    returnsUserDeviceModel.RETURNS_ACCOUNT
                  );

                  if (primaryUser == null) {
                    returnsUserDeviceModel.initPushNotifications(userProfile.returnsNumber);
                  }

                  //
                  // Return the profile.
                  return userProfile;
                },

                // Profile error!
                function (error) {
                  throw error;
                }
              );
            },
            function (err) {
              throw err;
            }

          );
        },

        //
        // No credentials found.
        function (error) {
          throw error;
        }

      );
      };

      /**
      * retrieveCredentials
      *
      * Loads credentials from SecureStorage object, passes them to specified
      * callback.
      *
      * @param callback
      */
      dataModel.retrieveCredentials = function () {
        var deferred = $q.defer();
        var creds = {
          userid: '',
          email: '',
          lastname: '',
          password: ''
        };

        secureStorageService.getValue('userid').then(
          function (userId) {
            creds.userid = userId;
            secureStorageService.getValue('lastname').then(
              function (lastName) {
                creds.lastname = lastName;
                secureStorageService.getValue('password').then(
                  function (password) {
                    creds.password = password;
                    deferred.resolve(creds);
                  },
                  function () {
                    deferred.reject();
                  }
                );
              },
              function () {
                deferred.reject();
              }
            );
          },
          function () {
            deferred.reject();
          }
        );

        return deferred.promise;
      };

      /**
      * storeCredentials
      *
      * Stores user credentials from SecureStorage object.
      *
      * @oaram String userid
      * @param String email
      * @param String lastname
      * @param String password
      */
      dataModel.storeCredentials = function (userid, email, lastname, password) {
        var deferred = $q.defer();

        //
        // Place userid and email in localStorage
        secureStorageService.setValue('userid', userid).then(
          function () {
            secureStorageService.setValue('email', email).then(
              function () {
                secureStorageService.setValue('lastname', lastname).then(
                  function () {
                    secureStorageService.setValue('password', password).then(
                      function () {
                        deferred.resolve(true);
                      },
                      function (msg) {
                        console.log(JSON.stringify(msg) + ' -- error saving _password');
                        deferred.reject();
                      }
                    );
                  },
                  function (msg) {
                    console.log(JSON.stringify(msg) + ' -- error saving _lastname');
                    deferred.reject();
                  }
                );
              },
              function (msg) {
                console.log(JSON.stringify(msg) + ' -- error saving _email');
                deferred.reject();
              }
            );
          },
          function (msg) {
            console.log(JSON.stringify(msg) + ' -- error saving _userid');
            deferred.reject();
          }
        );

        return deferred.promise;
      };


      /**
      * hasCredentials
      *
      * Return true if user credentials have been stored and retrieved from device, false otherwise.
      *
      * @return bool
      */
      dataModel.hasCredentials = function () {
        var creds = dataModel.storedCredentials;
        return (creds.userid || creds.email) && creds.lastname && creds.password;
      };


      /**
      * clearCredentials
      *
      * Removes all credentials from localStorage and keychain so that login is no longer persisted.
      */
      dataModel.clearCredentials = function () {
        var deferred = $q.defer();

        //
        // Remove userid and email from local storage
        secureStorageService.removeValue('userid').then(
          function () {
            secureStorageService.removeValue('email').then(
              function () {
                secureStorageService.removeValue('password').then(
                  function () {
                    deferred.resolve(true);
                  },
                  function () {
                    deferred.resolve(true);
                  }
                );
              },
              function () {
                secureStorageService.removeValue('password').then(
                  function () {
                    deferred.resolve(true);
                  },
                  function () {
                    deferred.resolve(true);
                  }
                );
              }
            );
          },
          function () {
            secureStorageService.removeValue('email').then(
              function () {
                secureStorageService.removeValue('password').then(
                  function () {
                    deferred.resolve(true);
                  },
                  function () {
                    deferred.resolve(true);
                  }
                );
              },
              function () {
                secureStorageService.removeValue('password').then(
                  function () {
                    deferred.resolve(true);
                  },
                  function () {
                    deferred.resolve(true);
                  }
                );
              }
            );
          }
        );

        return deferred.promise;
      };


      /**
      * setDeviceUser
      *
      * Set the current user for the device.
      *
      * @param String returnsId
      */
      dataModel.setDeviceUser = function () {
        var ev;

        //
        // validate the Returns Id
        if (typeof dataModel.data.userProfile.returnsNumber === 'undefined') {
          console.error('Could not validate Returns ID: ');
          console.error(dataModel.data.userProfile.returnsNumber);
          return;
        }

        //
        // Create event
        ev = document.createEvent('Event');
        ev.initEvent('deviceHasUser', true, true);
        ev.userProfile = dataModel.data.userProfile;

        //
        // Dispatch event
        document.dispatchEvent(ev);
      };


      /**
      * getUserProfileData
      *
      * Retrieve the user's profile data.
      *
      * @return Object
      */
      dataModel.getUserProfileData = function () {
        return dataModel.data.userProfile;
      };


      dataModel.getUserActivities = function (monthsBack, token) {
        var date = new Date();
        var index = 0;
        var activitiesArray = [];
        var activitiesLength = 0;
        var endDate = $filter('date')(date, 'yyyy-MM-dd');
        var startDate;
        var userActivities = [];

        date.setMonth(date.getMonth() - monthsBack);
        startDate = $filter('date')(date, 'yyyy-MM-dd');

        dataModel.data.userActivities = [];
        return returnsUserProfileEndpointService.getUserActivities(token, startDate, endDate).then(
        function (response) {
          activitiesArray = response.data;
          activitiesLength = activitiesArray.length;
          for (index = 0; index < activitiesLength; index += 1) {
            userActivities.unshift(UserActivity.map(activitiesArray[index]));
          }
          dataModel.data.userActivities = userActivities;
          return userActivities;
        },
        function (error) {
          throw error;
        }
    );
      };


      dataModel.redeemPoints = function (points) {
        // @todo Figure out why this doesn't work as a function in pointsSummary
        var pointsSummary = dataModel.data.userProfile.pointsSummary;
        pointsSummary.currentPointsBalance -= points;
        pointsSummary.pointsRedeemedYTD += points;
      };

      dataModel.restorePoints = function (points) {
        // @todo Figure out why this doesn't work as a function in pointsSummary
        var pointsSummary = dataModel.data.userProfile.pointsSummary;
        pointsSummary.currentPointsBalance += points;
        pointsSummary.pointsRedeemedYTD -= points;
      };

      updateUserProfile();

      return dataModel;
    }]);
