/* global lqMobileModule */
lqMobileModule.factory(
  'currencyService',
  [
    '$http',
    function (
      $http
    ) {
      'use strict';

      var service = {};

      var activeCurrency = null; // set initial to null to load currency from hotel

      // default currencies
      var availableCurrencies = [
        'USD',
        'CAD',
        'MXN',
        'COP'
      ];
      // setting currency from a reservation needs to overload
      // prevent the hotel load from resetting it to hotel's default
      // but only once; so treat this a toggle.
      var fromReservation = false;

      function inArray(needle, haystack) {
        var i = 0;
        var length = haystack.length;
        for (; i < length; i += 1) {
          if (haystack[i] === needle) {
            return true;
          }
        }
        return false;
      }

      service.setAllCurrencies = function (currencies) {
        var currencyArray = Object.keys(currencies);

        if (currencyArray.length) {
          availableCurrencies = [];
          currencyArray.forEach(function (currency) {
            availableCurrencies.push(currency);
          });
        }
      };

      service.getAllCurrencies = function () {
        return availableCurrencies;
      };

      service.getActiveCurrency = function () {
        return activeCurrency;
      };

      service.setActiveCurrency = function (currencyCode, pFromReservation) {
        // set default value to false
        pFromReservation = typeof pFromReservation === 'undefined' ? false : pFromReservation;
        currencyCode = currencyCode.trim();
        if (!inArray(currencyCode, availableCurrencies)) {
          return;
        }
        // if we are trying to set it not from a reservation (From a hotel)
        // and it was previously set from a reservation, toggle it back off
        // and don't adjust currency
        if (fromReservation && !pFromReservation) {
          fromReservation = false;
          return;
        }

        activeCurrency = currencyCode;
        fromReservation = pFromReservation;
      };


      return service;
    }
  ]
);
