/**
 * Created by jeffrey on 3/14/17.
 */
lqMobileModule.factory('androidGeolocationService',
  [
    '$q',
    'satelliteTrackingModelService',
    function (
      $q,
      satelliteTrackingModelService
    ) {
      var deviceType = window.device.platform.toLowerCase();
      var thisService = {};
      var androidDefaultGeolocationOptions = {};
      var androidPermissions;
      var fineLocationPermission;

      if (deviceType === 'android') {
        androidDefaultGeolocationOptions = {
          maximumAge: 3000000,
          timeout: 10000,
          enableHighAccuracy: true
        };
        androidPermissions = cordova.plugins.permissions;
        fineLocationPermission = androidPermissions.ACCESS_FINE_LOCATION;
      }

      function hasLocationPermission() {
        var deferred = $q.defer();
        androidPermissions.hasPermission(
          fineLocationPermission,
          function (response) {
            deferred.resolve(!!response.hasPermission);
          },
          deferred.reject
        );
        return deferred.promise;
      }

      function getSystemLocation(success, failure) {
        navigator.geolocation.getCurrentPosition(
          function (geoposition) {
            success(geoposition);
          },
          function (error) {
            failure(error);
          },
          androidDefaultGeolocationOptions
        );
      }

      function promptForLocationPermission() {
        var defer = $q.defer();
        androidPermissions.requestPermission(
          fineLocationPermission,
          function (response) {
            if (response.hasPermission) {
              satelliteTrackingModelService.locationPermissionGranted();
              defer.resolve(true);
            } else {
              satelliteTrackingModelService.locationPermissionDenied();
              // user declined permissions
              defer.reject(false);
            }
          },
          function () {
            console.log(arguments);
            // error requesting permissions ?
            defer.reject(false);
          }
        );
        return defer.promise;
      }

      thisService.getCurrentPosition = function (success, failure, promptForPermission) {
        hasLocationPermission().then(
          function (hasPermission) {
            if (hasPermission) {
              getSystemLocation(success, failure);
            } else if (promptForPermission) {
              promptForLocationPermission().then(
                function () {
                  // only returns ture
                  getSystemLocation(success, failure);
                },
                failure
              );
            } else {
              // does not have permission, do not prompt
              failure(false);
            }
          },
          function () {
            // error checking location permission ?
            console.log(arguments);
            failure(false);
          }
        );
      };

      return thisService;
    }
  ]
);
