/**
 * Created by jeffrey on 3/14/17.
 */
lqMobileModule.factory(
  'geolocationService',
  [
    '$q',
    'androidGeolocationService',
    'iOsGeolocationService',
    'browserGeolocationService',
    function (
      $q,
      androidGeolocationService,
      iOsGeolocationService,
      browserGeolocationService
    ) {
      var thisService = {};
      var deviceType = window.device.platform.toLowerCase();


      function getOverrideCoords() {
        var latField = document.getElementById('test_geo_lat');
        var lngField = document.getElementById('test_geo_lng');
        var hasLat = latField && latField.value.length && !isNaN(parseFloat(latField.value));
        var hasLng = lngField && lngField.value.length && !isNaN(parseFloat(lngField.value));

        if (hasLat && hasLng) {
          return {
            coords: {
              latitude: parseFloat(latField.value),
              longitude: parseFloat(lngField.value)
            }
          };
        }
        return undefined;
      }


      /**
       * @param {Boolean} promptForPermission -- only used for Android
       * @returns {Promise}
       */
      thisService.getCurrentPositionPromise = function (promptForPermission) {
        var deferred = $q.defer();
        var overrideCoords = getOverrideCoords();

        if (typeof promptForPermission === 'undefined') {
          promptForPermission = false;
        }

        if (overrideCoords) {
          deferred.resolve(overrideCoords);
        } else {
          thisService.getCurrentPosition(
            function (geoposition) {
              deferred.resolve(geoposition);
            },
            function (error) {
              deferred.reject(error);
            },
            promptForPermission
          );
        }

        return deferred.promise;
      };

      /**
      * getLat
      *
      * Get the current latittude for the device.
      *
      * @param Callback success
      * @param Callback failure
      * @parma Boolean promptForPermission -- only used for Android
      */
      thisService.getCurrentPosition = function (success, failure, promptForPermission) {

        if (typeof promptForPermission === 'undefined') {
          promptForPermission = false;
        }

        switch (deviceType) {
          case 'ios':
            iOsGeolocationService.getCurrentPosition(success, failure);
            break;
          case 'android':
            androidGeolocationService.getCurrentPosition(success, failure, !!promptForPermission);
            break;
          default:
            browserGeolocationService.getCurrentPosition(success, failure);
            break;
        }
      };

      return thisService;
    }
  ]
);
