/* global lqMobileModule,FingerprintAuth */
/**
 * Created by jeffrey on 1/13/17.
 */
lqMobileModule.factory('androidTouchIdService',
  [
    '$q',
    'secureStorageService',
    function (
      $q,
      secureStorageService
    ) {
      'use strict';

      var thisService = {};

      thisService.isTouchIdAvailable = function () {
        var deferred = $q.defer();

        if (typeof FingerprintAuth.isAvailable === 'function') {
          FingerprintAuth.isAvailable(
            function (result) {
              if (result.isAvailable && result.hasEnrolledFingerprints) {
                deferred.resolve(true);
              } else {
                deferred.reject(false);
              }
            },
            function () {
              deferred.reject(false);
            }
          );
        } else {
          deferred.reject(false);
        }
        return deferred.promise;
      };

      thisService.promptForFingerAuth = function (username, lastname) {
        var defer = $q.defer();
        secureStorageService.getValue('androidPasswordToken').then(
          function (token) {
            var decryptConfig = {
              clientId: 'lqMobileApp',
              username: 'lqMobileAndroidTouchKey',
              token: token,
              disableBackup: true
            };

            FingerprintAuth.decrypt(decryptConfig, function (decryptResult) {
              if (decryptResult.withFingerprint) {
                defer.resolve({
                  password: decryptResult.password,
                  username: username,
                  lastname: lastname
                });
              } else {
                defer.reject('???');
              }
            }, function (msg) {
              defer.reject(msg);
            });
          },
          function (msg) {
            defer.reject(msg);
          }
        );

        return defer.promise;
      };

      thisService.registerWithFingerAuthentication = function (password) {
        var defer = $q.defer();
        var encryptConfig = {
          clientId: 'lqMobileApp',
          username: 'lqMobileAndroidTouchKey',
          password: password,
          disableBackup: true
        };

        // when re-registering, delete any existing stored creds:
        FingerprintAuth.delete({
          clientId: 'lqMobileApp',
          username: 'lqMobileAndroidTouchKey'
        }, function () {
          FingerprintAuth.encrypt(encryptConfig,
            function (result) {
              // store token
              secureStorageService.setValue('androidPasswordToken', result.token).then(
                function () {
                  defer.resolve(true);
                },
                function (err) {
                  defer.resolve('failed to store token' + err);
                }
              );
            },
            function (err) {
              defer.reject('failed to encrypt: ' + err);
            }
          );
        }, function () {
          // if deletion fails, the entry probably didn't exist, store creds anyway:
          FingerprintAuth.encrypt(encryptConfig,
            function (result) {
              // store token
              secureStorageService.setValue('androidPasswordToken', result.token).then(
                function () {
                  defer.resolve(true);
                },
                function (err) {
                  defer.resolve('failed to store token' + err);
                }
              );
            },
            function (err2) {
              defer.reject('failed to encrypt: ' + err2);
            }
          );
        });

        return defer.promise;
      };

      return thisService;
    }
  ]
);
