/* global lqMobileModule */
/**
 * Created by jeffrey on 1/13/17.
 */
lqMobileModule.factory('iOsTouchIdService',
  [
    '$q',
    'secureStorageService',
    function (
      $q,
      secureStorageService
    ) {
      'use strict';

      var thisService = {};

      thisService.isTouchIdAvailable = function () {
        var deferred = $q.defer();
        if (typeof window.plugins.touchid.isAvailable === 'function') {
          window.plugins.touchid.isAvailable(
            function () {
              deferred.resolve(true); // success handler: TouchID available
            },
            function () {
              deferred.resolve(false); // error handler: no TouchID available
            }
          );
        } else {
          deferred.resolve(false);
        }
        return deferred.promise;
      };

      thisService.promptForFingerAuth = function (iOSPromptText, username, lastname) {
        var deferred = $q.defer();
        window.plugins.touchid.verifyFingerprint(
          iOSPromptText, // this will be shown in the native scanner popup
          function () {
            secureStorageService.getValue('touchIDPassword').then(
              function (password) {
                deferred.resolve({
                  password: password,
                  username: username,
                  lastname: lastname
                });
              },
              function (msg) {
                deferred.reject(msg);
              }
            );
          },
          function (msg) {
            deferred.reject(msg);
          }
        );
        return deferred.promise;
      };

      thisService.registerWithFingerAuthentication = function (password) {
        var defer = $q.defer();
        secureStorageService.setValue('touchIDPassword', password).then(
          function () {
            defer.resolve(true);
          },
          function () {
            defer.reject('error storing creds');
          }
        );
        return defer.promise;
      };

      return thisService;
    }
  ]
);
