/* global lqMobileModule, $, navigator*/
lqMobileModule.factory('cookieStorageModelService', ['$sessionStorage', '$cookies', 'lodash',
  function ($sessionStorage, $cookies, lodash) {
    'use strict';

    var modelService = {
      cookiesEnabled: navigator.cookieEnabled,
      previousSearches: 'previousSearches',
      favoriteHotels1: 'favoriteHotels1',
      favoriteHotels2: 'favoriteHotels2',
      localReservations: 'localReservations'
    };

    function setCookieValue(cookieName, cookieValue) {
      $.cookie(cookieName, JSON.stringify(cookieValue), {
        expires: 365 * 99,
        path: '/'
      });
    }

    function getCookie(cookieName) {
      if (typeof $.cookie(cookieName) !== 'undefined') {
        return JSON.parse($.cookie(cookieName));
      }
      return null;
    }

    modelService.setPreviousSearches = function (searches) {
      setCookieValue(modelService.previousSearches, searches);
    };

    modelService.getPreviousSearches = function () {
      return getCookie(modelService.previousSearches);
    };

    modelService.setFavoriteHotels = function (hotels) {
      var arrays = lodash.chunk(hotels, 5);
      if (arrays.length === 0) {
        $.removeCookie(modelService.favoriteHotels2, { path: '/' });
        $.removeCookie(modelService.favoriteHotels1, { path: '/' });
      } else if (arrays.length === 1) {
        setCookieValue(modelService.favoriteHotels1, arrays[0]);
        $.removeCookie(modelService.favoriteHotels2, { path: '/' });
      } else {
        setCookieValue(modelService.favoriteHotels1, arrays[0]);
        setCookieValue(modelService.favoriteHotels2, arrays[1]);
      }
    };

    modelService.getFavoriteHotels = function () {
      var cookie1 = getCookie(modelService.favoriteHotels1);
      var cookie2 = getCookie(modelService.favoriteHotels2);
      if (cookie1) {
        if (cookie2) {
          return lodash.union(cookie1, cookie2);
        }
        return cookie1;
      }
      return null;
    };

    modelService.setLocalReservations = function (localReservations) {
      setCookieValue(modelService.localReservations, localReservations);
    };

    modelService.getLocalReservations = function () {
      return getCookie(modelService.localReservations);
    };

    return modelService;
  }]);
