/* global lqMobileModule, $, navigator*/
lqMobileModule.factory(

    'persistentStorageModelService',
  [
    '$q',
    'lodash',
    function (
      $q,
      lodash
    ) {
      'use strict';

      var modelService = {
        previousSearches: 'previousSearches',
        favoriteHotels1: 'favoriteHotels1',
        favoriteHotels2: 'favoriteHotels2',
        localReservations: 'localReservations'
      };

      function isJsonString(str) {
        try {
          JSON.parse(str);
        } catch (e) {
          return false;
        }
        return true;
      }

       /**
       * getFile
       *
       * Used for retrieving file
       */
      function getFile() {
        var defer = $q.defer();
        window.requestFileSystem(LocalFileSystem.PERSISTENT, 0, function (fs) {
          fs.root.getFile('lq-mobile.txt', { create: true, exclusive: false }, function (fileEntry) {
            defer.resolve(fileEntry);
          }, function () {
            defer.reject();
            // TODO - error handling create file
          });
        }, function () {
          defer.reject();
          // TODO - error handling load FS
        });
        return defer.promise;
      }

      /**
       * @param writeString
       * @return {Deferred|*|{resolve, reject, promise}|{promise, resolve, reject}}
       */
      function writeToFile(writeString) {
        var defer = $q.defer();
        getFile().then(
          function (fileEntry) {
            fileEntry.createWriter(
              function (fileWriter) {
                fileWriter.seek(0);
                fileWriter.write(new Blob([writeString], { type: 'text/plain' }));
                defer.resolve(true);
              }
            );
          },
          function () {
            console.log('persistentStorageModelService.writeToFile: error writing to file');
            defer.reject('persistentStorageModelService.writeToFile: error writing to file');
          }
        );
        return defer.promise;
      }

      function getFullFileData() {
        var defer = $q.defer();

        getFile().then(
          function (fileEntry) {
            fileEntry.file(
              function (file) {
                var reader = new FileReader();
                reader.onloadend = function () {
                  defer.resolve(this.result);
                };
                reader.readAsText(file);
              },
              function (err) {
                console.log(err);
              }
            );
          },
          function () {
            defer.reject('error getting file');
          }
        );

        return defer.promise;
      }

      function truncateFile() {
        var defer = $q.defer();
        getFullFileData().then(
          function (fileData) {
            getFile().then(
              function (fileEntry) {
                fileEntry.createWriter(
                  function (fileWriter) {
                    fileWriter.seek(0);
                    // clear contents
                    fileWriter.truncate(0);
                    defer.resolve(fileData);
                  }
                );
              },
              function () {
                defer.reject('error getting file');
              }
            );
          },
          function () {
            defer.resolve('error reading file');
          }
        );
        return defer.promise;
      }

      // modelService.clearFile = function () {
      //   var defer = $q.defer();
      //
      //   truncateFile().then(
      //     function (result) {
      //       console.log('persistentStorageModelService.clearFile: done', result);
      //       defer.resolve(result);
      //     },
      //     function (msg) {
      //       console.log('persistentStorageModelService.clearFile: error clearing file', msg);
      //       defer.resolve(msg);
      //     }
      //   );
      //   return defer.promise;
      // };

      /**
      * getValue
      *
      * Returns the value at the given key.
      *
      * @param String key
      * @return {Promise<Mixed>} value
      */
      modelService.getValue = function (key) {
        var defer = $q.defer();

        getFullFileData().then(
          function (result) {
            var obj;
            var response;
            if (typeof result === 'string' && result.length > 0) {
              obj = JSON.parse(result);
            } else {
              obj = {};
            }
            if (obj.hasOwnProperty(key)) {
              if (isJsonString(obj[key])) {
                response = JSON.parse(obj[key]);
              } else {
                response = obj[key];
              }
              defer.resolve(response);
            } else {
              defer.reject('key not found');
            }
          },
          function (msg) {
            defer.reject(msg);
          }
        );

        return defer.promise;
      };

      /**
      * setValue
      *
      * Sets the value for key
      *
      * @param String key
      * @param Mixed value
      *
      * @return {Promise<bool>,fail<string>
      */
      modelService.setValue = function (key, value) {
        var defer = $q.defer();

        truncateFile().then(
          function (result) {
            var obj;
            if (typeof result === 'string' && result.length > 0) {
              obj = JSON.parse(result);
            } else {
              obj = {};
            }
            obj[key] = value;
            writeToFile(JSON.stringify(obj)).then(
              function (writeResult) {
                if (writeResult === true || writeResult === 'true') {
                  defer.resolve(true);
                } else {
                  defer.reject(writeResult);
                }
              }
            );
          },
          function (msg) {
            console.log('persistentStorageModelService.setValue: error truncating file');
            defer.reject(msg);
          }
        );
        return defer.promise;
      };

      /**
      * removeValue
      *
      * Remove the value at the given key.
      *
      * @param Strig key
      */
      modelService.removeValue = function (key) {
        return truncateFile().then(
          function (result) {
            var obj;
            if (typeof result === 'string' && result.length > 0) {
              obj = JSON.parse(result);
            } else {
              obj = {};
            }
            if (obj.hasOwnProperty(key)) {
              delete obj[key];
            }
            writeToFile(JSON.stringify(obj));
          },
          function () {
            console.log('persistentStorageModelService.removeValue: error truncating file');
          }
        );
      };

      //
      // Previous Searches
      modelService.setPreviousSearches = function (searches) {
        modelService.setValue(modelService.previousSearches, JSON.stringify(searches));
      };

      modelService.getPreviousSearches = function () {
        var defer = $q.defer();
        modelService.getValue(modelService.previousSearches).then(
          function (result) {
            defer.resolve(result);
          },
          function () {
            defer.resolve(null);
          }
        );
        return defer.promise;
      };


      //
      // Favorite Hotels
      modelService.setFavoriteHotels = function (hotels) {
        var defer = $q.defer();
        var arrays = lodash.chunk(hotels, 5);
        if (arrays.length === 0) {
          modelService.removeValue(modelService.favoriteHotels2).then(
            function () {
              modelService.removeValue(modelService.favoriteHotels1).then(
                function () {
                  defer.resolve();
                },
                function () {
                  defer.resolve();
                }
              );
            },
            function () {
              modelService.removeValue(modelService.favoriteHotels1).then(
                function () {
                  defer.resolve();
                },
                function () {
                  defer.resolve();
                }
              );
            }
          );
        } else if (arrays.length === 1) {
          modelService.setValue(modelService.favoriteHotels1, JSON.stringify(arrays[0])).then(
            function () {
              modelService.removeValue(modelService.favoriteHotels2).then(
                function () {
                  defer.resolve();
                },
                function () {
                  defer.resolve();
                }
              );
            },
            function () {
              console.log('error storing fav hotels');
              modelService.removeValue(modelService.favoriteHotels2).then(
                function () {
                  defer.resolve();
                },
                function () {
                  defer.resolve();
                }
              );
            }
          );
        } else {
          modelService.setValue(modelService.favoriteHotels1, JSON.stringify(arrays[0])).then(
            function () {
              modelService.setValue(modelService.favoriteHotels2, JSON.stringify(arrays[1])).then(
                function () {
                  defer.resolve();
                },
                function () {
                  defer.resolve();
                }
              );
            },
            function (msg) {
              console.log(JSON.stringify(msg) + '-- Error saving favorite Hotels');
              defer.resolve();
            }
          );
        }
        return defer.promise;
      };

      modelService.getFavoriteHotels = function () {
        var mainDefer = $q.defer();

        modelService.getValue(modelService.favoriteHotels1).then(
          function (list1) {
            modelService.getValue(modelService.favoriteHotels2).then(
              function (list2) {
                mainDefer.resolve(lodash.union(list1, list2));
              },
              function () {
                mainDefer.resolve(list1);
              }
            );
          },
          function () {
            modelService.getValue(modelService.favoriteHotels2).then(
              function (list2) {
                mainDefer.resolve(list2);
              },
              function () {
                mainDefer.resolve([]);
              }
            );
          }
        );

        return mainDefer.promise;
      };


      //
      // Reservations
      modelService.setLocalReservations = function (localReservations) {
        modelService.setValue(modelService.localReservations, JSON.stringify(localReservations));
      };

      modelService.getLocalReservations = function () {
        var defer = $q.defer();
        modelService.getValue(modelService.localReservations).then(
          function (localRes) {
            if (typeof localRes === 'undefined' || localRes === null) {
              defer.resolve([]);
            } else {
              defer.resolve(localRes);
            }
          },
          function () {
            defer.resolve([]);
          }
        );
        return defer.promise;
      };

      return modelService;
    }
  ]
);
