/**
 * Created by jeffrey on 1/25/17.
 */
lqMobileModule.factory(
    'secureStorageService',
  [
    '$q',
    function (
      $q
    ) {
      var thisService = {};
      var SECURE_STORAGE_NAMESPACE = 'lqAppStorage';
      // ios can't store booleans, so here is a list to typecast
      var BOOLEAN_STORAGE_KEYS = ['userUsesTouchId'];

      function internalSetValue(key, value) {
        var ss;
        var defer = $q.defer();

        if (BOOLEAN_STORAGE_KEYS.indexOf(key) > -1) {
          if (value === true) {
            value = 'true';
          } else {
            value = 'false';
          }
        }

        //
        // Place lastname and password in KeyChain
        ss = new cordova.plugins.SecureStorage(
          //
          // Storage unit created.
          function () {
            //
            // Get stored credentials
            ss.set(
              function () {
                defer.resolve(true);
              },
              function (error) {
                defer.reject(error);
              },
              key,
              value
            );
          },
          //
          // Log error if unable to create Store.
          function (error) {
            console.error('SecureStorage Error: ' + error);
            defer.reject(error);
          },
          //
          // Namespace for secure storage.
          SECURE_STORAGE_NAMESPACE
        );

        return defer.promise;
      }

      /**
       * @param key
       *
       * @return {Promise<mixed>,failure<string>}
       */
      thisService.getValue = function (key) {
        var defer = $q.defer();

        var ss = new cordova.plugins.SecureStorage(
          //
          // Storage unit created.
          function () {
            //
            // Get stored credentials
            ss.get(
              function (value) {
                if (BOOLEAN_STORAGE_KEYS.indexOf(key) > -1) {
                  if (value === 'true') {
                    value = true;
                  } else {
                    value = false;
                  }
                }
                defer.resolve(value);
              },
              function (error) {
                defer.reject(error);
              },
              key
            );
          },
          //
          // Log error if unable to create Store.
          function (error) {
            console.error('SecureStorage Error: ' + error);
            defer.reject(error);
          },
          //
          // Namespace for secure storage.
          SECURE_STORAGE_NAMESPACE
        );
        return defer.promise;
      };

      /**
       * @param key
       *
       * @return {Promise<bool>,failure<string>}
       */
      thisService.removeValue = function (key) {
        var defer = $q.defer();
        var ss = new cordova.plugins.SecureStorage(
          //
          // Storage unit created.
          function () {
            //
            // Get stored credentials
            ss.remove(
              function (keyRemoved) {
                defer.resolve(key == keyRemoved);
              },
              function (error) {
                defer.reject(error);
              },
              key
            );
          },
          //
          // Log error if unable to create Store.
          function (error) {
            console.error('SecureStorage Error: ' + error);
            defer.reject(error);
          },
          //
          // Namespace for secure storage.
          SECURE_STORAGE_NAMESPACE
        );
        return defer.promise;
      };

      /**
       * @param key
       * @param value
       *
       * @return {Promise<bool>,failure<string>}
       */
      thisService.setValue = function (key, value) {
        var defer = $q.defer();

        // try to remove the key first
        thisService.removeValue(key).then(
          function () {
            internalSetValue(key, value).then(
              function (result) {
                defer.resolve(result);
              },
              function (error) {
                defer.reject(error);
              }
            );
          },
          function () {
            internalSetValue(key, value).then(
              function (result) {
                defer.resolve(result);
              },
              function (error) {
                defer.reject(error);
              }
            );
          }
        );

        return defer.promise;
      };

      /**
       * @return {Promise}
       */
      thisService.isAvailable = function () {
        var defer = $q.defer();
        var ss;

        ss = new cordova.plugins.SecureStorage(
          function () {
            ss = null;
            defer.resolve();
          },
          function () {
            ss = null;
            defer.reject();

            // notify user
            navigator.notification.alert(
              SECURE_STORAGE_UNAVAILABLE_MESSAGE,
              function () {},
              SECURE_STORAGE_UNAVAILABLE_TITLE
            );
          },
          //
          // Namespace for secure storage.
          SECURE_STORAGE_NAMESPACE);

        return defer.promise;
      };

      return thisService;
    }
  ]
);
