/* global lqMobileModule, $, angular, window*/
lqMobileModule.factory(
  'storageModelService',
  [
    function () {
      'use strict';

      var StorageModel = window.sessionStorage;
      var namespace = 'lqStorage-';

      var modelService = {
        userProfile: 'userProfile',
        searchParam: 'searchParam',
        hotelDetails: 'hotelDetails',
        bookingData: 'bookingData',
        reservations: 'reservations'
      };


      /**
      * getValue
      *
      * Returns the value at the given key.
      *
      * @param String key
      * @return Mixed value
      */
      modelService.getValue = function (key) {
        var value;
        key = namespace + key;
        value = StorageModel.getItem(key);
        if (value != null) {
          return JSON.parse(value);
        }

        return null;
      };

      /**
      * setValue
      *
      * Sets the value for key
      *
      * @param String key
      * @param Mixed value
      */
      modelService.setValue = function (key, value) {
        if (typeof value !== 'undefined') {
          key = namespace + key;
          StorageModel.setItem(key, JSON.stringify(value));
        }
      };

      /**
      * removeValue
      *
      * Remove the value at the given key.
      *
      * @param Strig key
      */
      modelService.clearValue = function (key) {
        key = namespace + key;
        StorageModel.removeItem(key);
      };


      modelService.clearAllSessionData = function () {
        modelService.clearValue(modelService.userProfile);
        modelService.clearValue(modelService.searchParam);
      };

      modelService.setUserProfile = function (userProfile) {
        modelService.setValue(modelService.userProfile, userProfile);
      };

      modelService.getUserProfile = function () {
        return modelService.getValue(modelService.userProfile);
      };

      modelService.clearUserProfileData = function () {
        modelService.clearValue(modelService.userProfile);
      };

      modelService.setSearchParam = function (searchParam) {
        modelService.setValue(modelService.searchParam, searchParam);
      };

      modelService.getSearchParam = function () {
        return modelService.getValue(modelService.searchParam);
      };

      modelService.setHotelDetails = function (hotelDetails) {
        modelService.setValue(modelService.hotelDetails, hotelDetails);
      };

      modelService.getHotelDetails = function () {
        return modelService.getValue(modelService.hotelDetails);
      };

      modelService.setBookingData = function (bookingData) {
        modelService.setValue(modelService.bookingData, bookingData);
      };

      modelService.getBookingData = function () {
        return modelService.getValue(modelService.bookingData);
      };

      modelService.setReservations = function (reservations) {
        modelService.setValue(modelService.reservations, reservations);
      };

      modelService.getReservations = function () {
        return modelService.getValue(modelService.reservations);
      };

      modelService.clearReservations = function () {
        modelService.clearValue(modelService.reservations);
      };

      return modelService;
    }
  ]
);
