/* global LQMobile, $, lqmobileConfigs, LQ*/
LQMobile.Util = {};

LQMobile.Util.clearArray = function (array) {
  if (array === undefined) {
    return [];
  } else {
    while (array.length) {
      array.pop();
    }
    return array;
  }
};

LQMobile.Util.getFirstItemText = function (selector) {
  'use strict';

  return $(selector).first().text();
};

LQMobile.Util.dateToString = function (dateObj, seperator) {
  'use strict';
  var sep = seperator || '-';

  var month = dateObj.getMonth() + 1,
    day = dateObj.getDate(),
    year = dateObj.getFullYear(),
    result = year + sep + month + sep + day;
  return result;
};

LQMobile.Util.changeCssAttr = function (selector, attr, value) {
  'use strict';

  $(selector).css(attr, value);
};

LQMobile.Util.isValidDate = function (val) {
  'use strict';

  if (val === undefined || !(val instanceof Date)) {
    return false;
  }
  return isFinite(val.getTime());
};


LQMobile.Util.sanitizeDate = function (val) {
  'use strict';

  var result,
    month,
    day,
    year,
    partialResult,
    d;
  if (val !== undefined && val !== '') {
    if (Object.prototype.toString.call(val) === '[object Date]') {
      result = val;
    } else if ((val.length === 10) && (/^\d{1,2}\/\d{1,2}\/\d{4}$/.test(val))) {
      month = val.substring(0, 2);
      day = val.substring(3, 5);
      year = val.substring(6, 10);
      partialResult = new Date(year, month - 1, day, 0, 0, 0, 0);
      if (LQMobile.Util.isValidDate(partialResult)) {
        result = partialResult;
      }
    } else {
      d = new Date(val);
      if (LQMobile.Util.isValidDate(d)) {
        result = d;
      } else {
        d = new Date(Date.parse(val));
        if (LQMobile.Util.isValidDate(d)) {
          result = d;
        } else {
          result = null;
        }
      }
    }
  }
  return result;
};

LQMobile.Util.toggleClass = function (target, clss) {
  'use strict';

  $(target).toggleClass(clss);
};

LQMobile.Util.getRateCode = function (rateType) {
  'use strict';

  if (lqmobileConfigs.rateTypeCodes[rateType]) {
    return lqmobileConfigs.rateTypeCodes[rateType];
  }
  return '';
};

LQMobile.Util.getDefaultRates = function () {
  'use strict';

  if (lqmobileConfigs.rateTypeCodes) {
    return lqmobileConfigs.rateTypeCodes;
  }
  return {};
};

LQMobile.Util.getDefaultRate = function () {
  'use strict';

  return 'RAC';
};

LQMobile.Util.getMaxAllowedAdults = function () {
  'use strict';

  return lqmobileConfigs.maxAllowedAdults;
};

LQMobile.Util.getInstantHoldRateCode = function () {
  'use strict';

  return 'LQHOLD';
};

LQMobile.Util.creditCardRegexes = {
  Visa: /^4[0-9]{12}(?:[0-9]{3})?$/,
  MasterCard: /^5[1-5][0-9]{14}$/,
  DinersClub: /^3(?:0[0-5]|[68][0-9])[0-9]{11}$/,
  Amex: /^3[47][0-9]{13}$/,
  Discover: /^6(?:011|5[0-9]{2})[0-9]{12}$/,
  JCB: /^(?:3[0-9]{15}|(2131|1800)[0-9]{11})$/,
  UATP: /^(?:1[0-9]{14,15})$/
};

LQMobile.Util.creditCardNames = {
  VI: 'Visa',
  MC: 'MasterCard',
  DC: 'DinersClub',
  AX: 'Amex',
  DS: 'Discover',
  JCB: 'JCB',
  TP: 'UATP',
  UNKNOWN: 'UNKNOWN'
};

LQMobile.Util.creditCardAbbreviations = {
  Visa: 'VI',
  MasterCard: 'MC',
  DinersClub: 'DC',
  Amex: 'AX',
  Discover: 'DS',
  JCB: 'JCB',
  UATP: 'TP',
  UNKNOWN: 'UNKNOWN'
};

LQMobile.Util.findCCType = function (ccNum) {
  'use strict';

  if (LQMobile.Util.creditCardRegexes.Visa.test(ccNum)) {
    return 'VI';
  }
  if (LQMobile.Util.creditCardRegexes.MasterCard.test(ccNum)) {
    return 'MC';
  }
  if (LQMobile.Util.creditCardRegexes.DinersClub.test(ccNum)) {
    return 'DC';
  }
  if (LQMobile.Util.creditCardRegexes.Amex.test(ccNum)) {
    return 'AX';
  }
  if (LQMobile.Util.creditCardRegexes.Discover.test(ccNum)) {
    return 'DS';
  }
  if (LQMobile.Util.creditCardRegexes.JCB.test(ccNum)) {
    return 'JCB';
  }
  if (LQMobile.Util.creditCardRegexes.UATP.test(ccNum)) {
    return 'TP';
  }
  return 'UNKNOWN';
};
