/* eslint-disable */
/*
The MIT License (MIT)

Copyright (c) 2014 Gregory McGee

Permission is hereby granted, free of charge, to any person obtaining a copy
of this software and associated documentation files (the "Software"), to deal
in the Software without restriction, including without limitation the rights
to use, copy, modify, merge, publish, distribute, sublicense, and/or sell
copies of the Software, and to permit persons to whom the Software is
furnished to do so, subject to the following conditions:

The above copyright notice and this permission notice shall be included in all
copies or substantial portions of the Software.

THE SOFTWARE IS PROVIDED "AS IS", WITHOUT WARRANTY OF ANY KIND, EXPRESS OR
IMPLIED, INCLUDING BUT NOT LIMITED TO THE WARRANTIES OF MERCHANTABILITY,
FITNESS FOR A PARTICULAR PURPOSE AND NONINFRINGEMENT. IN NO EVENT SHALL THE
AUTHORS OR COPYRIGHT HOLDERS BE LIABLE FOR ANY CLAIM, DAMAGES OR OTHER
LIABILITY, WHETHER IN AN ACTION OF CONTRACT, TORT OR OTHERWISE, ARISING FROM,
OUT OF OR IN CONNECTION WITH THE SOFTWARE OR THE USE OR OTHER DEALINGS IN THE
SOFTWARE.
*/

(function() {
	angular.module('gm.datepickerMultiSelect', ['ui.bootstrap'])
	.config(['$provide', function($provide) {
		$provide.decorator('daypickerDirective', ['$delegate', function($delegate) {
			var directive = $delegate[0];

			/* Override compile */
			var link = directive.link;

			directive.compile = function() {
				return function(scope, element, attrs, ctrl) {
					link.apply(this, arguments);

					var selectedDates = [];

					/* Called when multiSelect model is updated */
					scope.$on('update', function(event, newDates) {
						selectedDates = newDates;
						update();
					});

					/* Get dates pushed into multiSelect array before Datepicker is ready */
					scope.$emit('requestSelectedDates');

					/* Fires when date is selected or when month is changed. */
					scope.$watch(function () {
						return ctrl.activeDate.getTime();
					}, update);

					function update() {
						var checkInDay = Math.min.apply(Math, selectedDates),
							checkOutDay = Math.max.apply(Math, selectedDates);
						angular.forEach(scope.rows, function(row) {
							angular.forEach(row, function(day) {
								var ddate = day.date.setHours(0, 0, 0, 0),
									dIndex = selectedDates.indexOf(ddate);
								day.selected = dIndex > -1;
								if (day.selected) {
									if (checkInDay === ddate) day.selectedClass = 'checkInDay';
									else if (checkOutDay === ddate) day.selectedClass = 'checkOutDay';
								}

							});
						});
					}
				}
			}

			return $delegate;
		}]);
	}])
	.directive('multiSelect', function() {
		return {
			require: ['ngModel'],
			link: function(scope, elem, attrs, ctrls) {
				var selectedDates;
				var selectRange;
				var hglghtCheckOut;
				var entries = 0;

				/* Called when directive is compiled */
				scope.$on('requestSelectedDates', function() {
					scope.$broadcast('update', selectedDates);
				});

				scope.$watchCollection(attrs.multiSelect, function(newVal) {
					selectedDates = newVal || [];
					scope.$broadcast('update', selectedDates);
				});

				attrs.$observe('selectRange', function(newVal) {
				  selectRange = !!newVal && newVal !== "false";
				});

				scope.$watch(attrs.hglghtCheckout, function(newVal) {
				  	hglghtCheckOut = !!newVal && newVal !== "false";
				});

				function getNewDates (checkIn, checkOut) {
					var selectedDates = [],
						tempVal = checkIn;
					while(tempVal <= checkOut) {
						selectedDates.push(tempVal);
						// Set a day ahead after pushing to prevent duplicating last date
						tempVal = new Date(tempVal).setHours(24);
					}
					return selectedDates;
				};

				function selectedDateIsInvalid (checkIn, checkOut) {
					var cin = new Date(checkIn),
						cout = new Date(checkOut),
						mind = new Date(),
						maxd = new Date(),
						maxn = new Date(checkIn);

					mind.setDate(mind.getDate()-1); //yesterday

					maxd.setDate(maxd.getDate()+366); //today next year

					maxn.setDate(maxn.getDate()+99); // checkin date + 99 = max nights

					if (compareDates(cin,mind) === 0 || compareDates(cout,maxd) === 0)
						return true;
					if(compareDates(cout, maxn) === 1) {
						scope.$broadcast('addAlertCalendar', {alertType:'danger', alertMessage:'Please select less than 100 nights.'});
						return true;
					}
					return false;
				}

				function compareDates (dateA, dateB) {
					if (dateA.getFullYear() > dateB.getFullYear()) return 1;
					if (dateB.getFullYear() > dateA.getFullYear()) return -1;
					if (dateA.getMonth() > dateB.getMonth()) return 1;
					if (dateB.getMonth() > dateA.getMonth()) return -1;
					if (dateA.getDate() > dateB.getDate()) return 1;
					if (dateB.getDate() > dateA.getDate()) return -1;
					return 0;
				}

				scope.$watch(attrs.ngModel, function(newVal, oldVal) {
					if(!newVal) return;

					var dateVal = newVal.getTime(),
						oldDates = selectedDates,
						oldCheckIn = Math.min.apply(Math, oldDates),
						oldCheckOut = Math.max.apply(Math, oldDates),
						checkInDay = oldCheckIn,
						checkOutDay = oldCheckOut;

					if(selectRange) {
						if (hglghtCheckOut && compareDates(new Date(dateVal), new Date(oldCheckOut)) !== 0) {
						/* if we are selecting the checkout date && it's not the same */
							checkOutDay = dateVal;
							if (compareDates(new Date(dateVal), new Date(oldCheckIn)) !== 1) {
								checkInDay = new Date(dateVal).setHours(-24);
							}
						}

						if (!hglghtCheckOut &&compareDates(new Date(dateVal), new Date(oldCheckIn)) !== 0) {
						/* if we are selecting the checkin date && it's not the same */
							checkInDay = dateVal;
							if (compareDates(new Date(dateVal), new Date(oldCheckOut)) !== -1) {
								checkOutDay = new Date(dateVal).setHours(24);
							}
						}

						if (selectedDateIsInvalid(checkInDay, checkOutDay)) {
							checkInDay = oldCheckIn;
							checkOutDay = oldCheckOut;
						} else {
							scope.$broadcast('clearAlertsCalendar');
						}

						selectedDates = getNewDates(checkInDay, checkOutDay);
						scope.$broadcast('update', selectedDates);
	  					scope.$broadcast('updateCalendarCollection', selectedDates);
	  					scope.$broadcast('hglghtCheckOut');

				  	} else {
	  					if(selectedDates.indexOf(dateVal) < 0) {
	  						selectedDates.push(dateVal);
	  					} else {
	  						selectedDates.splice(selectedDates.indexOf(dateVal), 1);
	  					}
					}
				});
			}
		}
	});
})();
